#include "hal.h"

/*
 * halJPEGD.c : JPEG driver for "JPEG decoder only" IP
 */

extern unsigned int SetRegister(unsigned int addr, unsigned int val);
extern unsigned int GetRegister(unsigned int addr);

#define HAL_SETREG	SetRegister
#define HAL_GETREG	GetRegister

void halJPEGDInit_i(void)
{
	//reset the IP
	HAL_SETREG(HAL_REG_JPEGD_SWReset, 1);

	//determine system endian, ARC700 is LittleEndian
	HAL_SETREG(HAL_REG_JPEGD_SysLittleEndian, 1);

	//masked out any interrupt
	HAL_SETREG(HAL_REG_JPEGD_IntMask, 7);
}

unsigned int halJPEGD_GetIntMask(void)
{
	return HAL_GETREG(HAL_REG_JPEGD_IntMask);
}

static bool_T halJPEGDWaitDecCompSig_ii(void)
{
	unsigned int nRegVal;
	int i=0, j=0;

	while (TRUE)
	{
		//get interrupt status
		nRegVal = HAL_GETREG(HAL_REG_JPEGD_IntRawSts);
		if (i++ > 1000000)
			return TRUE;
		for (j=0; j<1000;j++)
		{
			;
		}
		if (nRegVal == 0) continue;

		//clear interrupt status
		HAL_SETREG(HAL_REG_JPEGD_ClearInt, 7);

		if (nRegVal & 0x1)
			return (TRUE);
		else if (nRegVal & 0x2)
			return (FALSE);
	}
}

static bool_T halJPEGDWaitSOSStopSig_ii(void)
{
	unsigned int nRegVal, i=0;

	while (TRUE)
	{
		//get interrupt status
		nRegVal = HAL_GETREG(HAL_REG_JPEGD_IntRawSts);
		if (i++ > 10000)
			return FALSE;
		if (nRegVal == 0) continue;

		//clear interrupt status
		HAL_SETREG(HAL_REG_JPEGD_ClearInt, 7);

		if (nRegVal & 0x4)
			return (TRUE);
		else if (nRegVal & 0x2)
			return (FALSE);
		else
			return (FALSE);
	}
}

halRawImgFmt_E halJPEGDDecodeP1(
		unsigned int nBSAddr,
		unsigned int *pnWidth,
		unsigned int *pnHeight)
{
	unsigned int nYSamFact;
	unsigned int nCbSamFact;
	unsigned int nCrSamFact;
	halRawImgFmt_E nImgFmt;

	/* soft reset */
	HAL_SETREG(HAL_REG_JPEGD_SWReset, 1);

	/* bitstream address */
	HAL_SETREG(HAL_REG_JPEGD_BSAddr, nBSAddr);

	/* not disable stop after parsing SOS (i.e. stop after parsing SOS) */
	/* let decoder stop after parsing SOS */
	HAL_SETREG(HAL_REG_JPEGD_DisStopAfterSOS, 0);

	/* starts to decode  */
	HAL_SETREG(HAL_REG_JPEGD_Start, 1);

	if (!halJPEGDWaitSOSStopSig_ii())
		return (halcRawImgFmt_Unknown);

	nYSamFact  = HAL_GETREG(HAL_REG_JPEGD_YSamFact);
	nCbSamFact = HAL_GETREG(HAL_REG_JPEGD_CbSamFact);
	nCrSamFact = HAL_GETREG(HAL_REG_JPEGD_CrSamFact);
    
	if (nYSamFact == 0x11 && nCbSamFact == 0x11 && nCrSamFact == 0x11)
		nImgFmt = halcRawImgFmt_YCbCr444;
	else if (nYSamFact == 0x21 && nCbSamFact == 0x11 && nCrSamFact == 0x11)
		nImgFmt = halcRawImgFmt_YCbCr422h;
	else if (nYSamFact == 0x12 && nCbSamFact == 0x11 && nCrSamFact == 0x11)
		nImgFmt = halcRawImgFmt_YCbCr422v;
	else if (nYSamFact == 0x22 && nCbSamFact == 0x11 && nCrSamFact == 0x11)
		nImgFmt = halcRawImgFmt_YCbCr420;
	else if (nYSamFact == 0x11 && nCbSamFact == 0x00 && nCrSamFact == 0x00)
		nImgFmt = halcRawImgFmt_GrayLevel;
	else
		return (halcRawImgFmt_Unknown);

	*pnWidth  = HAL_GETREG(HAL_REG_JPEGD_DecImgWidth);
	*pnHeight = HAL_GETREG(HAL_REG_JPEGD_DecImgHeight);
    
	return (nImgFmt);
}

bool_T halJPEGDDecodeP2(unsigned int nYAddr,
					   unsigned int nCbAddr,
					   unsigned int nCrAddr)
{
	HAL_SETREG(HAL_REG_JPEGD_YAddr,  nYAddr);
	HAL_SETREG(HAL_REG_JPEGD_CbAddr, nCbAddr);
	HAL_SETREG(HAL_REG_JPEGD_CrAddr, nCrAddr);
	
	HAL_SETREG(HAL_REG_JPEGD_DisStopAfterSOS, 1);

	return (halJPEGDWaitDecCompSig_ii());
}
				

//#endif

