#include "sky_8850_dma.h"
#include "sky_8850_i2s.h"

void dma_reset(void)
{
	DMA_W32(DMA_BYTE_SIZE, 0);
	DMA_W32(DMA_CONFIG, 0);	
}

/* ---------------------------------------------------- */

void dma_set_dev(unsigned int srcDev, unsigned int desDev)
{
	DMA_W32(DMA_CONFIG, DMA_R32(DMA_CONFIG) | (srcDev | desDev));	
}

/* ---------------------------------------------------- */

void dma_set_addr(unsigned int srcAddr, unsigned int desAddr)
{
	DMA_W32(DMA_SRC_ADDR, srcAddr);
	DMA_W32(DMA_DEST_ADDR, desAddr);	
}

/* ---------------------------------------------------- */

void dma_set_bytes(unsigned int bytes)
{
	DMA_W32(DMA_BYTE_SIZE, bytes);	
}

/* ---------------------------------------------------- */

void dma_start(unsigned int src_dev, unsigned int des_dev, unsigned int src_addr, unsigned int des_addr, unsigned int size)
{
	unsigned int dma_tr_size;
	
	if(size){
		/* reset DMA */
		dma_reset();
		/* set DMA device */
		dma_set_dev(src_dev, des_dev);
		/* set DMA address */
		dma_set_addr(src_addr, des_addr);

		if(size>DMA_MAX_LEN)
			dma_tr_size = DMA_MAX_LEN;
		else
			dma_tr_size = size;

		/* start DMA */
		dma_set_bytes(dma_tr_size);
	}
}

/* ---------------------------------------------------- */

void dma_sram_to_i2s(unsigned int src_addr, unsigned int size)
{
	 dma_start(DMA_SOURCE_SRAM, DMA_TARGET_I2S_AD, src_addr, (I2S_BASE_ADDR + PCM_WRITE_PORT), size);
}

/* ---------------------------------------------------- */

void dma_sram_to_sram(unsigned int src_addr, unsigned int des_addr, unsigned int size)
{
	 dma_start(DMA_SOURCE_SRAM, DMA_TARGET_SRAM, src_addr, des_addr, size);
}

