/*
 *	ARCmedia Monitor - Monitor the status of the ARCmedia engine
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version
 *	2 of the License, or (at your option) any later version.
 *	
 *	(c) Copyright 2005		ARC International plc
 *
 */
 
#include <linux/module.h>
#include <linux/config.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/proc_fs.h>
#include <linux/mm.h>
#include <linux/miscdevice.h>
#include <linux/smp_lock.h>
#include <linux/init.h>
#include <asm/uaccess.h>
#include <asm/arcmedia.h>

MODULE_AUTHOR("ARC International");
MODULE_DESCRIPTION("ARCmedia monitor");
MODULE_LICENSE("GPL");

/*------------------------------------------------------------------------*/

#ifdef CONFIG_OLD_ARCMEDIA_BCR
static int decode_media_bcr(unsigned long reg, char* dest)
#else
static int decode_simd_bcr(unsigned long reg, char* dest)
#endif
{
	int len = 0;
	if (! dest) return 0;

	/* Klunky kernel code - we love it. */
	len += sprintf(dest+len,"SIMD BCR: 0x%x",reg);
	len += sprintf(dest+len,", Core v%d", MEDIA_VERSION(reg));

	if (DMA_VERSION(reg))
		len += sprintf(dest+len,", DMA v%d", DMA_VERSION(reg));
	else
		len += sprintf(dest+len,", no DMA");

	switch (SDM_SIZE(reg)) {
		case SDM_16K:
			len += sprintf(dest+len,", 16KB SDM");
			break;
		case SDM_32K:
			len += sprintf(dest+len,", 32KB SDM");
			break;
		case SDM_64K:
			len += sprintf(dest+len,", 64KB SDM");
			break;
		default:
			len += sprintf(dest+len,", unknown SDM sz");
			break;
	}
		
	switch (SCM_SIZE(reg)) {
		case SCM_NONE:
			len += sprintf(dest+len,", No SCM");
			break;
		case SCM_5K:
			len += sprintf(dest+len,", 5KB SCM");
			break;
		case SCM_10K:
			len += sprintf(dest+len,", 10KB SCM");
			break;
		case SCM_20K:
			len += sprintf(dest+len,", 20KB SCM");
			break;
		default:
			sprintf(dest+len,", unknown SCM sz");
			break;
	}
		
	switch (SCQ_SIZE(reg)) {
		case SCQ_NONE:
			len += sprintf(dest+len,", No SCQ");
			break;
		case SCQ_320:
			len += sprintf(dest+len,", 320B SCQ");
			break;
		case SCQ_640:
			len += sprintf(dest+len,", 640B SCQ");
			break;
		case SCQ_1280:
			len += sprintf(dest+len,", 1280B SCQ");
			break;
		default:
			len += sprintf(dest+len,", unknown SCQ sz");
			break;
	}
		
	switch (VRF_SIZE(reg)) {
		case VRF_16:
			len += sprintf(dest+len,", 16 VRF");
			break;
		case VRF_24:
			len += sprintf(dest+len,", 24 VRF");
			break;
		case VRF_32:
			len += sprintf(dest+len,", 32 VRF");
			break;
		case VRF_64:
			len += sprintf(dest+len,", 64 VRF");
			break;
		default:
			len += sprintf(dest+len,", unknown VRF sz");
			break;
	}
		
#ifndef CONFIG_OLD_ARCMEDIA_BCR
	len += sprintf(dest+len,", VLC is %s",( VLC_PRESENT(reg) ? "present":"absent"));
#endif

	return len;
}

/*------------------------------------------------------------------------*/
 
#ifndef CONFIG_OLD_ARCMEDIA_BCR

static int decode_vlc_bcr(unsigned long reg, char* dest)
{
	int len = 0;
  int trailing = 0;
	if (! dest) return 0;

	/* Klunky kernel code - we love it. */
	len += sprintf(dest+len,"VLC BCR: 0x%x",reg);
	len += sprintf(dest+len,", ver %d", VLC_VERSION(reg));
  switch(VLC_FIFO_DEPTH(reg))
	{
		case VLC_FIFO_4:
			len += sprintf(dest+len,", 4x32 FIFO");
			break;
		case VLC_FIFO_8:
			len += sprintf(dest+len,", 8x32 FIFO");
			break;
	  case VLC_FIFO_16:
			len += sprintf(dest+len,", 16x32 FIFO");
			break;
	  default:
			len += sprintf(dest+len,", Unknown FIFO sz");
			break;
	}
  
	len += sprintf(dest+len,", Codec support - ");
	if (VLC_LEGACY(reg)) { len += sprintf(dest+len,"MPEG"); trailing = 1; }
	if (VLC_VC1(reg)) { if (trailing) len += sprintf(dest+len,"-VC1"); else { len += sprintf(dest+len,"VC1"); trailing = 1; }}
	if (VLC_H264_BSLN(reg))  { if (trailing) len += sprintf(dest+len,"-H264 Baseline"); else { len += sprintf(dest+len,"H264 Baseline"); trailing = 1; }};
	if (VLC_H264_MAIN(reg))  { if (trailing) len += sprintf(dest+len,"-H264 Main"); else { len += sprintf(dest+len,"H264 Main"); trailing = 1; }};

	len += sprintf(dest+len,", VLC DMA v%d", VLC_DMA_VERSION(reg));

	return len;
}

#endif

/*------------------------------------------------------------------------*/

int ARCmedia_monitor_read_procmem(char *buf, char **start, off_t offset,
				  int length, int *eof, void *data)
{
	int len = 0;
	unsigned long reg;

#ifdef CONFIG_OLD_ARCMEDIA_BCR
	if (reg = read_new_aux_reg(ARCMEDIA_MEDIA_BUILD_AUX_REG))
		len = decode_media_bcr(reg, buf);
	else
		len = sprintf(buf,"Not installed");
#else
	if (reg = read_new_aux_reg(ARCMEDIA_SIMD_DMA_BUILD))
	{
		len = decode_simd_bcr(reg, buf);
		if (VLC_PRESENT(reg))
	  {
			len += sprintf(buf + len,"\n");
			reg = read_new_aux_reg(ARCMEDIA_SIMD_VLC_BUILD);
			len += decode_vlc_bcr(reg, buf + len);
			printk(" %s\n", buf);
		}
	}
	else
		len = sprintf(buf, "Not installed");
#endif
	
	len += sprintf(buf + len, "\n");

	*eof = 1;
	return len;
}

/*------------------------------------------------------------------------*/

static int ARCmedia_monitor_open(struct inode *inode, struct file *file)
{
#ifdef MODULE 
	MOD_INC_USE_COUNT;
#endif
	return 0;
}

/*------------------------------------------------------------------------*/

static int ARCmedia_monitor_release(struct inode *inode, struct file *file)
{
#ifdef MODULE 
	MOD_DEC_USE_COUNT;
#endif
	return 0;
}

/*------------------------------------------------------------------------*/

static int ARCmedia_monitor_ioctl(struct inode *inode, struct file *file,
	unsigned int cmd, unsigned long arg)
{
	unsigned long xpu;
	unsigned long *p = (unsigned long *)arg;

	switch (cmd) {
		case ARCMEDIA_IOC_ENABLE:
#ifdef CONFIG_OLD_ARCMEDIA_BCR
			if (read_new_aux_reg(ARCMEDIA_MEDIA_BUILD_AUX_REG))
#else
			if (read_new_aux_reg(ARCMEDIA_SIMD_DMA_BUILD))
#endif
			{
				xpu = read_new_aux_reg(XPU_AUX_REG);
				xpu |= MEDIA_ENABLE_BIT;
				write_new_aux_reg(XPU_AUX_REG, xpu);
				return 0;
			}
			break;
		case ARCMEDIA_IOC_DISABLE:
#ifdef CONFIG_OLD_ARCMEDIA_BCR
			if (read_new_aux_reg(ARCMEDIA_MEDIA_BUILD_AUX_REG))
#else
			if (read_new_aux_reg(ARCMEDIA_SIMD_DMA_BUILD))
#endif
			{
				xpu = read_new_aux_reg(XPU_AUX_REG);
				xpu &= ~MEDIA_ENABLE_BIT;
				write_new_aux_reg(XPU_AUX_REG, xpu);
				return 0;
			}
			break;

		default:
			break;
	}
	return -ENOTTY;
}


/*------------------------------------------------------------------------*/

static struct file_operations ARCmedia_monitor_fops = {
	owner:		THIS_MODULE,
	ioctl:		ARCmedia_monitor_ioctl,
	open:		ARCmedia_monitor_open,
	release:	ARCmedia_monitor_release
};

/*------------------------------------------------------------------------*/

static struct miscdevice ARCmedia_monitor_miscdev = {
	minor:		ARCMEDIA_MONITOR_MINOR,
	name:		"ARCmedia monitor",
	fops:		&ARCmedia_monitor_fops,
};

/*------------------------------------------------------------------------*/

static char banner[] __initdata = KERN_INFO "ARCmedia engine monitor\n";

/*------------------------------------------------------------------------*/

static int __init ARCmedia_monitor_init(void)
{
	int ret;
	unsigned long reg;
	struct proc_dir_entry *ent;
	char buf[1024];
	unsigned long page;

	ret = misc_register(&ARCmedia_monitor_miscdev);

	if (ret)
		return ret;

	printk(banner);

/* 	if (reg = read_new_aux_reg(0x71)) */
/* 	  printk(" Old media BCR detected\n"); */
/* 	if (reg = read_new_aux_reg(0xfd)) */
/* 	  printk(" New media BCR detected\n"); */

#ifdef CONFIG_OLD_ARCMEDIA_BCR
	reg = read_new_aux_reg(ARCMEDIA_MEDIA_BUILD_AUX_REG);
	decode_media_bcr(reg, buf);
	printk(" %s\n",buf);
#else
	reg = read_new_aux_reg(ARCMEDIA_SIMD_DMA_BUILD);
	decode_simd_bcr(reg, buf);
	printk(" %s\n",buf);
	reg = read_new_aux_reg(ARCMEDIA_SIMD_VLC_BUILD);
	decode_vlc_bcr(reg, buf);
	printk(" %s\n",buf);
#endif

	/* Enable ARCMEDIA extensions in XPU */
#ifdef CONFIG_OLD_ARCMEDIA_BCR
	if (reg = read_new_aux_reg(ARCMEDIA_MEDIA_BUILD_AUX_REG))
#else
	if (reg = read_new_aux_reg(ARCMEDIA_SIMD_VLC_BUILD))
#endif
	{
		printk("Enabling ARCmedia extensions\n");
		reg = read_new_aux_reg(XPU_AUX_REG);
		reg |= MEDIA_ENABLE_BIT;
		write_new_aux_reg(XPU_AUX_REG, reg);
	}
	else
		printk("ARCmedia engine not found, extensions disabled\n");

	if (ent = create_proc_entry(PROCFILENAME, 
				    0,
				    NULL))
	{
		ent->read_proc = ARCmedia_monitor_read_procmem;
	}

	return 0;
}

/*------------------------------------------------------------------------*/

static void __exit ARCmedia_monitor_exit(void)
{
	misc_deregister(&ARCmedia_monitor_miscdev);
	remove_proc_entry(PROCFILENAME, NULL);
}

/*------------------------------------------------------------------------*/

module_init(ARCmedia_monitor_init);
module_exit(ARCmedia_monitor_exit);

/*------------------------------------------------------------------------*/

