/*
 *	bvci_profiler.c
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version
 *	2 of the License, or (at your option) any later version.
 *	
 *	(c) Copyright 2006		ARC International plc
 *
 */
 
#include <linux/module.h>
#include <linux/config.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/proc_fs.h>
#include <linux/mm.h>
#include <linux/vmalloc.h>
#include <linux/mman.h>
#include <linux/miscdevice.h>
#include <linux/smp_lock.h>
#include <linux/init.h>
#include <asm/uaccess.h>
#include <asm/bvci_profiler.h>

MODULE_AUTHOR("ARC International");
MODULE_DESCRIPTION("BVCI profiler interface");
MODULE_LICENSE("GPL");

/*------------------------------------------------------------------------*/

static bvci_profiler *profiler = (bvci_profiler *) PROFILER_BASE_ADDRESS;

/*------------------------------------------------------------------------*/

int dump_bvci_profiler(char* buf, char* padding, bvci_profiler *prof)
{
  int len = 0;

  len += sprintf(buf+len,"   \nBVCI profiler contents:\n%sID        : 0x%08x\n",padding,prof->id);
  len += sprintf(buf+len,"%sctrl      : 0x%08x\n",padding,prof->bpr_ctrl);
  len += sprintf(buf+len,"%ssetup1    : 0x%08x\n",padding,prof->bpr_setup1);
  len += sprintf(buf+len,"%ssetup2    : 0x%08x\n",padding,prof->bpr_setup2);
  len += sprintf(buf+len,"%scyccnt    : 0x%08x\n",padding,prof->bpr_cyccnt);
  len += sprintf(buf+len,"%sindex     : 0x%08x\n",padding,prof->bpr_index);
  len += sprintf(buf+len,"%svacyccnt  : 0x%08x\n",padding,prof->bpr_vacyccnt);
  len += sprintf(buf+len,"%sidlcycnt  : 0x%08x\n",padding,prof->bpr_idlcyccnt);
  len += sprintf(buf+len,"%spktcnt    : 0x%08x\n",padding,prof->bpr_pktcnt);
  len += sprintf(buf+len,"%ssumlat    : 0x%08x\n",padding,prof->bpr_sumlat);
  len += sprintf(buf+len,"%sbprlat    : 0x%08x\n",padding,prof->bpr_lat);
  len += sprintf(buf+len,"%scychi     : 0x%08x\n",padding,prof->bpr_cychi);
  len += sprintf(buf+len,"%svacycht   : 0x%08x\n",padding,prof->bpr_vacychi);
  len += sprintf(buf+len,"%sidlcychi  : 0x%08x\n",padding,prof->bpr_idlcychi);
  len += sprintf(buf+len,"%sbpr_pkthi : 0x%08x\n",padding,prof->bpr_pkthi);
  len += sprintf(buf+len,"%ssumlathi  : 0x%08x\n",padding,prof->bpr_sumlathi);
  len += sprintf(buf+len,"%svacyccap  : 0x%08x\n",padding,prof->bpr_vacyccap);
  len += sprintf(buf+len,"%sidlcycap  : 0x%08x\n",padding,prof->bpr_idlcyccap);
  len += sprintf(buf+len,"%spktcap    : 0x%08x\n",padding,prof->bpr_pktcap);
  len += sprintf(buf+len,"%ssumlatcap : 0x%08x\n",padding,prof->bpr_sumlatcap);
  len += sprintf(buf+len,"%slatcap    : 0x%08x\n",padding,prof->bpr_latcap);

   return len;
}

/*------------------------------------------------------------------------*/

int bvci_profiler_read_procmem(char *buf, char **start, off_t offset,
				  int count, int *eof, void *data)
{
	int len = 0;
	unsigned long reg;

	len = dump_bvci_profiler(buf, " ", profiler);
	
	sprintf(buf,"\n");
	len++;
	*eof = 1;
	return len;
}

/*------------------------------------------------------------------------*/

static int bvci_profiler_open(struct inode *inode, struct file *file)
{
#ifdef MODULE 
	MOD_INC_USE_COUNT;
#endif
	return 0;
}

/*------------------------------------------------------------------------*/

static int bvci_profiler_release(struct inode *inode, struct file *file)
{
#ifdef MODULE 
	MOD_DEC_USE_COUNT;
#endif
	return 0;
}

/*------------------------------------------------------------------------*/

static int bvci_profiler_ioctl(struct inode *inode, struct file *file,
															 unsigned int cmd, unsigned long arg)
{
	unsigned long tmpval;
	bvci_profiler tmpprof;
	int result = -ENOTTY;

	switch (cmd) {
	  case BVCI_GET_CTRL:
			result = copy_to_user((void *) arg, &(profiler->bpr_ctrl), sizeof(unsigned long)) ? -EFAULT : 0;
			break;
	  case BVCI_SET_CTRL:
			if (!(result = copy_from_user(&tmpval, (void *) arg, sizeof(unsigned long)) ? -EFAULT : 0))
			{
				profiler->bpr_ctrl = tmpval;
			}
			break;
	  case BVCI_GET_PROFILER:
			result = copy_to_user((void *) arg, profiler, sizeof(bvci_profiler)) ? -EFAULT : 0;
			break;
	  case BVCI_SET_PROFILER:
			if (!(result = copy_from_user(&tmpprof, (void *) arg, sizeof(bvci_profiler)) ? -EFAULT : 0))
			{
				memcpy(profiler,&tmpprof,sizeof(bvci_profiler));
			}
			break;
		default:
			break;
	}
	return result;
}

/*------------------------------------------------------------------------*/

static int bvci_profiler_mmap(struct file *file, struct vm_area_struct * vma)
{
	unsigned long off = vma->vm_pgoff << PAGE_SHIFT;
	unsigned long physical =  PROFILER_BASE_ADDRESS + off;
	unsigned long vsize = vma->vm_end - vma->vm_start;
	long psize = sizeof(bvci_profiler) - off;

	printk("offset is %d vsize is %d psize is %d\n",off,vsize,psize);

/*  	if ((psize < 0) || (vsize > psize)) */
/*  		return -EINVAL; */

	vma->vm_page_prot = pgprot_noncached(vma->vm_page_prot);
	if (io_remap_page_range(vma->vm_start, physical, vsize, vma->vm_page_prot))		
		return -EAGAIN;

	return 0;
}

/*------------------------------------------------------------------------*/

static struct file_operations bvci_profiler_fops = {
	owner:		THIS_MODULE,
	ioctl:		bvci_profiler_ioctl,
	open:		  bvci_profiler_open,
	release:	bvci_profiler_release,
  mmap:     bvci_profiler_mmap
};

/*------------------------------------------------------------------------*/

static struct miscdevice bvci_profiler_miscdev = {
	minor:	BVCI_PROFILER_MINOR,
	name:		"BVCI profiler",
	fops:		&bvci_profiler_fops,
};

/*------------------------------------------------------------------------*/

static char banner[] __initdata = KERN_INFO "ARC BVCI profiler\n";

/*------------------------------------------------------------------------*/

static int __init bvci_profiler_init(void)
{
	int ret;
	unsigned long reg;
	struct proc_dir_entry *ent;
	char buf[1024];

	ret = misc_register(&bvci_profiler_miscdev);

	if (ret)
		return ret;

	printk(banner);


	if (ent = create_proc_entry(PROCFILENAME, 
				    0,
				    NULL))
	{
		ent->read_proc = bvci_profiler_read_procmem;
	}

	return 0;
}

/*------------------------------------------------------------------------*/

static void __exit bvci_profiler_exit(void)
{
	misc_deregister(&bvci_profiler_miscdev);
	remove_proc_entry(PROCFILENAME, NULL);
}

/*------------------------------------------------------------------------*/

module_init(bvci_profiler_init);
module_exit(bvci_profiler_exit);

/*------------------------------------------------------------------------*/

