/*
 * Very simple but very effective user-space memory tester.
 * Originally by Simon Kirby <sim@stormix.com> <sim@neato.org>
 * Version 2 by Charles Cazabon <memtest@discworld.dyndns.org>
 * Version 3 not publicly released.
 * Version 4 rewrite:
 * Copyright (C) 2005 Charles Cazabon <memtest@discworld.dyndns.org>
 * Licensed under the terms of the GNU General Public License version 2 (only).
 * See the file COPYING for details.
 *
 * This file contains the functions for the actual tests, called from the
 * main routine in memtester.c.  See other comments in that file.
 *
 */
/*
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
*/
#include <linux/kernel.h>

#include "types.h"
#include "sizes.h"

char progress[] = "-\\|/";
#define PROGRESSLEN 4
#define PROGRESSOFTEN 2500

extern unsigned int mem_userdefined_startaddr;
extern unsigned int mem_userdefined_pattern;
extern unsigned int mem_userdefined_size;


/* Function definitions. */

int compare_regions(ulv *bufa, ulv *bufb, size_t count) {
    int r = 0;
    size_t i;
    volatile ulv *p1 = bufa;
    volatile ulv *p2 = bufb;
    volatile unsigned int p1_val, p2_val;

    for (i = 0; i < count; i++, p1++, p2++) {
        p1_val = *p1;
        p2_val = *p2;
        if (p1_val != p2_val) {
            printk("FAILURE: 0x%08lx != 0x%08lx at offset 0x%08lx.\n", 
                (ul) p1_val, (ul) p2_val, (ul) i);
            /* printk("Skipping to next test..."); */
            printk("\nERROR......................halt\n");
            for(;;)
            {
            ;
            }

            r = -1;
        }
    }
    return r;
}

int test_stuck_address(ulv *bufa, size_t count) {
    ulv *p1 = bufa;
    unsigned int j;
    size_t i;

//	printk("           ");
    printk("\n  %-20s: ", " ");
    //fflush(stdout);
    for (j = 0; j < 16; j++) {
//	    printk("\b\b\b\b\b\b\b\b\b\b\b");
        p1 = (ulv *) bufa;
//        printk("setting %3u", j);
        //fflush(stdout);
        printk("=");
        for (i = 0; i < count; i++) {
            *p1 = ((j + i) % 2) == 0 ? (ul) p1 : ~((ul) p1);
            *p1++;
        }
//        printk("\b\b\b\b\b\b\b\b\b\b\b");
//        printk("testing %3u", j);
//        printk(".");
        //fflush(stdout);
        p1 = (ulv *) bufa;
        for (i = 0; i < count; i++, p1++) {
            if (*p1 != (((j + i) % 2) == 0 ? (ul) p1 : ~((ul) p1))) {
                printk(
                    "FAILURE: possible bad address line at offset 0x%08lx.\n", 
                    (ul) i);
                printk("Skipping to next test...\n");
                //fflush(stdout);
                return -1;
            }
        }
    }
//    printk("\b\b\b\b\b\b\b\b\b\b\b           \b\b\b\b\b\b\b\b\b\b\b");
    //fflush(stdout);
    return 0;
}

int test_random_value(ulv *bufa, ulv *bufb, size_t count) {
    ulv *p1 = bufa;
    ulv *p2 = bufb;
    ul j = 0;
    size_t i;

//	putchar(' ');
printk(" ");
    //fflush(stdout);
    for (i = 0; i < count; i++) {
        *p1++ = *p2++ = rand_ul();
		if (!(i % PROGRESSOFTEN)) {
//			putchar('\b');
//			putchar(progress[++j % PROGRESSLEN]);
			printk("\b");
			printk("%c", (progress[++j % PROGRESSLEN]));
            //fflush(stdout);
		}
    }
	printk("\b \b");
    //fflush(stdout);
    return compare_regions(bufa, bufb, count);
}

int test_xor_comparison(ulv *bufa, ulv *bufb, size_t count) {
    ulv *p1 = bufa;
    ulv *p2 = bufb;
    size_t i;
    ul q = rand_ul();

    for (i = 0; i < count; i++) {
        *p1++ ^= q;
        *p2++ ^= q;
    }
    return compare_regions(bufa, bufb, count);
}

int test_sub_comparison(ulv *bufa, ulv *bufb, size_t count) {
    ulv *p1 = bufa;
    ulv *p2 = bufb;
    size_t i;
    ul q = rand_ul();

    for (i = 0; i < count; i++) {
        *p1++ -= q;
        *p2++ -= q;
    }
    return compare_regions(bufa, bufb, count);
}

int test_mul_comparison(ulv *bufa, ulv *bufb, size_t count) {
    ulv *p1 = bufa;
    ulv *p2 = bufb;
    size_t i;
    ul q = rand_ul();

    for (i = 0; i < count; i++) {
        *p1++ *= q;
        *p2++ *= q;
    }
    return compare_regions(bufa, bufb, count);
}

int test_div_comparison(ulv *bufa, ulv *bufb, size_t count) {
    ulv *p1 = bufa;
    ulv *p2 = bufb;
    size_t i;
    ul q = rand_ul();

    for (i = 0; i < count; i++) {
        if (!q) {
            q++;
        }
        *p1++ /= q;
        *p2++ /= q;
    }
    return compare_regions(bufa, bufb, count);
}

int test_or_comparison(ulv *bufa, ulv *bufb, size_t count) {
    ulv *p1 = bufa;
    ulv *p2 = bufb;
    size_t i;
    ul q = rand_ul();

    for (i = 0; i < count; i++) {
        *p1++ |= q;
        *p2++ |= q;
    }
    return compare_regions(bufa, bufb, count);
}

int test_and_comparison(ulv *bufa, ulv *bufb, size_t count) {
    ulv *p1 = bufa;
    ulv *p2 = bufb;
    size_t i;
    ul q = rand_ul();

    for (i = 0; i < count; i++) {
        *p1++ &= q;
        *p2++ &= q;
    }
    return compare_regions(bufa, bufb, count);
}

int test_seqinc_comparison(ulv *bufa, ulv *bufb, size_t count) {
    ulv *p1 = bufa;
    ulv *p2 = bufb;
    size_t i;

    ul q = rand_ul();
    for (i = 0; i < count; i++) {
        *p1++ = *p2++ = (i + q);
    }
    return compare_regions(bufa, bufb, count);
}

int test_solidbits_comparison(ulv *bufa, ulv *bufb, size_t count) {
    ulv *p1 = bufa;
    ulv *p2 = bufb;
    unsigned int j;
    ul q;
    size_t i;

//	printk("           ");
    //fflush(stdout);
    printk("================================================================");
    printk("\n  %-20s: ", " ");

    for (j = 0; j < 64; j++) {
//    for (j = 0; j < 10; j++) {
//	    printk("\b\b\b\b\b\b\b\b\b\b\b");
        printk(".");
        q = (j % 2) == 0 ? UL_ONEBITS : 0;
//        printk("setting %3u", j);
        //fflush(stdout);
        p1 = (ulv *) bufa;
        p2 = (ulv *) bufb;
        for (i = 0; i < count; i++) {
            *p1++ = *p2++ = (i % 2) == 0 ? q : ~q;
        }
//        printk("\b\b\b\b\b\b\b\b\b\b\b");
//        printk("testing %3u", j);
        //fflush(stdout);
        if (compare_regions(bufa, bufb, count)) {
            return -1;
        }
    }
//    printk("\b\b\b\b\b\b\b\b\b\b\b           \b\b\b\b\b\b\b\b\b\b\b");
    //fflush(stdout);
    return 0;
}

int test_checkerboard_comparison(ulv *bufa, ulv *bufb, size_t count) {
    ulv *p1 = bufa;
    ulv *p2 = bufb;
    unsigned int j;
    ul q;
    size_t i;

//	printk("           ");
    //fflush(stdout);
    printk("================================================================");
    printk("\n  %-20s: ", " ");

    for (j = 0; j < 64; j++) {
        printk(".");
//    for (j = 0; j < 10; j++) {
//	    printk("\b\b\b\b\b\b\b\b\b\b\b");
        q = (j % 2) == 0 ? CHECKERBOARD1 : CHECKERBOARD2;
//        printk("setting %3u", j);
        //fflush(stdout);
        p1 = (ulv *) bufa;
        p2 = (ulv *) bufb;
        for (i = 0; i < count; i++) {
            *p1++ = *p2++ = (i % 2) == 0 ? q : ~q;
        }
//        printk("\b\b\b\b\b\b\b\b\b\b\b");
//        printk("testing %3u", j);
        //fflush(stdout);
        if (compare_regions(bufa, bufb, count)) {
            return -1;
        }
    }
//    printk("\b\b\b\b\b\b\b\b\b\b\b           \b\b\b\b\b\b\b\b\b\b\b");
    //fflush(stdout);
    return 0;
}

int test_blockseq_comparison(ulv *bufa, ulv *bufb, size_t count) {
    ulv *p1 = bufa;
    ulv *p2 = bufb;
    unsigned int j;
    size_t i;

//	printk("           ");
    //fflush(stdout);
    printk("================================================================");
    printk("\n  %-20s: ", " ");
    for (j = 0; j < 256; j++) {
//    for (j = 0; j < 256; j+=15) {
//	    printk("\b\b\b\b\b\b\b\b\b\b\b");
        if ( (j%4)== 0)
          printk(".");

        p1 = (ulv *) bufa;
        p2 = (ulv *) bufb;
//        printk("setting %3u", j);
        //fflush(stdout);
////printk("UL_BYTE(j)=0x%x\n", (ul) UL_BYTE(j));
        for (i = 0; i < count; i++) {
            *p1++ = *p2++ = (ul) UL_BYTE(j);
        }
//        printk("\b\b\b\b\b\b\b\b\b\b\b");
//        printk("testing %3u", j);
        //fflush(stdout);
        if (compare_regions(bufa, bufb, count)) {
            return -1;
        }
    }
//    printk("\b\b\b\b\b\b\b\b\b\b\b           \b\b\b\b\b\b\b\b\b\b\b");
    //fflush(stdout);
    return 0;
}

int test_randomseq_comparison(ulv *bufa, ulv *bufb, size_t count) {
    ulv *p1 = bufa;
    ulv *p2 = bufb;
    ulv *psrc, *pdst;
    unsigned int j, full_count = count*2;
    size_t i;
    unsigned int rand_size;	// 0x2000 ~ 0xa8000
    unsigned int rand_srcoffset;	// 0 ~ 0x40000
    unsigned int rand_dstoffset;	// 0 ~ 0x40000
    unsigned int addrcount;
    unsigned long total_test_bytes=0;

    printk("================================================================");
    printk("\n  %-20s: ", " ");
    for (j = 0; j < 64; j++) {
        printk(".");
        p1 = (ulv *) (0x80010000);
        p2 = (ulv *) (bufa);
        for (i = 0; i < full_count; i+=rand_dstoffset, p2 += rand_dstoffset) {
            rand_size = ((rand_ul()%0xa6000)>>4)<<4;
            rand_srcoffset = ((rand_ul()%0x40000)>>4)<<4;
            if ((rand_srcoffset+rand_size) > 0xa6000)
              rand_size = 0xa6000 - rand_srcoffset;

            p1 = (ulv *) (0x80010000 + rand_srcoffset);

            rand_dstoffset = ((rand_ul()%0x80000)>>4)<<4;
//printk("rand_srcoffset = 0x%x rand_dstoffset=0x%x size=0x%x p1=%x p2=%x\n", rand_srcoffset, rand_dstoffset, rand_size, p1, p2);
            if ( (i+rand_size) >= full_count)
            {
//printk("!!!!!!!!!!!!!!!!!!!!!!!\n");
              rand_size = ((full_count - i)>>4)<<4;
            }
            total_test_bytes += rand_size;
//            p2 += rand_dstoffset;
//            memcpy(p2, p1, rand_size);
            psrc = p1;
            pdst = p2;
            for(addrcount=0; addrcount < rand_size; addrcount+=4)
            {
              *pdst++ = *psrc++;
            }

            if (compare_regions(p1, p2, rand_size/4)) {
                return -1;
            }

        }
    }
    printk("\n  %-20s: %lu (%luMBytes).... ", "Total write bytes", total_test_bytes, total_test_bytes/(1024*1024));

    return 0;
}


int test_walkbits0_comparison(ulv *bufa, ulv *bufb, size_t count) {
    ulv *p1 = bufa;
    ulv *p2 = bufb;
    unsigned int j, start_offset;
    size_t i;

//	printk("           ");
    //fflush(stdout);
    printk("================================================================");
    printk("\n  %-20s: ", " ");

    for (j = 0; j < UL_LEN * 2; j++) {
        printk(".");
//	    printk("\b\b\b\b\b\b\b\b\b\b\b");
//        start_offset = (count/4)*(j%4);      
        start_offset = 0;
        p1 = (ulv *) (bufa + start_offset);
        p2 = (ulv *) (bufb + start_offset);
//        printk("setting %3u", j);
        //fflush(stdout);
        for (i = 0; i < count; i++) {
//        for (i = start_offset; i < start_offset+(count/4); i++) {

            if (j < UL_LEN) { /* Walk it up. */
                *p1++ = *p2++ = 0x00000001 << j;
            } else { /* Walk it back down. */
                *p1++ = *p2++ = 0x00000001 << (UL_LEN * 2 - j - 1);
            }
        }

//        printk("\b\b\b\b\b\b\b\b\b\b\b");
//        printk("testing %3u", j);
        //fflush(stdout);
        if (compare_regions(bufa, bufb, count)) {
            return -1;
        }
    }
//    printk("\b\b\b\b\b\b\b\b\b\b\b           \b\b\b\b\b\b\b\b\b\b\b");
    //fflush(stdout);
    return 0;
}

int test_walkbits1_comparison(ulv *bufa, ulv *bufb, size_t count) {
    ulv *p1 = bufa;
    ulv *p2 = bufb;
    unsigned int j, start_offset;
    size_t i;

//	printk("           ");
    //fflush(stdout);
    printk("================================================================");
    printk("\n  %-20s: ", " ");

    for (j = 0; j < UL_LEN * 2; j++) {
        printk(".");
//	    printk("\b\b\b\b\b\b\b\b\b\b\b");
//        start_offset = (count/4)*(j%4);
        start_offset = 0;
        p1 = (ulv *) (bufa+ start_offset);
        p2 = (ulv *) (bufb + start_offset);
//        printk("setting %3u", j);
        //fflush(stdout);
        for (i = 0; i < count; i++) {
//printk("start=0x%x  end=0x%x  p1=0x%x p2=0x%x  origp1=0x%x p2=0x%x\n", start_offset, start_offset+(count/4), p1, p2, bufa, bufb);
//        for (i = start_offset; i < start_offset+(count/4); i++) {

            if (j < UL_LEN) { /* Walk it up. */
                *p1++ = *p2++ = UL_ONEBITS ^ (0x00000001 << j);
            } else { /* Walk it back down. */
                *p1++ = *p2++ = UL_ONEBITS ^ (0x00000001 << (UL_LEN * 2 - j - 1));
            }
        }
//        printk("\b\b\b\b\b\b\b\b\b\b\b");
//        printk("testing %3u", j);
        //fflush(stdout);
        if (compare_regions(bufa, bufb, count)) {
            return -1;
        }
    }
//    printk("\b\b\b\b\b\b\b\b\b\b\b           \b\b\b\b\b\b\b\b\b\b\b");
    //fflush(stdout);
    return 0;
}

int test_bitspread_comparison(ulv *bufa, ulv *bufb, size_t count) {
    ulv *p1 = bufa;
    ulv *p2 = bufb;
    unsigned int j;
    size_t i;

//	printk("           ");
    //fflush(stdout);
    printk("================================================================");
    printk("\n  %-20s: ", " ");

    for (j = 0; j < UL_LEN * 2; j++) {
        printk(".");
//    for (j = 0; j < UL_LEN * 2; j+=7) {
//	    printk("\b\b\b\b\b\b\b\b\b\b\b");
        p1 = (ulv *) bufa;
        p2 = (ulv *) bufb;
//        printk("setting %3u", j);
        //fflush(stdout);
        for (i = 0; i < count; i++) {
            if (j < UL_LEN) { /* Walk it up. */
                *p1++ = *p2++ = (i % 2 == 0)
                    ? (0x00000001 << j) | (0x00000001 << (j + 2))
                    : UL_ONEBITS ^ ((0x00000001 << j)
                                    | (0x00000001 << (j + 2)));
            } else { /* Walk it back down. */
                *p1++ = *p2++ = (i % 2 == 0)
                    ? (0x00000001 << (UL_LEN * 2 - 1 - j)) | (0x00000001 << (UL_LEN * 2 + 1 - j))
                    : UL_ONEBITS ^ (0x00000001 << (UL_LEN * 2 - 1 - j)
                                    | (0x00000001 << (UL_LEN * 2 + 1 - j)));
            }
        }
//        printk("\b\b\b\b\b\b\b\b\b\b\b");
//        printk("testing %3u", j);
        //fflush(stdout);
        if (compare_regions(bufa, bufb, count)) {
            return -1;
        }
    }
//    printk("\b\b\b\b\b\b\b\b\b\b\b           \b\b\b\b\b\b\b\b\b\b\b");
    //fflush(stdout);
    return 0;
}

int test_bitflip_comparison(ulv *bufa, ulv *bufb, size_t count) {
    ulv *p1 = bufa;
    ulv *p2 = bufb;
    unsigned int j, k;
    ul q;
    size_t i;

//	printk("           ");
    printk("================================");
    printk("\n  %-20s: ", " ");

    //fflush(stdout);
    for (k = 0; k < UL_LEN; k++) {
        printk(".");
        q = 0x00000001 << k;
        for (j = 0; j < 8; j++) {
//    	    printk("\b\b\b\b\b\b\b\b\b\b\b");
            q = ~q;
//            printk("setting %3u", k * 8 + j);
            //fflush(stdout);
            p1 = (ulv *) bufa;
            p2 = (ulv *) bufb;
            for (i = 0; i < count; i++) {
                *p1++ = *p2++ = (i % 2) == 0 ? q : ~q;
            }
//            printk("\b\b\b\b\b\b\b\b\b\b\b");
//            printk("testing %3u", k * 8 + j);
            //fflush(stdout);
            if (compare_regions(bufa, bufb, count)) {
                return -1;
            }
        }
    }
//    printk("\b\b\b\b\b\b\b\b\b\b\b           \b\b\b\b\b\b\b\b\b\b\b");
    //fflush(stdout);
    return 0;
}

int test_userdefined_comparison(ulv *bufa, ulv *bufb, size_t count) {
    ulv *p1 = bufa;
    ulv *p2 = bufb;
    size_t i;

    ul q = mem_userdefined_pattern;
    for (i = 0; i < count; i++) {
        *p1++ = *p2++ = (q);
    }
    return compare_regions(bufa, bufb, count);
}


