/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) Oct 01, 2004
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 *	linux/arch/arc/mm/cache_arc700.c
 *
 *	Copyright (C) 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Authors: Amit Bhor, Rahul Trivedi
 * I-Cache and D-cache control functionality. 
 */


#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/bootmem.h>
#include <asm/page.h>
#include <asm/cache.h>
#include <asm/mmu_context.h>
/* Currently the ICACHE & DCACHE are set to 2-way and 4-way set associative respectively */ 
#define ICACHE_WAY_NUM 2
#define DCACHE_WAY_NUM 4

#define FS_FLAG 0x100
#define IM_SET_DC_CTRL 0x40
//#define ARC_CACHE_DEBUG

#if 0
/* cache operations */
void (*_flush_cache_all)(void);
void (*___flush_cache_all)(void);
void (*_flush_cache_mm)(struct mm_struct *mm);
void (*_flush_cache_range)(struct mm_struct *mm, unsigned long start,
								  unsigned long end);
void (*_flush_cache_page)(struct vm_area_struct *vma,
								 unsigned long page);
void (*_flush_icache_all)(void);
void (*_flush_icache_range)(unsigned long start, unsigned long end);
void (*_flush_icache_page)(struct vm_area_struct *vma,
								  struct page *page);
void (*_flush_cache_sigtramp)(unsigned long addr);
void (*_flush_page_to_ram)(struct page *page);
void (*_dma_cache_wback_inv)(unsigned long start, unsigned long size); 
void (*_dma_cache_wback)(unsigned long start, unsigned long size); 
void (*_dma_cache_inv)(unsigned long start, unsigned long size); 

#endif 

int icache_size_value[4] = {8192, 16384, 32768, 65536};
int dcache_size_value[4] = {8192, 16384, 32768, 65536};

unsigned int icache_size, icache_type, icache_line_length;
unsigned int dcache_size, dcache_type, dcache_line_length;

/* Needed to avoid Cache aliasing */
unsigned int dc_has_aliasing;
/* needed to ensure correct number of blocks get invalidated/flushed */
#define DCACHE_LINE_MASK  (~(dcache_line_length - 1))
#define ICACHE_LINE_MASK  (~(icache_line_length - 1))

// New proposed ECO to fix the cache aliasing problem -
// The invalidate/flush registers will now have to be provided with bits 17:13 
// of the virtual adrress in bits 4:0 of the physical address

#define ICACHE_ALIASING_HW_FIX
#ifdef ICACHE_ALIASING_HW_FIX
#define EXTRACT_VIRT_BITS(vaddr)	(((unsigned long )(vaddr) >> 13) & 0x1F)	
#else
#define EXTRACT_VIRT_BITS(vaddr)	0
#endif

inline void a7_probe_cache(void)
{
	unsigned int icache_build, dcache_build,temp;
	int offset;

	/* load the icache build register */
	icache_build = read_new_aux_reg(ARC_REG_I_CACHE_BUILD_REG);
	/* get the icache type */
	icache_type = ((icache_build & CACHE_CONFIG_MASK) >> CACHE_CONFIG_SHIFT);

	/* get the icache size */
	offset = (icache_build & CACHE_SIZE_MASK) >> CACHE_SIZE_SHIFT;
	icache_size = icache_size_value[offset - 4];
	/* check whether icache way size greater than PAGE_SIZE */
	if((icache_size/ICACHE_WAY_NUM) > ARC_shmlba )
		ARC_shmlba = icache_size/ICACHE_WAY_NUM;

	icache_line_length = 32;
	if(icache_type == 0x3) {
		printk("Detected I-cache : \n"); 
		printk("  Type=2 way set-associative, ");
		printk("Line length=%u, Size=%uK", icache_line_length, (icache_size/1024));
#ifdef CONFIG_ARC700_USE_ICACHE
		printk(" (enabled)\n");
#else
		printk(" (disabled)\n");
		/* Disable I-cache*/
		temp = read_new_aux_reg(ARC_REG_IC_CTRL);
		write_new_aux_reg(ARC_REG_IC_CTRL, (temp|0x1));
#endif	
	}	

	/* load the dcache build register */
	dcache_build = read_new_aux_reg(ARC_REG_D_CACHE_BUILD_REG);
	
	/* load the dcache build register and 
	   initialize the dcache control register */
	__asm__ __volatile__( 
		"lr %0, [%1]\n\t"
		"sr 0x40, [%2]\n\t" /* set IM and set replacement policy as round robin */
		:"=r"(dcache_build)
		:"i"(ARC_REG_D_CACHE_BUILD_REG), "i"(ARC_REG_DC_CTRL)
		);
	/* get the dcache type */
	dcache_type = ((dcache_build & CACHE_CONFIG_MASK) >> CACHE_CONFIG_SHIFT);

	/* get the dcache size */
	offset = (dcache_build & CACHE_SIZE_MASK) >> CACHE_SIZE_SHIFT;
	dcache_size = dcache_size_value[offset - 4];
	
	/* check whether dcache way size greater than PAGE_SIZE */
	if((dcache_size/DCACHE_WAY_NUM) > ARC_shmlba ) {
		ARC_shmlba = dcache_size/DCACHE_WAY_NUM;
		dc_has_aliasing = 1;
	}
	dcache_line_length = 32;
	if(dcache_type == 0x2) {
		printk("Detected D-cache : \n"); 
		printk("  Type=4 way set-associative, ");
		printk("Line length=%u, Size=%uK", dcache_line_length, (dcache_size/1024));

#ifdef CONFIG_ARC700_USE_DCACHE
		printk(" (enabled)\n");
#else
		printk(" (disabled)\n");
		/* Set Invalidate mode so that the dirty pages are flushed out */
		temp = read_new_aux_reg(ARC_REG_DC_CTRL);
		write_new_aux_reg(ARC_REG_DC_CTRL, (temp | IM_SET_DC_CTRL));
		/* Flush D cache */
		write_new_aux_reg(ARC_REG_DC_IVDC, 0x1);		
		/* Disable D cache */
		write_new_aux_reg(ARC_REG_DC_CTRL, 0x1);
#endif	

	}					
}


#ifdef CONFIG_ARC700_USE_DCACHE
inline void flush_and_inv_dcache_all(void)
{
	unsigned long dc_ctrl_org =	 read_new_aux_reg(ARC_REG_DC_CTRL);
	unsigned long dc_ctrl = dc_ctrl_org;
	unsigned long flags;
	save_and_cli(flags);
	/*
	 * Set the IM bit in the ctrl reg so that the cache is flushed before
		 * it is invalidated
	 */
	dc_ctrl |= IM_SET_DC_CTRL;
	write_new_aux_reg(ARC_REG_DC_CTRL,dc_ctrl);

	write_new_aux_reg(ARC_REG_DC_IVDC,0x1);
	/* 
	 * wait for the flush to complete, poll on the FS Bit 
	 */
	while (read_new_aux_reg(ARC_REG_DC_CTRL) & FS_FLAG);
	
	write_new_aux_reg(ARC_REG_DC_CTRL,dc_ctrl_org);
	restore_flags(flags);
}

/*
 * start, end - kernel virt addrs
 */

void flush_dcache_range(unsigned long kstart, unsigned long kend)
{
	unsigned long flags;
	unsigned long start =__pa(kstart);
	unsigned long end =__pa(kend);
	
	start &= DCACHE_LINE_MASK; 
	save_flags(flags); cli();

	while (end >= start)	{
		write_new_aux_reg(ARC_REG_DC_FLDL,start|EXTRACT_VIRT_BITS(__va(start)));
		start = start + dcache_line_length;
	}
	

	/* 
	 * wait for the flush to complete, poll on the FS Bit 
	 */
	while (read_new_aux_reg(ARC_REG_DC_CTRL) & FS_FLAG);
		
	restore_flags(flags);
}


void flush_dcache_page(struct page *page)
{
 
#ifdef ARC_CACHE_DEBUG
	printk("%s: va - %08lx pa - %08lx\n",__FUNCTION__,page_address(page),__pa(page_address(page)));
	if(__pa(page_address(page)) < PAGE_OFFSET)
		__asm__ __volatile__("flag 0x1");
#endif 
	flush_dcache_range((unsigned long)page_address(page),(unsigned long)page_address(page)+PAGE_SIZE);
 
}

/*
 * page is the kernel virtual addr and ARC_REG_DC_FLDL expects a phy addr
 */
void flush_dcache_page_virt(unsigned long *page)
{
	  flush_dcache_range((unsigned long)page,(unsigned long)page + PAGE_SIZE);			  
  
}

void flush_dcache_all()
{
	unsigned long value;
	unsigned long flags;
	save_flags(flags); cli();
	write_new_aux_reg(ARC_REG_DC_FLSH, 1);
	
	/* poll till the flush is over */ 
	do {
	  value = read_new_aux_reg(ARC_REG_DC_CTRL);
	} while(value & 0x100); /* Check with the FS bit */
	
	restore_flags(flags);
}

/*
 * Fixme :: Might need to be corrected so that dma works correctly, leaving
 * for now.
 * All the addrs must be physical addrs, that means the addr
 * translation must be done before writing the addr to the cache line.
 */

inline void flush_and_inv_dcache_range(unsigned long start, unsigned long end)
{
		unsigned long flags;
	unsigned long dc_ctrl_org =	 read_new_aux_reg(ARC_REG_DC_CTRL);
	unsigned long dc_ctrl = dc_ctrl_org;

	start &=DCACHE_LINE_MASK;	
		save_flags(flags); cli();
	/*
	 * Set the IM bit in the ctrl reg so that the cache is flushed before
		 * it is invalidated
	 */
	dc_ctrl |= IM_SET_DC_CTRL;
	write_new_aux_reg(ARC_REG_DC_CTRL,dc_ctrl);
	
	while (end >= start)	{
		write_new_aux_reg(ARC_REG_DC_IVDL,start|EXTRACT_VIRT_BITS(start));
		start = start + dcache_line_length;
	}
	
	/* 
	 * wait for the flush to complete, poll on the FS Bit 
	 */
	while (read_new_aux_reg(ARC_REG_DC_CTRL) & FS_FLAG);
		
	write_new_aux_reg(ARC_REG_DC_CTRL,dc_ctrl_org);
	restore_flags(flags);
}

inline void inv_dcache_range(unsigned long start, unsigned long end)
{
	unsigned long flags;

	start &=DCACHE_LINE_MASK; 

	save_flags(flags); cli();

	while (end >= start)	{
		write_new_aux_reg(ARC_REG_DC_IVDL,start|EXTRACT_VIRT_BITS(start));
		start = start + dcache_line_length;
	}

	restore_flags(flags);
}

inline void inv_dcache_all()
{
	unsigned long flags;
	save_and_cli(flags);

	write_new_aux_reg(ARC_REG_DC_IVDC,0x1);
	restore_flags(flags);
}

void flush_page_to_ram(struct page *page)
{
	flush_dcache_page(page);
}

void dma_cache_wback_inv(unsigned long start, unsigned long size)
{
	flush_and_inv_dcache_range(start, start + size);
}

void dma_cache_wback(unsigned long start, unsigned long size)
{
	flush_dcache_range(start, start + size);
}

void dma_cache_inv(unsigned long start, unsigned long size)
{
	inv_dcache_range(start, start + size);
}

#endif /*CONFIG_ARC700_USE_DCACHE */ 



#ifdef CONFIG_ARC700_USE_ICACHE

void flush_icache_all()
{
	unsigned long flags;
	save_and_cli(flags);
	
	write_new_aux_reg(ARC_REG_IC_IVIC, 1);

	/*
	 * lr will not complete till the icache inv operation is not over
	 */
	read_new_aux_reg(ARC_REG_IC_CTRL);
	restore_flags(flags);
}



/*
 * start, end - kernel virt addrs
 */

void flush_icache_range(unsigned long kstart, unsigned long kend)
{
	unsigned long flags;
	unsigned long start =__pa(kstart);
	unsigned long end =__pa(kend);

	start &=ICACHE_LINE_MASK; 
	save_flags(flags); cli();
	
	while (end >= start)	{
		write_new_aux_reg(ARC_REG_IC_IVIL,start|EXTRACT_VIRT_BITS(__va(start)));
		start = start + icache_line_length;
	}

	restore_flags(flags);
}


void flush_icache_page(struct vm_area_struct *vma, struct page *page)
{
	if(!(vma->vm_flags & VM_EXEC))
		return;
	flush_icache_range((unsigned long)page_address(page),(unsigned long)page_address(page)+PAGE_SIZE);
}

#endif	/*CONFIG_ARC700_USE_ICACHE */

#ifdef CONFIG_ARC700_CACHE
void flush_cache_all()
{
	flush_icache_all();
	flush_and_inv_dcache_all();
}

void flush_cache_mm(struct mm_struct *mm)
{
	if(mm->context.asid != NO_ASID)
		flush_cache_all();
}

/*
 * FIXME: start and end user virtual addrs. How to ensure the correct cache
 * is getting invalidate??
 */

void flush_cache_range(struct mm_struct *mm, unsigned long start,
								 unsigned long end)
{
	if(mm->context.asid != NO_ASID)
		flush_cache_all();
}

void flush_cache_page(struct vm_area_struct *vma, unsigned long page)
{
	struct mm_struct *mm = vma->vm_mm;
	pgd_t *pgdp;
	pmd_t *pmdp;
	pte_t *ptep;
	unsigned long paddr;

	if(mm->context.asid == NO_ASID)
		return;

	page  &= PAGE_MASK;
	pgdp = pgd_offset(mm, page);
	pmdp = pmd_offset(pgdp, page);
	ptep = pte_offset(pmdp, page);

	paddr = pte_val(*ptep);
	paddr &= ~(PAGE_SIZE - 1);
	/* If the page isn't marked present then it couldn't possibly
	   be in the cache
	*/
	if(!(pte_val(*ptep) & _PAGE_PRESENT))
		return;

	if((mm == current->active_mm) && (pte_val(*ptep) & _PAGE_VALID)) {
	  
		flush_dcache_page_virt((__va(paddr)));
		if(vma->vm_flags & VM_EXEC)
			flush_icache_page(vma,virt_to_page(__va(paddr)));
	}	
}
#endif

void a7_cache_init(void)
{
	ARC_shmlba = PAGE_SIZE;
	a7_probe_cache();
#if 0
	_flush_cache_all = a7_flush_cache_all;
	___flush_cache_all	= a7_flush_cache_all;
	_flush_cache_mm = a7_flush_cache_mm;
	_flush_cache_range = a7_flush_cache_range;
	_flush_cache_page = a7_flush_cache_page;
	_flush_cache_sigtramp = a7_flush_cache_sigtramp;
	_flush_page_to_ram = a7_flush_page_to_ram;
	_flush_icache_page = a7_flush_icache_page;
	_flush_icache_range = a7_flush_icache_range;
	_dma_cache_wback_inv = a7_dma_cache_wback_inv;
	_dma_cache_wback = a7_dma_cache_wback;
	_dma_cache_inv = a7_dma_cache_inv;
#endif

}

/*
/#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <asm/arcregs.h>

void __flush_icache_all()
{
	printk("[icache] : %s\n" __FUNCTION__);
	write_new_aux_reg (ARC_REG_IC_IVIC, 0x1);
}

void __flush_icache_range(unsigned long start, unsigned long end)
{
	printk("[icache] : %s : start=0x%lx, end=0x%lx\n", __FUNCTION__, start, end);
	write_new_aux_reg (ARC_REG_IC_IVIC, 0x1);
}

void __flush_icache_page( struct vm_area_struct *vma, struct page *page)
{
	printk("[icache] : %s : vaddr=0x%08lx\n", __FUNCTION__, page_address(page));
	write_new_aux_reg (ARC_REG_IC_IVIC, 0x1);
}

void __flush_cache_all()
{
	printk("[cache] : %s\n" __FUNCTION__);
	write_new_aux_reg (ARC_REG_IC_IVIC, 0x1);
}

void __flush_cache_range(struct mm_struct *mm, unsigned long start, unsigned long end)
{
	printk("[cache] : %s : start=0x%lx, end=0x%lx\n", __FUNCTION__, start, end);
	write_new_aux_reg (ARC_REG_IC_IVIC, 0x1);
}

void __flush_cache_page( struct vm_area_struct *vma, struct page *page)
{
	printk("[cache] : %s : vaddr=0x%08lx\n", __FUNCTION__, page_address(page));
	write_new_aux_reg (ARC_REG_IC_IVIC, 0x1);
}

void __flush_cache_mm(struct mm_struct *mm)
{
	printk("[cache] : %s : mm=0x%lx\n", __FUNCTION__, mm);
	write_new_aux_reg (ARC_REG_IC_IVIC, 0x1);
}

void __flush_page_to_ram( struct page *page)
{
	printk("[cache] : %s : vaddr=0x%08lx\n", __FUNCTION__, page_address(page));
	write_new_aux_reg (ARC_REG_IC_IVIC, 0x1);
}
*/



