/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) Oct 01, 2004
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 *  arch/arc/mm/tlbex.S
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Authors: Rahul Trivedi, Amit Bhor
 *
 * TLB Exception Handling Code for A700 
 */

	.cpu A7
	
#include <linux/config.h>
#include <linux/init.h>
#include <linux/linkage.h>
#include <asm/entry.h>		
#include <asm/ptrace.h>		
#include <asm/mmu.h>
#include <asm/traps.h>
#include <asm/arcregs.h>
#include <asm/tlb.h>
#include <asm/page.h>
#include <asm/pgtable.h>	
		
#ifdef CONFIG_ARC_TRACE_UNIT  /* wschen 2008-02-25 */
.global trace_buffer
.section .data 
trace_buffer:
.rept 16
.long 0x00000000,0x00000000,0x00000000
.endr
#endif

.section .text, "ax",@progbits

.macro ARC_ENTRY name
  .globl SYMBOL_NAME(\name) 
  .align 4		
  SYMBOL_NAME_LABEL(\name)
.endm  

#ifdef CONFIG_ARC_TRACE_UNIT
.macro STORE_TRACE_INDEX index
	/* get entry defined by index */
	mov %r0,0x00000000 | (\index << 2) /* source */
	sr %r0,[0xad] 
	lr %r0,[0xae]
	st %r0,[%r1]
	add %r1,%r1,4
	mov %r0,0x00000001 | (\index << 2) /* dest */
	sr %r0,[0xad] 
	lr %r0,[0xae]
	st %r0,[%r1]
	add %r1,%r1,4
	mov %r0,0x00000002 | (\index << 2) /* other */
	sr %r0,[0xad] 
	lr %r0,[0xae]
	st %r0,[%r1]
	add %r1,%r1,4
.endm

.macro STORE_TRACE
	sub %r0,%r0,%r0
	sub %r1,%r1,%r1
	mov %r1,trace_buffer
	STORE_TRACE_INDEX(0)
	STORE_TRACE_INDEX(1)
	STORE_TRACE_INDEX(2)
	STORE_TRACE_INDEX(3)
	STORE_TRACE_INDEX(4)
	STORE_TRACE_INDEX(5)
	STORE_TRACE_INDEX(6)
	STORE_TRACE_INDEX(7)
	STORE_TRACE_INDEX(8)
	STORE_TRACE_INDEX(9)
	STORE_TRACE_INDEX(10)
	STORE_TRACE_INDEX(11)
	STORE_TRACE_INDEX(12)
	STORE_TRACE_INDEX(13)
	STORE_TRACE_INDEX(14)
	STORE_TRACE_INDEX(15)
.endm
#endif

/* This macro does the page-table lookup for the faulting address. 
 * At the end of the macro pte faulted on will be in register pte
 * and the table in which pte belongs will be in ptr.
 */
.macro LOAD_PTE pte, ptr, label 	
	lr	\pte, [efa]          /* faulting address */
	lr	\ptr, [ARC_REG_SCRATCH_DATA0] /* current pgd */
	lsr	\pte, \pte, PGDIR_SHIFT         /* get pgd offset */ 
	asl	\pte, \pte, 2	  /* multiply by 4 as each entry is 4 bytes */
	ld	\ptr, [\ptr, \pte]     /* get ptebase */
	and.f	\ptr, \ptr, PAGE_MASK /* Ignoring protection and other flags */
	bz	\label      /* if the pgd entry is 0 do page fault */
	lr	\pte, [efa]          /* load faulting address once again */
	lsr	\pte, \pte, PAGE_SHIFT        /* get pte offset */
	and	\pte, \pte, 0x7FF      /* page table - 11 bits */
	asl	\pte, \pte, 2        /* multiply by 4 */ 
	ld.a	\pte, [\ptr, \pte]    /* get the pte entry faulted on and 
					the table on which it faulted in ptr */
.endm  	

.macro PTE_PRESENT pte, ptr, label  
	and	\pte, \pte,(_PAGE_PRESENT | _PAGE_READ) 
	xor.f	\pte, \pte, (_PAGE_PRESENT | _PAGE_READ)
	bnz	\label		
	ld	\pte, [\ptr, 0] 
.endm

.macro PTE_K_PRESENT pte, ptr, label  
	and	\pte, \pte,(_PAGE_PRESENT | _PAGE_K_READ) 
	xor.f	\pte, \pte, (_PAGE_PRESENT | _PAGE_K_READ)
	bnz	\label		
	ld	\pte, [\ptr, 0] 
.endm
	
.macro PTE_PRESENTEXEC pte, ptr, label  
	and	\pte, \pte,(_PAGE_PRESENT | _PAGE_READ | _PAGE_EXECUTE) 
	xor.f	\pte, \pte, (_PAGE_PRESENT | _PAGE_READ | _PAGE_EXECUTE) 
	bnz	\label		
	ld	\pte, [\ptr, 0] 
.endm

.macro PTE_K_PRESENTEXEC pte, ptr, label  
	and	\pte, \pte,(_PAGE_PRESENT | _PAGE_K_READ | _PAGE_K_EXECUTE) 
	xor.f	\pte, \pte, (_PAGE_PRESENT | _PAGE_K_READ | _PAGE_K_EXECUTE) 
	bnz	\label		
	ld	\pte, [\ptr, 0] 
.endm
	
.macro PTE_READWRITABLE pte, ptr, label  
	and	\pte, \pte,(_PAGE_PRESENT | _PAGE_READ | _PAGE_WRITE) 
	xor.f	\pte, \pte, (_PAGE_PRESENT | _PAGE_READ | _PAGE_WRITE)
	bnz	\label	
	ld	\pte, [\ptr, 0]
.endm
		
.macro PTE_K_READWRITABLE pte, ptr, label  
	and	\pte, \pte,(_PAGE_PRESENT | _PAGE_K_READ | _PAGE_K_WRITE) 
	xor.f	\pte, \pte, (_PAGE_PRESENT | _PAGE_K_READ | _PAGE_K_WRITE)
	bnz	\label	
	ld	\pte, [\ptr, 0]
.endm
		
.macro PTE_WRITABLE pte, ptr, label  
	and	\pte, \pte,(_PAGE_PRESENT | _PAGE_WRITE) 
	xor.f	\pte, \pte, (_PAGE_PRESENT | _PAGE_WRITE)
	bnz	\label		
	ld	\pte, [\ptr, 0] 
.endm

.macro PTE_K_WRITABLE pte, ptr, label  
	and	\pte, \pte,(_PAGE_PRESENT | _PAGE_K_WRITE) 
	xor.f	\pte, \pte, (_PAGE_PRESENT | _PAGE_K_WRITE)
	bnz	\label		
	ld	\pte, [\ptr, 0] 
.endm
		
.macro PTE_MAKEVALID pte, ptr 
	or	\pte, \pte, (_PAGE_VALID | _PAGE_ACCESSED) 
	st	\pte, [\ptr, 0] 	
.endm

.macro PTE_MAKEWRITE pte, ptr 
	or	\pte, \pte, (_PAGE_VALID | _PAGE_ACCESSED | _PAGE_MODIFIED) 
	st	\pte, [\ptr, 0] 	
.endm		
	
.macro DO_FAULT write
	st	r9, [SYMBOL_NAME(ex_saved_reg1)]
	lr	r9, [erstatus]
	and.f	0, r9, STATUS_U_MASK
	mov	r9, sp	
	; r9 has the user stack pointer or the kernel stack pointer 
	; in case we came here from the kernel ( kernel threads ) 
	
	; using flags set by instruction "and.f	" above.
	bz	1f	; dont switch to kernel stack 
	ld	sp, [SYMBOL_NAME(_current_task)]
	add	sp, sp, ( 0x2000 - 4 )	; 0x2000 = THREAD_SIZE, the size of 
					; task_struct
1:	SAVE_ALL_SYS

#if 1 //wschen 2008-07-16
	mov	r1, \write  
	lr	r2, [efa]  
	mov	r0, sp
#endif

; We dont want exceptions to be disabled while the fault is handled. 
; Now that we have saved the context we return from exception hence re-enable
; exceptions and remain in kernel mode. Interrupts remain disabled.
	ld	r9, [sp, PT_status32]	;read erstatus so we return to the same state of interrupt enables.  
	; set mode we are returning to as kernel mode
	and	r9, r9, ~(STATUS_U_MASK | STATUS_E1_MASK \
				| STATUS_E2_MASK | STATUS_DE_MASK | STATUS_L_MASK)
	sr	r9, [erstatus]
	mov	r9, 2f
	sr	r9, [eret]
	rtie
2:
#if 0 //wschen 2008-07-16
	mov	r1, \write  
	lr	r2, [efa]  
	mov	r0, sp         	
#endif
	jl	do_page_fault  
	j	ret_from_exception 
.endm

.macro TLB_RELOAD pte, ptr 
	ld	\pte, [\ptr, 0]
	and	\ptr, \pte, 0xe00    /* get G,L,V bits from PTE */
	lsr	\ptr, \ptr           /* right shift by 1 */
	and	\pte, \pte, TLBPD1_MASK  /* mask out unused bits */	
	sr	\pte, [ARC_REG_TLBPD1]  
	lr 	\pte,[ARC_REG_TLBPD0]
	or	\ptr,\ptr,\pte 
	sr 	\ptr,[ARC_REG_TLBPD0]
.endm      

#if 1 //wschen 2008-07-16
.macro TLB_WRITE_DATA 
	//
	// Always checks whether instruction will be kicked out by dtlb miss
	//
        lr      r0,[eret]               /* instruction address */
        and     r0,r0,PAGE_MASK         /* VPN of instruction address */
;       lr      r1,[ARC_REG_TLBPD0]     /* Data VPN+ASID - already in r1 from TLB_RELOAD*/ 
        and     r1,r1,0xff              /* Data ASID */
        or      r0,r0,r1                /* Instruction address + Data ASID */
        
        lr      r1,[ARC_REG_TLBPD0]     /* save TLBPD0 containing data TLB*/
        sr      r0,[ARC_REG_TLBPD0]     /* write instruction address to TLBPD0 */
        sr	TLBProbe, [ARC_REG_TLBCOMMAND] /* Look for instruction */
        lr      r0,[ARC_REG_TLBINDEX]   /* r0 = index where instruction is, if at all */
        sr      r1,[ARC_REG_TLBPD0]     /* restore TLBPD0 */

        sr	TLBGetIndex, [ARC_REG_TLBCOMMAND] 
        lr      r1,[ARC_REG_TLBINDEX]   /* r1 = index where MMU wants to put data */
        brne    r0,r1,88f               /* if no match on indices, go around */
        xor     r1,r1,1                 /* flip bottom bit of data index */
        sr      r1,[ARC_REG_TLBINDEX]   /* and put it back */
88:
	/* Here i am not checking if the index returned is error */
	/* I assume that i will always get an index to the store the TLB 
	   entry */
	/* i.e. there will be atleast one way that is unlocked */
	sr	TLBWrite, [ARC_REG_TLBCOMMAND] 
.endm
#endif

#if 0 //wschen 2008-07-16
.macro TLB_WRITE
#else
.macro TLB_WRITE_INSTR
#endif

#if 0	//Barry 2008-07-16  //wschen 2008-07-18 
; BUG catching hack
	sr	TLBProbe, [ARC_REG_TLBCOMMAND]
	lr	r0, [ARC_REG_TLBINDEX]
	cmp	r0, 0x80000000
	beq	1f
	j	do_hardware_bug
1:
#endif
	sr	TLBGetIndex, [ARC_REG_TLBCOMMAND] 
	/* Here i am not checking if the index returned is error */
	/* I assume that i will always get an index to the store the TLB entry */
	/* i.e. there will be atleast one way that is unlocked */
	sr	TLBWrite, [ARC_REG_TLBCOMMAND] 
.endm

.macro SAVE_TEMP
	st	r0, [ex_saved_reg1] 
	st	r1, [ex_saved_reg2]
#ifdef	CONFIG_ARC_TLB_DBG
	lr	r0, [efa]
	cmp	r0, 0
	bnz	1f
	flag	0x1	;halt
	nop
1:
#endif	/* CONFIG_ARC_TLB_DBG */ 
.endm
	
.macro RESTORE_TEMP
	ld	r0, [ex_saved_reg1]  
	ld	r1, [ex_saved_reg2] 
.endm
		
ARC_ENTRY EV_TLBMissI

	SAVE_TEMP
#ifdef CONFIG_ARC_TRACE_UNIT
	STORE_TRACE
#endif
	lr	r0, [efa]
	brhs	r0, VMALLOC_START, kernel_inst_miss
#ifdef CONFIG_ARC_TLB_DBG
; BUG catching hack
;	cmp	r0, 0x40
;	bne	1f
;	flag 0x1
;	nop
;1:
; end hack
#endif
	LOAD_PTE r0, r1, no_page_itlb
	PTE_PRESENTEXEC r0, r1, no_page_itlb
reload_tlb_inst_miss:		
	PTE_MAKEVALID r0, r1
	TLB_RELOAD r0, r1
#if 0 //wschen 2008-07-16
	TLB_WRITE
#else
	TLB_WRITE_INSTR
#endif
	RESTORE_TEMP	
	rtie
kernel_inst_miss:	
	LOAD_PTE r0, r1, no_page_itlb
	PTE_K_PRESENTEXEC r0, r1, no_page_itlb
	b	reload_tlb_inst_miss
no_page_itlb: 
	RESTORE_TEMP	
	DO_FAULT 0
	
SYMBOL_NAME_LABEL(data_tlb_miss_load)

	lr	r0, [efa]
	brhs	r0, VMALLOC_START, kernel_data_miss_load	
	LOAD_PTE r0, r1, no_page_dtlb_load
	PTE_PRESENT r0, r1, no_page_dtlb_load
reload_tlb_data_miss_load:	
	PTE_MAKEVALID r0, r1
	TLB_RELOAD r0, r1
#if 0 //wschen 2008-07-16
	TLB_WRITE
#else
	TLB_WRITE_DATA
#endif
	RESTORE_TEMP	
	rtie
kernel_data_miss_load:
	LOAD_PTE r0, r1, no_page_dtlb_load	
	PTE_K_PRESENT r0, r1, no_page_dtlb_load
	b	reload_tlb_data_miss_load
no_page_dtlb_load: 
	RESTORE_TEMP	
	DO_FAULT 0

SYMBOL_NAME_LABEL(data_tlb_miss_store)	

	lr	r0, [efa]
	brhs	r0, VMALLOC_START, kernel_data_miss_store
	LOAD_PTE r0, r1, no_page_dtlb_store
	PTE_WRITABLE r0, r1, no_page_dtlb_store
reload_tlb_data_miss_store:	
	PTE_MAKEWRITE r0, r1
	TLB_RELOAD r0, r1
#if 0 //wschen 2008-07-16
	TLB_WRITE
#else
	TLB_WRITE_DATA
#endif
	RESTORE_TEMP	
	rtie
kernel_data_miss_store:
	LOAD_PTE r0, r1, no_page_dtlb_store		
	PTE_K_WRITABLE r0, r1, no_page_dtlb_store
	b	reload_tlb_data_miss_store	
no_page_dtlb_store: 
	RESTORE_TEMP	
	DO_FAULT 1

SYMBOL_NAME_LABEL(data_tlb_miss_xchg)	

	lr	r0, [efa]
	brhs	r0, VMALLOC_START, kernel_data_miss_xchg
	LOAD_PTE r0, r1, no_page_dtlb_xchg
	PTE_READWRITABLE r0, r1, no_page_dtlb_xchg
reload_tlb_data_miss_xchg:		
	PTE_MAKEWRITE r0, r1
	TLB_RELOAD r0, r1
#if 0 //wschen 2008-07-16
	TLB_WRITE
#else
	TLB_WRITE_DATA
#endif
	RESTORE_TEMP	
	rtie
kernel_data_miss_xchg:
	LOAD_PTE r0, r1, no_page_dtlb_xchg	
	PTE_K_READWRITABLE r0, r1, no_page_dtlb_xchg
	b	reload_tlb_data_miss_xchg	
no_page_dtlb_xchg: 
	RESTORE_TEMP	
	DO_FAULT 1 
			
ARC_ENTRY EV_TLBMissD

	SAVE_TEMP
#ifdef CONFIG_ARC_TRACE_UNIT
	STORE_TRACE
#endif
	lr	r0, [ecr]
	and	r0, r0, ECAUSE_CODE_MASK
	brne	r0, DATA_TLB_LOAD, not_data_load
	j	data_tlb_miss_load
not_data_load:		
	brne	r0, DATA_TLB_STORE, not_data_store
	j	data_tlb_miss_store
not_data_store:
	j	data_tlb_miss_xchg



/* To handle a duplicate TLB entry bug that is seen on the ARC700 hardware */	
SYMBOL_NAME_LABEL(do_hardware_bug)
	st	r9, [SYMBOL_NAME(ex_saved_reg1)]
	lr	r9, [erstatus]
	and.f	0, r9, STATUS_U_MASK
	mov	r9, sp	
	; r9 has the user stack pointer or the kernel stack pointer 
	; in case we came here from the kernel ( kernel threads ) 
	
	; using flags set by instruction "and.f	" above.
	bz	1f	; dont switch to kernel stack 
	ld	sp, [SYMBOL_NAME(_current_task)]
	add	sp, sp, ( 0x2000 - 4 )	; 0x2000 = THREAD_SIZE, the size of 
					; task_struct
1:	SAVE_ALL_SYS
; We dont want exceptions to be disabled while the fault is handled. 
; Now that we have saved the context we return from exception hence re-enable
; exceptions and remain in kernel mode. Interrupts remain disabled.
	ld	r9, [sp, PT_status32]	;read erstatus so we return to the same state of interrupt enables.  
	; set mode we are returning to as kernel mode
	and	r9, r9, ~(STATUS_U_MASK | STATUS_E1_MASK \
				| STATUS_E2_MASK | STATUS_DE_MASK | STATUS_L_MASK)
	sr	r9, [erstatus]
	mov	r9, 2f
	sr	r9, [eret]
	rtie
2:
	lr	r0, [ecr]  
	lr	r1, [efa]  
	mov	r2, sp         	
	jl	duplicate_tlb_fault
	j	ret_from_exception	


