/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) Oct 01, 2004
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/* arch/arc/proc/arc700/signal.c */

#include <linux/sys.h> /* for NR_syscalls */
#include <linux/config.h>
#include <linux/sched.h>
#include <linux/mm.h>
#include <linux/smp.h>
#include <linux/smp_lock.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/signal.h>
#include <linux/wait.h>
#include <linux/ptrace.h>
#include <linux/unistd.h>
#include <linux/personality.h>
#include <asm/ucontext.h>
#include <asm/uaccess.h>
/* for changing permissions of stackpage*/
#include <asm/pgtable.h>
#include <asm/arcregs.h> 
#include <asm/tlb.h>

#define _BLOCKABLE (~(sigmask(SIGKILL) | sigmask(SIGSTOP)))
int do_signal(struct pt_regs *, sigset_t *);
 
#if 1 //wschen 2008-07-18
#define EXEC_DISABLE 0
#define EXEC_ENABLE 1
void mod_tlb_permission(unsigned long frame_vaddr,
               unsigned int asid, int exec_or_not);
#endif

int copy_siginfo_to_user(siginfo_t *to, siginfo_t *from)
{
	int err = -EFAULT;;

	if (!access_ok (VERIFY_WRITE, to, sizeof(siginfo_t)))
		goto out;

	if (from->si_code < 0)
		return __copy_to_user(to, from, sizeof(siginfo_t));

	/* If you change siginfo_t structure, please be sure
	   this code is fixed accordingly.
	   It should never copy any pad contained in the structure
	   to avoid security leaks, but must copy the generic
	   3 ints plus the relevant union member.  */
	err = __put_user(from->si_signo, &to->si_signo);
	err |= __put_user(from->si_errno, &to->si_errno);
	err |= __put_user((short)from->si_code, &to->si_code);
	/* First 32bits of unions are always present.  */
	err |= __put_user(from->si_pid, &to->si_pid);
	switch (from->si_code >> 16) {
	case __SI_FAULT >> 16:
		break;
	case __SI_CHLD >> 16:
		err |= __put_user(from->si_utime, &to->si_utime);
		err |= __put_user(from->si_stime, &to->si_stime);
		err |= __put_user(from->si_status, &to->si_status);
	default:
		err |= __put_user(from->si_uid, &to->si_uid);
		break;
	/* case __SI_RT: This is not generated by the kernel as of now.  */
	}
out:
	return err;
}

/*
 * atomically swap in the new signal mask, and wait for a signal.
 */
asmlinkage int sys_sigsuspend(int restart, unsigned long oldmask, old_sigset_t mask, struct pt_regs *regs)
{
	sigset_t saveset;

	mask &= _BLOCKABLE;
	spin_lock_irq(&current->sigmask_lock);
	saveset = current->blocked;
	siginitset(&current->blocked, mask);
	recalc_sigpending(current);
	spin_unlock_irq(&current->sigmask_lock);
	regs->r0 = -EINTR;

	while (1) {
		current->state = TASK_INTERRUPTIBLE;
		schedule();
		if (do_signal(regs, &saveset ))
			return regs->r0;
	}
}

asmlinkage int
sys_rt_sigsuspend(sigset_t *unewset, size_t sigsetsize, struct pt_regs *regs)
{
	sigset_t saveset, newset;

	/* XXX: Don't preclude handling different sized sigset_t's. */
	if (sigsetsize != sizeof(sigset_t))
		return -EINVAL;

	if (copy_from_user(&newset, unewset, sizeof(newset)))
		return -EFAULT;
	sigdelsetmask(&newset, ~_BLOCKABLE);

	spin_lock_irq(&current->sigmask_lock);
	saveset = current->blocked;
	current->blocked = newset;
	recalc_sigpending(current);
	spin_unlock_irq(&current->sigmask_lock);
	regs->r0 = -EINTR;

	while (1) {
		current->state = TASK_INTERRUPTIBLE;
		schedule();
		if (do_signal(regs,&saveset))
			return regs->r0;
	}
}

asmlinkage int 
sys_sigaction(int sig, const struct old_sigaction *act,
	      struct old_sigaction *oact)
{
	struct k_sigaction new_ka, old_ka;
	int ret;

	if (act) {
		old_sigset_t mask;
		if (verify_area(VERIFY_READ, act, sizeof(*act)) ||
		    __get_user(new_ka.sa.sa_handler, &act->sa_handler) ||
		    __get_user(new_ka.sa.sa_restorer, &act->sa_restorer))
			return -EFAULT;
		__get_user(new_ka.sa.sa_flags, &act->sa_flags);
		__get_user(mask, &act->sa_mask);
		siginitset(&new_ka.sa.sa_mask, mask);
	}

	ret = do_sigaction(sig, act ? &new_ka : NULL, oact ? &old_ka : NULL);

	if (!ret && oact) {
		if (verify_area(VERIFY_WRITE, oact, sizeof(*oact)) ||
		    __put_user(old_ka.sa.sa_handler, &oact->sa_handler) ||
		    __put_user(old_ka.sa.sa_restorer, &oact->sa_restorer))
			return -EFAULT;
		__put_user(old_ka.sa.sa_flags, &oact->sa_flags);
		__put_user(old_ka.sa.sa_mask.sig[0], &oact->sa_mask);
	}

	return ret;
}

asmlinkage int
sys_sigaltstack(const stack_t *uss, stack_t *uoss,struct pt_regs *regs)
{
	return do_sigaltstack(uss, uoss, regs->sp);
}
		

/*
 * Do a signal return; undo the signal stack.
 */
#if 0 //wschen 2008-07-18
struct sigframe
{
	struct sigcontext sc;
	unsigned long extramask[_NSIG_WORDS-1];
	unsigned long retcode[5];
};
#else
struct sigframe {
	struct ucontext uc;
	unsigned long retcode[5];
};
#endif

#if 0 //wschen 2008-07-18
struct rt_sigframe
{
	struct siginfo *pinfo;
	void *puc;
	struct siginfo info;
	struct ucontext uc;
	unsigned long retcode[5];
};
#else
struct rt_sigframe {
	struct siginfo info;
	struct sigframe sig;
};
#endif

static int
restore_sigcontext(struct pt_regs *regs, struct sigcontext *sc)
{
	int err = 0;

	__get_user(regs->r0, &sc->r0);
	__get_user(regs->r1, &sc->r1);
	__get_user(regs->r2, &sc->r2);
	__get_user(regs->r3, &sc->r3);
	__get_user(regs->r4, &sc->r4);
	__get_user(regs->r5, &sc->r5);
	__get_user(regs->r6, &sc->r6);
	__get_user(regs->r7, &sc->r7);
	__get_user(regs->r8, &sc->r8);
	__get_user(regs->r9, &sc->r9);
	__get_user(regs->r10, &sc->r10);
	__get_user(regs->r11, &sc->r11);
	__get_user(regs->r12, &sc->r12);
	__get_user(regs->r26, &sc->r26);
	__get_user(regs->fp, &sc->fp);
	__get_user(regs->blink, &sc->blink);
	__get_user(regs->ret, &sc->ret);
	__get_user(regs->status32, &sc->status32);
	__get_user(regs->lp_count, &sc->lp_count);
	__get_user(regs->lp_end, &sc->lp_end);
	__get_user(regs->lp_start, &sc->lp_start);
	__get_user(regs->bta, &sc->bta);
	__get_user(regs->sp, &sc->sp);

	return err;
}

#if 1 //wschen 2008-07-18
static int restore_sigframe(struct pt_regs *regs, struct sigframe __user * sf)
{
	sigset_t set;
	int err;
	unsigned long flags;

	err = __copy_from_user(&set, &sf->uc.uc_sigmask, sizeof(set));
	if (err == 0) {
		sigdelsetmask(&set, ~_BLOCKABLE);
		local_irq_save(flags);
		current->blocked = set;
		recalc_sigpending(current);
		local_irq_restore(flags);
	}

	__get_user(regs->r0, &sf->uc.uc_mcontext.r0);
	__get_user(regs->r1, &sf->uc.uc_mcontext.r1);
	__get_user(regs->r2, &sf->uc.uc_mcontext.r2);
	__get_user(regs->r3, &sf->uc.uc_mcontext.r3);
	__get_user(regs->r4, &sf->uc.uc_mcontext.r4);
	__get_user(regs->r5, &sf->uc.uc_mcontext.r5);
	__get_user(regs->r6, &sf->uc.uc_mcontext.r6);
	__get_user(regs->r7, &sf->uc.uc_mcontext.r7);
	__get_user(regs->r8, &sf->uc.uc_mcontext.r8);
	__get_user(regs->r9, &sf->uc.uc_mcontext.r9);
	__get_user(regs->r10, &sf->uc.uc_mcontext.r10);
	__get_user(regs->r11, &sf->uc.uc_mcontext.r11);
	__get_user(regs->r12, &sf->uc.uc_mcontext.r12);
	__get_user(regs->r26, &sf->uc.uc_mcontext.r26);
	__get_user(regs->fp, &sf->uc.uc_mcontext.fp);
	__get_user(regs->blink, &sf->uc.uc_mcontext.blink);
	__get_user(regs->ret, &sf->uc.uc_mcontext.ret);
	__get_user(regs->status32, &sf->uc.uc_mcontext.status32);
	__get_user(regs->lp_count, &sf->uc.uc_mcontext.lp_count);
	__get_user(regs->lp_end, &sf->uc.uc_mcontext.lp_end);
	__get_user(regs->lp_start, &sf->uc.uc_mcontext.lp_start);
	__get_user(regs->bta, &sf->uc.uc_mcontext.bta);
	__get_user(regs->sp, &sf->uc.uc_mcontext.sp);

	return err;
}
#endif

asmlinkage int sys_sigreturn(struct pt_regs *regs)
{
	struct sigframe *frame;
	unsigned long frame_vaddr,frame_tlbpd1;
	int flag=0;
	pte_t *ptep, pte;
	sigset_t set;

	/*
	 * Since we stacked the signal on a word boundary,
	 * then 'sp' should be word aligned here.  If it's
	 * not, then the user is trying to mess with us.
	 */
	if (regs->sp & 3)
		goto badframe;

	 
	frame = ((struct sigframe *)regs->sp) ;

	if (verify_area(VERIFY_READ, frame, sizeof (*frame)))
		goto badframe;
#if 0 //wschen 2008-07-18


	if (__get_user(set.sig[0], &frame->sc.oldmask)
	    || (_NSIG_WORDS > 1
	        && __copy_from_user(&set.sig[1], &frame->extramask,
				    sizeof(frame->extramask))))
		goto badframe;

	sigdelsetmask(&set, ~_BLOCKABLE);
	spin_lock_irq(&current->sigmask_lock);
	current->blocked = set;
	recalc_sigpending(current);
	spin_unlock_irq(&current->sigmask_lock);

	if (restore_sigcontext(regs, &frame->sc))
		goto badframe;

	/* Check if the retcode is on two different page frames */
	if((regs->sp & PAGE_MASK) != ((regs->sp - sizeof(frame->retcode)) & PAGE_MASK))
		flag = 1;

	/*
	 * disabling the execute permissions of the stack enabled in setup_frame
	 * In the pte entry and in the TLB entry.
	 */
	frame_vaddr = (unsigned long)frame;
	ptep = pte_offset(pmd_offset(pgd_offset(current->mm,frame_vaddr),frame_vaddr),frame_vaddr);
	pte = *ptep;
	pte = pte_exprotect(pte);
	set_pte(ptep,pte);

	frame_vaddr = frame_vaddr & PAGE_MASK;
	/* get the ASID */
	frame_vaddr = frame_vaddr | (current->mm->context.asid & 0xff); 
	write_new_aux_reg(ARC_REG_TLBPD0,frame_vaddr);
	write_new_aux_reg(ARC_REG_TLBCOMMAND,TLBProbe);
	
	if(read_new_aux_reg(ARC_REG_TLBINDEX)!=0x80000000)
	{
		write_new_aux_reg(ARC_REG_TLBCOMMAND,TLBRead);
		frame_tlbpd1 = read_new_aux_reg(ARC_REG_TLBPD1);
		/* disable execute permissions for the user stack */
		frame_tlbpd1  = frame_tlbpd1 & ~0x00000008; 
	
		write_new_aux_reg(ARC_REG_TLBPD1,frame_tlbpd1);
		write_new_aux_reg(ARC_REG_TLBCOMMAND,TLBWrite);
	}
#else
	if (restore_sigframe(regs, frame))
		goto badframe;

	/* Check if the retcode is on two different page frames */
	if ((regs->sp & PAGE_MASK) !=
	    ((regs->sp - sizeof(frame->retcode)) & PAGE_MASK))
		flag = 1;

	/*
	 * disabling the execute permissions of the stack enabled in 
	 * setup_frame
	 * In the pte entry and in the TLB entry.
	 */
	frame_vaddr = (unsigned long)frame;
	ptep =
	    pte_offset(pmd_offset
		       (pgd_offset(current->mm, frame_vaddr), frame_vaddr),
		       frame_vaddr);
	pte = *ptep;
	pte = pte_exprotect(pte);
	set_pte(ptep, pte);


    mod_tlb_permission(frame_vaddr, current->mm->context.asid, 
                        EXEC_DISABLE);
#endif
	 	
	if(flag)
	{
#if 0 //wschen 2008-07-18
		/* regs->sp is pointing to the prev virtual page after restore_sigcontext*/
		frame_vaddr = (unsigned long)regs->sp;
		/* get the ASID */
		ptep = pte_offset(pmd_offset(pgd_offset(current->mm,frame_vaddr),frame_vaddr),frame_vaddr);
		pte = *ptep;
		pte = pte_exprotect(pte);
		set_pte(ptep,pte);

		frame_vaddr = frame_vaddr & PAGE_MASK;
		frame_vaddr = frame_vaddr | (current->mm->context.asid & 0xff);
		write_new_aux_reg(ARC_REG_TLBPD0,frame_vaddr);
		write_new_aux_reg(ARC_REG_TLBCOMMAND,TLBProbe);
	
		if(read_new_aux_reg(ARC_REG_TLBINDEX)!=0x80000000)
		{
			write_new_aux_reg(ARC_REG_TLBCOMMAND,TLBRead);
			frame_tlbpd1 = read_new_aux_reg(ARC_REG_TLBPD1);
			/* disable execute */
			frame_tlbpd1  = frame_tlbpd1 | ~0x00000008; 
	
			write_new_aux_reg(ARC_REG_TLBPD1,frame_tlbpd1);
			write_new_aux_reg(ARC_REG_TLBCOMMAND,TLBWrite);
	
		}
#else
		/* regs->sp is pointing to the prev virtual page after 
		   restore_sigcontext */
		frame_vaddr = (unsigned long)regs->sp;
		/* get the ASID */
		ptep =
		    pte_offset(pmd_offset
			       (pgd_offset(current->mm, frame_vaddr),
				frame_vaddr), frame_vaddr);
		pte = *ptep;
		pte = pte_exprotect(pte);
		set_pte(ptep, pte);


        mod_tlb_permission(frame_vaddr, current->mm->context.asid, 
                            EXEC_DISABLE);
#endif
	}
	return regs->r0;

badframe:
	force_sig(SIGSEGV, current);
	return 0;
}

asmlinkage int sys_rt_sigreturn(struct pt_regs *regs)
{
	struct rt_sigframe *frame;
	sigset_t set;
	unsigned long frame_vaddr,frame_tlbpd1;
	int flag=0;
	pte_t *ptep, pte;
	/*
	 * Since we stacked the signal on a word boundary,
	 * then 'sp' should be word aligned here.  If it's
	 * not, then the user is trying to mess with us.
	 */
	if (regs->sp & 3)
		goto badframe;

	frame = ((struct rt_sigframe *)regs->sp) ;
#if 0 //wschen 2008-07-18


	if (verify_area(VERIFY_READ, frame, sizeof (*frame)))
		goto badframe;
	if (__copy_from_user(&set, &frame->uc.uc_sigmask, sizeof(set)))
		goto badframe;

	sigdelsetmask(&set, ~_BLOCKABLE);
	spin_lock_irq(&current->sigmask_lock);
	current->blocked = set;
	recalc_sigpending(current);
	spin_unlock_irq(&current->sigmask_lock);

	if (restore_sigcontext(regs, &frame->uc.uc_mcontext))
		goto badframe;


	/* Check if the retcode is on two different page frames */
	if((regs->sp & PAGE_MASK) != ((regs->sp - sizeof(frame->retcode)) & PAGE_MASK))
		flag = 1;

	/*
	 * disabling the execute permissions of the stack enabled in setup_frame
	 * In the pte entry and in the TLB entry.
	 */
	
	frame_vaddr = (unsigned long)frame;
	ptep = pte_offset(pmd_offset(pgd_offset(current->mm,frame_vaddr),frame_vaddr),frame_vaddr);
	pte = *ptep;
	pte = pte_exprotect(pte);
	set_pte(ptep,pte);

	frame_vaddr = frame_vaddr & PAGE_MASK;
	/* get the ASID */
	frame_vaddr = frame_vaddr | (current->mm->context.asid & 0xff);
	write_new_aux_reg(ARC_REG_TLBPD0,frame_vaddr);
	write_new_aux_reg(ARC_REG_TLBCOMMAND,TLBProbe);
	
	if(read_new_aux_reg(ARC_REG_TLBINDEX)!=0x80000000)
	{ 	

		write_new_aux_reg(ARC_REG_TLBCOMMAND,TLBRead);
		frame_tlbpd1 = read_new_aux_reg(ARC_REG_TLBPD1);
		/* disable execute */
		frame_tlbpd1  = frame_tlbpd1 & ~0x00000008; 
	
		write_new_aux_reg(ARC_REG_TLBPD1,frame_tlbpd1);
		write_new_aux_reg(ARC_REG_TLBCOMMAND,TLBWrite);
	} 	
	if(flag)
	{
		/* regs->sp is pointing to the prev virtual page */
		frame_vaddr = (unsigned long)regs->sp;
		/* get the ASID */
		ptep = pte_offset(pmd_offset(pgd_offset(current->mm,frame_vaddr),frame_vaddr),frame_vaddr);
		pte = *ptep;
		pte = pte_exprotect(pte);
		set_pte(ptep,pte);

		frame_vaddr = frame_vaddr & PAGE_MASK;
		frame_vaddr = frame_vaddr | (current->mm->context.asid & 0xff);
		write_new_aux_reg(ARC_REG_TLBPD0,frame_vaddr);
		write_new_aux_reg(ARC_REG_TLBCOMMAND,TLBProbe);
	
		if(read_new_aux_reg(ARC_REG_TLBINDEX)!=0x80000000)
		{	

			write_new_aux_reg(ARC_REG_TLBCOMMAND,TLBRead);
			frame_tlbpd1 = read_new_aux_reg(ARC_REG_TLBPD1);
			/* disable execute perms */
			frame_tlbpd1  = frame_tlbpd1 | ~0x00000008; 
	
			write_new_aux_reg(ARC_REG_TLBPD1,frame_tlbpd1);
			write_new_aux_reg(ARC_REG_TLBCOMMAND,TLBWrite);
		
		}
	}
#else
	if (!access_ok(VERIFY_READ, frame, sizeof(*frame)))
		goto badframe;

	if (restore_sigframe(regs, &frame->sig))
		goto badframe;

	if (do_sigaltstack(&frame->sig.uc.uc_stack, NULL, regs->sp) == -EFAULT)
		goto badframe;

	/* Check if the retcode is on two different page frames */
/* 	if((regs->sp & PAGE_MASK) != ((regs->sp - sizeof(frame->retcode)) 
	& PAGE_MASK)) */
/* 		flag = 1; */

	/*
	 * disabling the execute permissions of the stack enabled in 
	 * setup_frame
	 * In the pte entry and in the TLB entry.
	 */

	frame_vaddr = (unsigned long)frame;
	ptep =
	    pte_offset(pmd_offset
		       (pgd_offset(current->mm, frame_vaddr), frame_vaddr),
		       frame_vaddr);
	pte = *ptep;
	pte = pte_exprotect(pte);
	set_pte(ptep, pte);


    mod_tlb_permission(frame_vaddr, current->mm->context.asid, 
                            EXEC_DISABLE);

	if (flag) {
		/* regs->sp is pointing to the prev virtual page */
		frame_vaddr = (unsigned long)regs->sp;
		/* get the ASID */
		ptep =
		    pte_offset(pmd_offset
			       (pgd_offset(current->mm, frame_vaddr),
				frame_vaddr), frame_vaddr);
		pte = *ptep;
		pte = pte_exprotect(pte);
		set_pte(ptep, pte);


        mod_tlb_permission(frame_vaddr, current->mm->context.asid, 
                            EXEC_DISABLE);
	}
#endif

	return regs->r0;

badframe:
	force_sig(SIGSEGV, current);
	return 0;
}
#if 1 //wschen 2008-07-18
static int
setup_sigframe(struct sigframe __user * sf, struct pt_regs *regs,
	       sigset_t * set)
{

	int err = 0;

	__put_user(regs->r0, &sf->uc.uc_mcontext.r0);
	__put_user(regs->r1, &sf->uc.uc_mcontext.r1);
	__put_user(regs->r2, &sf->uc.uc_mcontext.r2);
	__put_user(regs->r3, &sf->uc.uc_mcontext.r3);
	__put_user(regs->r4, &sf->uc.uc_mcontext.r4);
	__put_user(regs->r5, &sf->uc.uc_mcontext.r5);
	__put_user(regs->r6, &sf->uc.uc_mcontext.r6);
	__put_user(regs->r7, &sf->uc.uc_mcontext.r7);
	__put_user(regs->r8, &sf->uc.uc_mcontext.r8);
	__put_user(regs->r9, &sf->uc.uc_mcontext.r9);
	__put_user(regs->r10, &sf->uc.uc_mcontext.r10);
	__put_user(regs->r11, &sf->uc.uc_mcontext.r11);
	__put_user(regs->r12, &sf->uc.uc_mcontext.r12);
	__put_user(regs->r26, &sf->uc.uc_mcontext.r26);
	__put_user(regs->fp, &sf->uc.uc_mcontext.fp);
	__put_user(regs->blink, &sf->uc.uc_mcontext.blink);
	__put_user(regs->ret, &sf->uc.uc_mcontext.ret);
	__put_user(regs->status32, &sf->uc.uc_mcontext.status32);
	__put_user(regs->lp_count, &sf->uc.uc_mcontext.lp_count);
	__put_user(regs->lp_end, &sf->uc.uc_mcontext.lp_end);
	__put_user(regs->lp_start, &sf->uc.uc_mcontext.lp_start);
	__put_user(regs->bta, &sf->uc.uc_mcontext.bta);
	__put_user(regs->sp, &sf->uc.uc_mcontext.sp);

	__copy_to_user(&sf->uc.uc_sigmask, set, sizeof(sigset_t));

	return err;
}
#endif

static int
setup_sigcontext(struct sigcontext *sc, struct pt_regs *regs, unsigned long mask)
{
	int err = 0;

	__put_user(regs->r0, &sc->r0);
	__put_user(regs->r1, &sc->r1);
	__put_user(regs->r2, &sc->r2);
	__put_user(regs->r3, &sc->r3);
	__put_user(regs->r4, &sc->r4);
	__put_user(regs->r5, &sc->r5);
	__put_user(regs->r6, &sc->r6);
	__put_user(regs->r7, &sc->r7);
	__put_user(regs->r8, &sc->r8);
	__put_user(regs->r9, &sc->r9);
	__put_user(regs->r10, &sc->r10);
	__put_user(regs->r11, &sc->r11);
	__put_user(regs->r12, &sc->r12);
	__put_user(regs->r26, &sc->r26);
	__put_user(regs->fp, &sc->fp);
	__put_user(regs->blink, &sc->blink);
	__put_user(regs->ret, &sc->ret);
	__put_user(regs->status32, &sc->status32);
	__put_user(regs->lp_count, &sc->lp_count);
	__put_user(regs->lp_end, &sc->lp_end);
	__put_user(regs->lp_start, &sc->lp_start);
	__put_user(regs->bta, &sc->bta);
	__put_user(regs->sp, &sc->sp);
	__put_user(mask, &sc->oldmask);

	return err;
}

static inline void *get_sigframe(struct k_sigaction *ka, struct pt_regs *regs,
				 unsigned long framesize)
{
	unsigned long sp = regs->sp;
#if 0 //wschen 2008-07-18
	/*
	 * This is the X/Open sanctioned signal stack switching.
	 */
	if ((ka->sa.sa_flags & SA_ONSTACK) && !sas_ss_flags(sp))
		sp = current->sas_ss_sp + current->sas_ss_size;

	/*
	 * No matter what happens, 'sp' must be word
	 * aligned otherwise nasty things could happen
	 */
	/* ATPCS B01 mandates 8-byte alignment */
	return (void *)((sp - framesize) & ~7);
#else
	void __user *frame;

	/*
	 * This is the X/Open sanctioned signal stack switching.
	 */
	if ((ka->sa.sa_flags & SA_ONSTACK) && !sas_ss_flags(sp))
		sp = current->sas_ss_sp + current->sas_ss_size;

	/*
	 * No matter what happens, 'sp' must be word
	 * aligned otherwise nasty things could happen
	 */
	/* ATPCS B01 mandates 8-byte alignment */
	frame = (void __user *)((sp - framesize) & ~7);

	/*
	 * Check that we can actually write to the signal frame.
	 */
	if (!access_ok(VERIFY_WRITE, frame, framesize))
		frame = NULL;

	return frame;
#endif
}

static void setup_frame(int sig, struct k_sigaction *ka,
			sigset_t *set, struct pt_regs *regs)
{
#if 0 //wschen 2008-07-18


	struct sigframe *frame;
	unsigned long retcode, frame_vaddr,frame_tlbpd1, frame_paddr;
	pte_t *ptep, pte;
	int err = 0, size;

	frame = get_sigframe(ka, regs, sizeof(struct sigframe));

	if (!access_ok(VERIFY_WRITE, frame, sizeof (*frame)))
		goto segv_and_exit;

	err |= setup_sigcontext(&frame->sc, regs, set->sig[0]);

	if (_NSIG_WORDS > 1) {
		err |= __copy_to_user(frame->extramask, &set->sig[1],
				      sizeof(frame->extramask));
	}

	/* Set up to return from userspace.  If provided, use a stub
	   already in userspace.  */
	if (ka->sa.sa_flags & SA_RESTORER) {
		retcode = (unsigned long)ka->sa.sa_restorer;
	} else {
                unsigned int code;
		retcode = (unsigned long)&frame->retcode;

		/* A7 is middle endian ! */
		/* code for mov r8, __NR_SIGRETURN */
		code = 0x1dc1208a;
		__put_user(code,(unsigned long *)(&frame->retcode[0]));
		/* code for trap0 */
		code = 0x003f226f;
		__put_user(code,(unsigned long *)(&frame->retcode[1]));


		code = 0x7000264a; /* code for nop */
		__put_user(code,(unsigned long *)(&frame->retcode[2]));
		code = 0x7000264a; /* code for nop */
		__put_user(code,(unsigned long *)(&frame->retcode[3]));
	}
	if (err)
		goto segv_and_exit;

	if (current->exec_domain && current->exec_domain->signal_invmap && sig < 32)
		regs->r0 = current->exec_domain->signal_invmap[sig];
	else
		regs->r0 = sig;

	/*
	 * To be able execute the code on the stack(retcode), we need to enable the execute permissions of the stack
	 * In the pte entry and in the TLB entry.
	 */
		
	frame_vaddr = (unsigned long)(&frame->retcode);
	size = sizeof(frame->retcode);

	/* check if retcode crosses a page boundary */
	if((frame_vaddr & PAGE_MASK)!= ((frame_vaddr+size) & PAGE_MASK))
	  {
	
	    frame_vaddr = (frame_vaddr + size) & PAGE_MASK;
	    ptep = pte_offset(pmd_offset(pgd_offset(current->mm,frame_vaddr),frame_vaddr),frame_vaddr);
	    pte = *ptep;
	    pte = pte_mkexec(pte);
	    set_pte(ptep,pte);

	    /* size number of bytes lie on the prev phy stack page */
	    size = frame_vaddr - (unsigned long)(&frame->retcode);
	    
	    /* Flush dcache line, and inv Icache line for frame->retcode */
	    frame_paddr  = pte_val(pte) & PAGE_MASK;
	    flush_dcache_range(frame_paddr,frame_paddr + sizeof(frame->retcode) - size);
	    flush_icache_range(frame_paddr,frame_paddr + sizeof(frame->retcode) - size);
	    
	    /* Append the asid to the virtual addr */
	    frame_vaddr = frame_vaddr | (current->mm->context.asid & 0xff);
	    write_new_aux_reg(ARC_REG_TLBPD0, frame_vaddr);
	    write_new_aux_reg(ARC_REG_TLBCOMMAND,TLBProbe);

	    /* Found an entry in the TLB */
	    if(read_new_aux_reg(ARC_REG_TLBINDEX)!=0x80000000)
	      {

		write_new_aux_reg(ARC_REG_TLBCOMMAND,TLBRead);
		frame_tlbpd1 = read_new_aux_reg(ARC_REG_TLBPD1);
		frame_tlbpd1  = frame_tlbpd1 | 0x00000008; // enable execute
	
		write_new_aux_reg(ARC_REG_TLBPD1,frame_tlbpd1);
		write_new_aux_reg(ARC_REG_TLBCOMMAND,TLBWrite);
	      }	

	    frame_vaddr = (unsigned long)(&frame->retcode);
	  }

	/* Get the offset of retcode in the page */
	frame_paddr = frame_vaddr & ~PAGE_MASK;
	
	frame_vaddr = frame_vaddr & PAGE_MASK;
	ptep = pte_offset(pmd_offset(pgd_offset(current->mm,frame_vaddr),frame_vaddr),frame_vaddr);
	pte = *ptep;
	pte = pte_mkexec(pte);
	set_pte(ptep,pte);

	/* Flush dcache line, and inv Icache line for frame->retcode */
	frame_paddr  = frame_paddr | (pte_val(pte) & PAGE_MASK);	
	flush_dcache_range(frame_paddr, frame_paddr + size);
	flush_icache_range(frame_paddr, frame_paddr + size);

/* 	printk("Stack page va = 0x%08lx, pa = 0x%08lx\n",frame_vaddr,frame_paddr); */
	/* Append the asid to the virtual addr */
	frame_vaddr = frame_vaddr | (current->mm->context.asid & 0xff);
	write_new_aux_reg(ARC_REG_TLBPD0, frame_vaddr);
	write_new_aux_reg(ARC_REG_TLBCOMMAND,TLBProbe);

	/* Found an entry in the TLB */
	if(read_new_aux_reg(ARC_REG_TLBINDEX)!=0x80000000)
	  {

	    write_new_aux_reg(ARC_REG_TLBCOMMAND,TLBRead);
	    frame_tlbpd1 = read_new_aux_reg(ARC_REG_TLBPD1);
	    frame_tlbpd1  = frame_tlbpd1 | 0x00000008; // enable execute
	    
	    write_new_aux_reg(ARC_REG_TLBPD1,frame_tlbpd1);
	    write_new_aux_reg(ARC_REG_TLBCOMMAND,TLBWrite);
	  }	

	regs->sp = ((unsigned long)frame);
	regs->blink = retcode;
	regs->ret =(unsigned long)ka->sa.sa_handler;
	return;

segv_and_exit:
	if (sig == SIGSEGV)
		ka->sa.sa_handler = SIG_DFL;
	force_sig(SIGSEGV, current);
	__asm__ __volatile__ ("mov	r0,0"::);

#else
	struct sigframe __user *frame;
	unsigned long retcode, frame_vaddr, frame_tlbpd1, frame_paddr;
	pte_t *ptep, pte;
	int err = 0, size, size_on_pg1, size_on_pg2;

	frame = get_sigframe(ka, regs, sizeof(struct sigframe));

	if (!frame)
		return;

	err |= setup_sigframe(frame, regs, set);

	/* Set up to return from userspace.  If provided, use a stub
	   already in userspace.  */
	if (ka->sa.sa_flags & SA_RESTORER) {
		retcode = (unsigned long)ka->sa.sa_restorer;
	} else {
		unsigned int code;
		retcode = (unsigned long)&frame->retcode;

		/* A7 is middle endian ! */
		/* code for mov r8, __NR_SIGRETURN */
		code = 0x1dc1208a;
		__put_user(code, (unsigned long *)(&frame->retcode[0]));
		/* code for trap0 */
		code = 0x003f226f;
		__put_user(code, (unsigned long *)(&frame->retcode[1]));

		code = 0x7000264a;	/* code for nop */
		__put_user(code, (unsigned long *)(&frame->retcode[2]));
		code = 0x7000264a;	/* code for nop */
		__put_user(code, (unsigned long *)(&frame->retcode[3]));
	}
	if (err)
		goto segv_and_exit;

	if (current->exec_domain && current->exec_domain->signal_invmap
	    && sig < 32)
		regs->r0 = current->exec_domain->signal_invmap[sig];
	else
		regs->r0 = sig;

	/*
	 * To be able execute the code on the stack(retcode), we need to enable
	 * the execute permissions of the stack
	 * In the pte entry and in the TLB entry.
	 */
    frame_vaddr = (unsigned long)(&frame->retcode);
	size = size_on_pg1 = sizeof(frame->retcode);
    size_on_pg2 = 0;

	/* check if retcode crosses a page boundary */
	if ((frame_vaddr & PAGE_MASK) != ((frame_vaddr + size) & PAGE_MASK)) {
        size_on_pg1 = ((frame_vaddr + size) & PAGE_MASK) - frame_vaddr;
        size_on_pg2 = sizeof(frame->retcode) - size_on_pg1;
    }

	/* Get the offset of retcode in the page */
	frame_paddr = frame_vaddr & ~PAGE_MASK;

	frame_vaddr = frame_vaddr & PAGE_MASK;
    ptep = pte_offset(
                    pmd_offset (
                            pgd_offset(current->mm, frame_vaddr),
				            frame_vaddr), 
                    frame_vaddr);

	pte = *ptep;
	pte = pte_mkexec(pte);
	set_pte(ptep, pte);

	/* Flush dcache line, and inv Icache line for frame->retcode */
	frame_paddr = frame_paddr | (pte_val(pte) & PAGE_MASK);
	flush_dcache_range(frame_paddr, frame_paddr + size_on_pg1);
	flush_icache_range(frame_paddr, frame_paddr + size_on_pg1);

	/* printk("Stack page va = 0x%08lx, pa = 0x%08lx\n",
	   frame_vaddr,frame_paddr); */

    mod_tlb_permission(frame_vaddr, current->mm->context.asid, 
                            EXEC_ENABLE);

    /* retcode straddles page boundary so do that for other pg too */
    if ( size_on_pg2 ) {    

	    frame_vaddr = (unsigned long)(&frame->retcode);
		frame_vaddr = (frame_vaddr + size) & PAGE_MASK;

		ptep = pte_offset(
                    pmd_offset (
                            pgd_offset(current->mm, frame_vaddr),
				            frame_vaddr), 
                    frame_vaddr);

		pte = *ptep;
		pte = pte_mkexec(pte);
		set_pte(ptep, pte);

		/* Flush dcache line, and inv Icache line for frame->retcode */
		frame_paddr = pte_val(pte) & PAGE_MASK;
		flush_dcache_range(frame_paddr, frame_paddr + size_on_pg2);
		flush_icache_range(frame_paddr, frame_paddr + size_on_pg2);

        mod_tlb_permission(frame_vaddr, current->mm->context.asid, 
                            EXEC_ENABLE);

	}


	regs->sp = ((unsigned long)frame);
	regs->blink = retcode;
	regs->ret = (unsigned long)ka->sa.sa_handler;
	return;

 segv_and_exit:
	if (sig == SIGSEGV)
		ka->sa.sa_handler = SIG_DFL;
	force_sig(SIGSEGV, current);
	__asm__ __volatile__("mov	r0,0"::);

#endif
}

static void setup_rt_frame(int sig, struct k_sigaction *ka, siginfo_t *info,
			   sigset_t *set, struct pt_regs *regs)
{
#if 0 //wschen 2008-07-18


	struct rt_sigframe *frame;
	unsigned long retcode,frame_vaddr,frame_tlbpd1,frame_paddr;
	int err = 0, size;
	pte_t *ptep,pte;

	frame = get_sigframe(ka, regs, sizeof(struct rt_sigframe));

	if (!access_ok(VERIFY_WRITE, frame, sizeof (*frame)))
		goto segv_and_exit;

	__put_user(&frame->info, &frame->pinfo);
	__put_user(&frame->uc, &frame->puc);
	err |= copy_siginfo_to_user(&frame->info, info);

	/* Clear all the bits of the ucontext we don't use.  */
	err |= __put_user(0, &frame->uc.uc_flags);
        err |= __put_user(0, &frame->uc.uc_link);
	err |= __put_user(0, &frame->uc.uc_stack.ss_sp);
	err |= __put_user(0, &frame->uc.uc_stack.ss_flags);
	err |= __put_user(0, &frame->uc.uc_stack.ss_size);

	err |= setup_sigcontext(&frame->uc.uc_mcontext,	regs, set->sig[0]);
	err |= __copy_to_user(&frame->uc.uc_sigmask, set, sizeof(*set));

	/* Set up to return from userspace.  If provided, use a stub
	   already in userspace.  */
	if (ka->sa.sa_flags & SA_RESTORER) {
		retcode = (unsigned long)ka->sa.sa_restorer;
	} else {
		unsigned int code;
		retcode = (unsigned long)&frame->retcode;


		/* A7 is middle endian ! */
		/* code for mov r8, __NR_RT_SIGRETURN */
		code = 0x1b42208a;
		__put_user(code,(unsigned long *)(&frame->retcode[0]));
		/* code for trap0 */
		code = 0x003f226f;
		__put_user(code,(unsigned long *)(&frame->retcode[1]));


		code = 0x7000264a; /* code for nop */
		__put_user(code,(unsigned long *)(&frame->retcode[2]));
		code = 0x7000264a; /* code for nop */
		__put_user(code,(unsigned long *)(&frame->retcode[3]));
	}

	if (err)
		goto segv_and_exit;

	if (current->exec_domain && current->exec_domain->signal_invmap && sig < 32)
		regs->r0 = current->exec_domain->signal_invmap[sig];
	else
		regs->r0 = sig;

	/*
	 * For realtime signals we must also set the second and third
	 * arguments for the signal handler.
	 *   -- Peter Maydell <pmaydell@chiark.greenend.org.uk> 2000-12-06
	 */
	regs->r1 = (unsigned long)frame->pinfo;
	regs->r2 = (unsigned long)frame->puc;

	/*
	 * To be able execute the code on the stack(retcode), we need to enable the execute permissions of the stack
	 * In the pte entry and in the TLB entry.
	 */
		
	frame_vaddr = (unsigned long)(&frame->retcode);
	size = sizeof(frame->retcode);

	/* check if retcode crosses a page boundary */
	if((frame_vaddr & PAGE_MASK)!= ((frame_vaddr+size) & PAGE_MASK))
	  {
	
	    frame_vaddr = (frame_vaddr + size) & PAGE_MASK;
	    ptep = pte_offset(pmd_offset(pgd_offset(current->mm,frame_vaddr),frame_vaddr),frame_vaddr);
	    pte = *ptep;
	    pte = pte_mkexec(pte);
	    set_pte(ptep,pte);

	    /* size number of bytes lie on the prev phy stack page */
	    size = frame_vaddr - (unsigned long)(&frame->retcode);
	    
	    /* Flush dcache line, and inv Icache line for frame->retcode */
	    frame_paddr  = pte_val(pte) & PAGE_MASK;	
	    flush_dcache_range(frame_paddr,frame_paddr + sizeof(frame->retcode) - size);
	    flush_icache_range(frame_paddr,frame_paddr + sizeof(frame->retcode) - size);

	    /* Append the asid to the virtual addr */
	    frame_vaddr = frame_vaddr | (current->mm->context.asid & 0xff);
	    write_new_aux_reg(ARC_REG_TLBPD0, frame_vaddr);
	    write_new_aux_reg(ARC_REG_TLBCOMMAND,TLBProbe);

	    /* Found an entry in the TLB */
	    if(read_new_aux_reg(ARC_REG_TLBINDEX)!=0x80000000)
	      {

		write_new_aux_reg(ARC_REG_TLBCOMMAND,TLBRead);
		frame_tlbpd1 = read_new_aux_reg(ARC_REG_TLBPD1);
		frame_tlbpd1  = frame_tlbpd1 | 0x00000008; // enable execute
	
		write_new_aux_reg(ARC_REG_TLBPD1,frame_tlbpd1);
		write_new_aux_reg(ARC_REG_TLBCOMMAND,TLBWrite);
	      }	

	    frame_vaddr = (unsigned long)(&frame->retcode);
	  }

	/* Get the offset of retcode in the page */
	frame_paddr = frame_vaddr & ~PAGE_MASK;
	
	frame_vaddr = frame_vaddr & PAGE_MASK;
	ptep = pte_offset(pmd_offset(pgd_offset(current->mm,frame_vaddr),frame_vaddr),frame_vaddr);
	pte = *ptep;
	pte = pte_mkexec(pte);
	set_pte(ptep,pte);

	/* Flush dcache line, and inv Icache line for frame->retcode */
	frame_paddr  = frame_paddr | (pte_val(pte) & PAGE_MASK);	
	flush_dcache_range(frame_paddr, frame_paddr + size);
	flush_icache_range(frame_paddr, frame_paddr + size);

/* 	printk("Stack page va = 0x%08lx, pa = 0x%08lx\n",frame_vaddr,frame_paddr); */
	/* Append the asid to the virtual addr */
	frame_vaddr = frame_vaddr | (current->mm->context.asid & 0xff);
	write_new_aux_reg(ARC_REG_TLBPD0, frame_vaddr);
	write_new_aux_reg(ARC_REG_TLBCOMMAND,TLBProbe);

	/* Found an entry in the TLB */
	if(read_new_aux_reg(ARC_REG_TLBINDEX)!=0x80000000)
	  {

	    write_new_aux_reg(ARC_REG_TLBCOMMAND,TLBRead);
	    frame_tlbpd1 = read_new_aux_reg(ARC_REG_TLBPD1);
	    frame_tlbpd1  = frame_tlbpd1 | 0x00000008; // enable execute
	    
	    write_new_aux_reg(ARC_REG_TLBPD1,frame_tlbpd1);
	    write_new_aux_reg(ARC_REG_TLBCOMMAND,TLBWrite);
	  }	

	regs->sp = (((unsigned long)frame) );
	regs->blink = retcode;
	regs->ret = ( unsigned long)ka->sa.sa_handler;
	return;

segv_and_exit:
	if (sig == SIGSEGV)
		ka->sa.sa_handler = SIG_DFL;
	force_sig(SIGSEGV, current);
#endif
}

/*
 * OK, we're invoking a handler
 */	
static void
handle_signal(unsigned long sig, struct k_sigaction *ka,
	      siginfo_t *info, sigset_t *oldset, struct pt_regs * regs)
{
  	unsigned long flags;
	/* Set up the stack frame */
	if (ka->sa.sa_flags & SA_SIGINFO)
		setup_rt_frame(sig, ka, info, oldset, regs);
	else
		setup_frame(sig, ka, oldset, regs);

#if 0 //wschen 2008-07-18
	if (ka->sa.sa_flags & SA_ONESHOT)
		ka->sa.sa_handler = SIG_DFL;


	if (!(ka->sa.sa_flags & SA_NODEFER)) {
		spin_lock_irq(&current->sigmask_lock);
		sigorsets(&current->blocked,&current->blocked,&ka->sa.sa_mask);
		sigaddset(&current->blocked,sig);
		recalc_sigpending(current);
		spin_unlock_irq(&current->sigmask_lock);
	}
#else
	local_irq_save(flags);
	sigorsets(&current->blocked, &current->blocked, &ka->sa.sa_mask);
	if (!(ka->sa.sa_flags & SA_NODEFER))
		sigaddset(&current->blocked, sig);
	recalc_sigpending(current);
	local_irq_restore(flags);

#endif
}

/*
 * Note that 'init' is a special process: it doesn't get signals it doesn't
 * want to handle. Thus you cannot kill init even with a SIGKILL even by
 * mistake.
 *
 * Note that we go through the signals twice: once to check the signals that
 * the kernel can handle, and then we build all the user-level signal handling
 * stack-frames in one go after that.
 */
asmlinkage int do_signal(struct pt_regs *regs, sigset_t *oldset)
{
	struct k_sigaction *ka;
	siginfo_t info;

	/*
	 * We want the common case to go fast, which
	 * is why we may in certain cases get here from
	 * kernel mode. Just return without doing anything
	 * if so.
	 */
	if (!user_mode(regs))
		return 0;

	if (!oldset)
		oldset = &current->blocked;

	for (;;) {
		unsigned long signr;

		spin_lock_irq (&current->sigmask_lock);
		signr = dequeue_signal(&current->blocked, &info);
		spin_unlock_irq (&current->sigmask_lock);

		if (!signr)
			break;

		if ((current->ptrace & PT_PTRACED) && signr != SIGKILL) {
			/* Let the debugger run.  */
			current->exit_code = signr;
			current->state = TASK_STOPPED;
			notify_parent(current, SIGCHLD);
			schedule();
	
			/* We're back.  Did the debugger cancel the sig?  */
			if (!(signr = current->exit_code))
				continue;
			current->exit_code = 0;

			/* The debugger continued.  Ignore SIGSTOP.  */
			if (signr == SIGSTOP)
				continue;

			/* Update the siginfo structure.  Is this good? */
			if (signr != info.si_signo) {
				info.si_signo = signr;
				info.si_errno = 0;
				info.si_code = SI_USER;
				info.si_pid = current->p_pptr->pid;
				info.si_uid = current->p_pptr->uid;
			}

			/* If the (new) signal is now blocked, requeue it.  */
			if (sigismember(&current->blocked, signr)) {
				send_sig_info(signr, &info, current);
				continue;
			}
		}

		ka = &current->sig->action[signr-1];
		if (ka->sa.sa_handler == SIG_IGN) {
			if (signr != SIGCHLD)
				continue;
			/* Check for SIGCHLD: it's special.  */
			while (sys_wait4(-1, NULL, WNOHANG, NULL) > 0)
				/* nothing */;
			continue;
		}

		if (ka->sa.sa_handler == SIG_DFL) {
			int exit_code = signr;

			/* Init gets no signals it doesn't want.  */
			if (current->pid == 1)
				continue;

			switch (signr) {
			case SIGCONT: case SIGCHLD: case SIGWINCH:
				continue;

			case SIGTSTP: case SIGTTIN: case SIGTTOU:
				if (is_orphaned_pgrp(current->pgrp))
					continue;
				/* FALLTHRU */

			case SIGSTOP:
				current->state = TASK_STOPPED;
				current->exit_code = signr;
				if (!(current->p_pptr->sig->action[SIGCHLD-1].sa.sa_flags & SA_NOCLDSTOP))
					notify_parent(current, SIGCHLD);
				schedule();
				continue;

			case SIGQUIT: case SIGILL: case SIGTRAP:
			case SIGABRT: case SIGFPE: case SIGSEGV:
			case SIGBUS: case SIGSYS: case SIGXCPU: case SIGXFSZ:
				if (do_coredump(signr, regs))
					exit_code |= 0x80;
				/* FALLTHRU */

			default:
			        sig_exit(signr, exit_code, &info); 
				/* NOTREACHED */
			}
		}

		/* Are we from a system call? */
		
		if (regs->orig_r8 >= 0 && regs->orig_r8 <= NR_syscalls) {
			switch (regs->r0) {
			case -ERESTARTNOHAND:
				regs->r0 = -EINTR;
				break;

			case -ERESTARTSYS:
				if (!(ka->sa.sa_flags & SA_RESTART)) {
					regs->r0 = -EINTR;
					break;
				}
				/* fallthrough */
			case -ERESTARTNOINTR:
			/* We don't need to restore r8, as its value will be restored anyway. */
				regs->r0 = regs->orig_r0;
				regs->ret -= 4; 
			}
		}
		/* Whee!  Actually deliver the signal.  */
		handle_signal(signr, ka, &info, oldset, regs);
		return 1;
	}

	
	if ((regs->orig_r8 >= 0 && regs->orig_r8 <= NR_syscalls) &&
	    (regs->r0 == -ERESTARTNOHAND ||
	     regs->r0 == -ERESTARTSYS ||
	     regs->r0 == -ERESTARTNOINTR)) {
		regs->r0 = regs->orig_r0;
		regs->ret -= 4;
	}
	return 0;
}

#if 1 //wschen 2008-07-18
void mod_tlb_permission(unsigned long frame_vaddr,
               unsigned int asid, int exec_or_not)
{
    unsigned long frame_tlbpd1;

	frame_vaddr = frame_vaddr & PAGE_MASK;
	/* get the ASID */
	frame_vaddr = frame_vaddr | (asid & 0xff);
	write_new_aux_reg(ARC_REG_TLBPD0, frame_vaddr);
	write_new_aux_reg(ARC_REG_TLBCOMMAND, TLBProbe);

#if 0 //wschen 2008-07-30
	if (read_new_aux_reg(ARC_REG_TLBINDEX) != 0x80000000) {
		write_new_aux_reg(ARC_REG_TLBCOMMAND, TLBRead);
		frame_tlbpd1 = read_new_aux_reg(ARC_REG_TLBPD1);

        if (EXEC_DISABLE == exec_or_not) {
		    /* disable execute permissions for the user stack */
		    frame_tlbpd1 = frame_tlbpd1 & ~0x00000008;
        }
        else {
			/* enable execute */
			frame_tlbpd1 = frame_tlbpd1 | 0x00000008;
        }

		write_new_aux_reg(ARC_REG_TLBPD1, frame_tlbpd1);
		write_new_aux_reg(ARC_REG_TLBCOMMAND, TLBWrite);
	}
#else
	if (read_new_aux_reg(ARC_REG_TLBINDEX) & 0x80000000 == 0) {
		write_new_aux_reg(ARC_REG_TLBCOMMAND, TLBRead);
		frame_tlbpd1 = read_new_aux_reg(ARC_REG_TLBPD1);

        if (EXEC_DISABLE == exec_or_not) {
		    /* disable execute permissions for the user stack */
		    frame_tlbpd1 = frame_tlbpd1 & ~0x00000008;
        }
        else {
			/* enable execute */
			frame_tlbpd1 = frame_tlbpd1 | 0x00000008;
        }

		write_new_aux_reg(ARC_REG_TLBPD1, frame_tlbpd1);
		write_new_aux_reg(ARC_REG_TLBCOMMAND, TLBWrite);
	}

#endif
}

#endif

