/*
	This file include configurations of display module.
*/

#include "arc_disp_param.h"
#include "arc_tcon.h"
#include "arc_pgu.h"
#include <linux/autoconf.h>

/*----------------------------------------------------------------------------*/
struct known_displays displays[] =
{
	//== 0: NTSC 720x480 TV out
	{ 
		//-- display resolution
		720, 	// xres
		480, 	// yres
		16,	 	// bpp
		//-- Display name
		"NTSC 720x480",
		//-- mode_control
		0,
		//-- clkcontrol
		0,
		//-- hsync
		0,
		//-- vsync
		0,
	},
	//== 1: PAL 720x576 TV out
	{ 
		//-- display resolution
		720, 	// xres
		576, 	// yres
		16,	 	// bpp
		//-- Display name
		"PAL 720x576",
		//-- mode_control
		0,
		//-- clkcontrol
		0,
		//-- hsync
		0,
		//-- vsync
		0,
	},
	//== 2: HDTV 1920x1080i TV out
	{ 
		//-- display resolution
		1920, 	// xres
		1080, 	// yres
		16,	 	// bpp
		//-- Display name
		"HDTV 1920x1080",
		//-- mode_control
		0,
		//-- clkcontrol
		0,
		//-- hsync
		0,
		//-- vsync
		0,
	},	
	//== 3: 1024x600 LCD panel use T-CON scalar
	{ 
		//-- display resolution
		800, 	// xres
		480, 	// yres
		16,	 	// bpp
		//-- Display name
		"1024x600 LCD panel use T-CON scalar",
		//-- mode_control
		// 0x200				     0x400					 0x800
		( STATCTRL_HPOL_MASK | STATCTRL_VPOL_MASK | STATCTRL_DPOL_MASK ),
		//-- clkcontrol
		( 0x00010004 ),
		//-- hsync
		// HSYNC_WIDTH = 0x10 (16)
		// H_BCK_PORTH = 0x57 (88 = 87 + 1)
		// HSYNC_CYC   = 0x41F (1056 = 1055 + 1)
		( 0x1057041F ),
		//-- vsync
		// VSYNC_WIDTH = 0x02 (2)
		// V_BCK_PORTH = 0x2C (45 = 44 + 1)
		// VSYNC_CYC   = 0x20C (525 = 524 + 1)
		( 0x022C020C ),
	},
	//== 4: 800x480 LCD panel with T-CON
	{ 
		//-- display resolution
		800, 	// xres
		480, 	// yres
		16,	 	// bpp
		//-- Display name
		"800x480 LCD panel with T-CON",
		//-- mode_control
		// 0x200				     0x400					 0x800
		( STATCTRL_HPOL_MASK | STATCTRL_VPOL_MASK | STATCTRL_DPOL_MASK ),
		//-- clkcontrol
		( 0x00010003 ),
		//-- hsync
		// HSYNC_WIDTH = 0x10 (16)
		// H_BCK_PORTH = 0x57 (88 = 87 + 1)
		// HSYNC_CYC   = 0x41F (1056 = 1055 + 1)
		( 0x1057041F ),
		//-- vsync
		// VSYNC_WIDTH = 0x02 (2)
		// V_BCK_PORTH = 0x2C (45 = 44 + 1)
		// VSYNC_CYC   = 0x20C (525 = 524 + 1)
		( 0x022C020C ),
	},
	//== 5: 800x600 LCD panel with T-CON
	{ 
		//-- display resolution
		800, 	// xres
		600, 	// yres
		16,	 	// bpp
		//-- Display name
		"800x600 LCD panel with T-CON",
		//-- mode_control
		// 0x200				     0x400					 0x800
		( STATCTRL_HPOL_MASK | STATCTRL_VPOL_MASK | STATCTRL_DPOL_MASK ),
		//-- clkcontrol
		( 0x00010003 ),
		//-- hsync
		// HSYNC_WIDTH = 0x64 (100)
		// H_BCK_PORTH = 0x9f (159+1)
		// HSYNC_CYC   = 0x3E7 (999+1)  
		( 0x649f03E7 ),	// H period = 1000
		//-- vsync
		// VSYNC_WIDTH = 0x02 (2)
		// V_BCK_PORTH = 0x13 (19+1)
		// VSYNC_CYC   = 0x273 (627+1)  
		( 0x02130273 ),	// V period = 628
	},
	//== 6: 1366x768 LCD panel use T-CON scalar
	{ 
		//-- display resolution
		800, 	// xres
		480, 	// yres
		16,	 	// bpp
		//-- Display name
		"1366x768 LCD panel use T-CON scalar",
		//-- mode_control
		// 0x200				     0x400					 0x800
		( STATCTRL_HPOL_MASK | STATCTRL_VPOL_MASK | STATCTRL_DPOL_MASK ),
		//-- clkcontrol
		( 0x00010004 ),
		//-- hsync
		// HSYNC_WIDTH = 0x10 (16)
		// H_BCK_PORTH = 0x57 (88 = 87 + 1)
		// HSYNC_CYC   = 0x41F (1056 = 1055 + 1)
		( 0x1057041F ),
		//-- vsync
		// VSYNC_WIDTH = 0x02 (2)
		// V_BCK_PORTH = 0x2C (45 = 44 + 1)
		// VSYNC_CYC   = 0x20C (525 = 524 + 1)
		( 0x022C020C ),
	},
	//== 7: 640x480 LCD panel with T-CON
	{ 
		//-- display resolution
		640, 	// xres
		480, 	// yres
		16,	 	// bpp
		//-- Display name
		"640x480 LCD panel with T-CON",
		//-- mode_control
		// 0x200				     0x400					 0x800
		( STATCTRL_HPOL_MASK | STATCTRL_VPOL_MASK | STATCTRL_DPOL_MASK ),
		//-- clkcontrol
		( 0x00020005 ),
		//-- hsync
		// HSYNC_WIDTH = 0x60 (96)
		// H_BCK_PORTH = 0x95 (150 = 149 + 1)
		// HSYNC_CYC   = 0x31F (800 = 799 + 1)
		( 0x6095031f ),
		//-- vsync
		// VSYNC_WIDTH = 0x02 (2)
		// V_BCK_PORTH = 0x23 (36 = 35 + 1)
		// VSYNC_CYC   = 0x20C (525 = 524 + 1)
		( 0x0223020C ),
	},
	//== 8: 800x480 LCD panel with T-CON (new timming)
	{ 
		//-- display resolution
		800, 	// xres
		480, 	// yres
		16,	 	// bpp
		//-- Display name
		"800x480 LCD panel with T-CON (new timming)",
		//-- mode_control
		// 0x200				     0x400					 0x800
		( STATCTRL_HPOL_MASK | STATCTRL_VPOL_MASK | STATCTRL_DPOL_MASK ),
		//-- clkcontrol
		( 0x00020005 ),
		//-- hsync
		// HSYNC_WIDTH = 0x10 (16)
		// H_BCK_PORTH = HSYNC_WIDTH + real_back_porch (1F = (16 + 16) - 1)
		// HSYNC_CYC   = 0x33B (828 = 827 + 1)
		( 0x10550390 ),
		//-- vsync
		// VSYNC_WIDTH = 0x03 (3)
		// V_BCK_PORTH = 0x0F (16 = 15 + 1)
		// VSYNC_CYC   = 0x1F7 (504 = 503 + 1)
		( 0x03200208 ),
	},
	//== x: Himax T-CON for 480x234 TFT panel(RGB 18 bits)
	{ 
		//-- display resolution
		480, 	// xres
		240, 	// yres
		16,	 	// bpp
		//-- Display name
		"Himax T-CON for 480x234 TFT panel(RGB 18 bits)",
		//-- mode_control
		// 0x200				     0x400					 0x800
		( STATCTRL_HPOL_MASK | STATCTRL_VPOL_MASK | STATCTRL_DPOL_MASK ),
		//-- clkcontrol
		( 0x00080010 ),
		//-- hsync
		// HSYNC_WIDTH = 0x05 (5)
		// H_BCK_PORTH = 0x35 (53+1)
		// HSYNC_CYC   = 0x257 (599+1)
		( 0x05350257 ),
		//-- vsync
		// VSYNC_WIDTH = 0x03 (3)
		// V_BCK_PORTH = 0x10 (17+1)
		// VSYNC_CYC   = 0x105 (261+1)
		( 0x03110105 ),
	},
	//== x: Himax HX8817A T-CON and CPT CLAA070WA03 TFT LCD (NTSC)
	{ 
		//-- display resolution
		720, 	// xres
		480, 	// yres
		16,	 	// bpp
		//-- Display name
		"Himax HX8817A T-CON and CPT CLAA070WA03 TFT LCD (NTSC)",
		//-- mode_control
		0,
		//-- clkcontrol
		0,
		//-- hsync
		0,
		//-- vsync
		0,
	},
	//== x: Himax HX8817A T-CON and CPT CLAA070WA03 TFT LCD (PAL).
	{ 
		//-- display resolution
		720, 	// xres
		576, 	// yres
		16,	 	// bpp
		//-- Display name
		"Himax HX8817A T-CON and CPT CLAA070WA03 TFT LCD (PAL)",
		//-- mode_control
		0,
		//-- clkcontrol
		0,
		//-- hsync
		0,
		//-- vsync
		0,
	},
};

