/************************************************************************
 *		This file include primitive functions which can control the
 *		TeraWins T-CON.
 ************************************************************************/
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/spinlock.h>
#include "arc_pgu.h"
#include "arc_tcon.h"
#include "arc_disp_param.h"

void terawins_set_osd_line_info(struct _terawins_osd_info* _osd);

#if CONFIG_TERAWINS_TCON

extern int *TerawinsCmdTbl[TERAWINS_END*2];

extern unsigned char font_index;	// index of font type.
extern struct _font_info font_info[FONT_END];

extern struct _terawins_osd_info video_speed;
extern struct _terawins_osd_info icon;
extern struct _terawins_osd_info subtitle;
extern struct _terawins_osd_info mute_vol;
extern struct _terawins_osd_info video_time;
extern struct _terawins_osd_info* terawins_osd[OI_ALL];
extern struct _terawins_osd_info blank;

extern char valid_icon[ICON_NUM * ICON_NUM];
extern char invalid_icon[ICON_NUM * ICON_NUM];
extern char mute_icon[MUTE_NUM];
extern char vol_icon[VOL_NUM];

//== Global variable
static unsigned long val = 0;
static volatile unsigned long vtval, vtval1;
unsigned long tval;
struct _osd_line osd_line_data;

static unsigned int gI2CAddrWA=0;

/*******************************************************************************
			Terawins I2C control
*******************************************************************************/
/* -------------------------------------------------------------------------- */
inline void terawins_delay(unsigned long count)
{
	unsigned long _x; 
	do { 
		for(_x=0;_x< count;_x++){
			__asm__ __volatile__("nop");
			//__builtin_arc_nop();
		}
	} while(0);
}

/* -------------------------------------------------------------------------- */
void terawins_i2c_reset(void)
{
	SCL_HI;
	SCL_OUTPUT;
	SDA_HI;
	SDA_OUTPUT;	

	terawins_delay(TERAWINS_TIME_1);
}

/* -------------------------------------------------------------------------- */
void terawins_i2c_start(void)
{
	SDA_HI;
	SDA_OUTPUT;
	SCL_HI;
	SCL_OUTPUT;
	terawins_delay(TERAWINS_TIME_1);	
	SDA_LO;
	terawins_delay(TERAWINS_TIME_1);	
	SCL_LO;
}

/* -------------------------------------------------------------------------- */
void terawins_i2c_stop(void)
{
	SDA_LO;
	SCL_HI;
	terawins_delay(TERAWINS_TIME_1);
	SDA_HI;
}

/* -------------------------------------------------------------------------- */
void terawins_i2c_ack(int bit)
{
	/* Here's the deal. The slave is slow, and sometimes needs to wait
       before it can receive the acknowledge. Therefore it forces the clock
       low until it is ready. We need to poll the clock line until it goes
       high before we release the ack. */
    
	SCL_LO;      /* Set the clock low */
	if ( bit )
	{
		SDA_HI;
	}
	else
	{
		SDA_LO;
	}
    
	SCL_INPUT;   /* Set the clock to input */
	#if 0
	while(!SCL);  /* and wait for the slave to release it */      
	#else
	do{
		SCL;
	}while(!vtval);
	#endif
	terawins_delay(TERAWINS_TIME_1);
	SCL_OUTPUT;
	SCL_LO;
	terawins_delay(TERAWINS_TIME_2);	
}

/* -------------------------------------------------------------------------- */
int terawins_i2c_getack(void)
{
	int ret = 1;

	SDA_INPUT;
	terawins_delay(TERAWINS_TIME_2);
	SCL_HI;
	#if 0
	if (SDA)
	    ret = 0;
	#else
	SDA;
	if (vtval)
	    ret = 0;

	#endif
	
	terawins_delay(TERAWINS_TIME_2);

	/* pull up SDA first, and set SDA to OUTPUT */
	//Flowbee
//	SDA_LO;   
	SDA_HI;
	SDA_OUTPUT;  
	  
	SCL_LO;

	return ret;
}

/* -------------------------------------------------------------------------- */
void terawins_i2c_outb(unsigned char byte)
{
    int i;
   /* clock out each bit, MSB first */
   for ( i=0x80; i; i>>=1 ) {
      	if ( i & byte ){
        	SDA_HI;
      	}
      	else{
        	SDA_LO;
      	}
	terawins_delay(TERAWINS_TIME_1);
      	SCL_HI;
	terawins_delay(TERAWINS_TIME_1);
	SCL_LO;
   }

   SDA_HI;
}

/* -------------------------------------------------------------------------- */
unsigned char terawins_i2c_inb(int msb, int ack)
{
	int i;
	unsigned char byte = 0;

	if(msb){
	/* clock in each bit, MSB first */
		for ( i=0x80; i; i>>=1 ) {
			terawins_delay(TERAWINS_TIME_1);
			SDA_HI;
			SDA_INPUT;   /* And set to input */
			SCL_HI;
			terawins_delay(TERAWINS_TIME_1);
			#if 0
			if ( SDA )
				byte |= i;

			#else
			SDA;
			if (vtval)
				byte |= i;

			#endif

			SCL_LO;
			SDA_OUTPUT;
			terawins_delay(TERAWINS_TIME_1);
		}
	}else{
	/* clock in each bit, LSB first */
		for(i=0; i<8; i++){
			SDA_HI;
			SDA_INPUT;   /* And set to input */
			SCL_HI;
			terawins_delay(TERAWINS_TIME_1);		
			#if 0
			if ( SDA )
				byte |= (1 << i);

			#else
			SDA;
			if (vtval)
				byte |= (1 << i);

			#endif

			SCL_LO;
			SDA_OUTPUT;
			terawins_delay(TERAWINS_TIME_1);			
		}
	}
   
	terawins_i2c_ack(ack);
	return byte;
}

/* -------------------------------------------------------------------------- */
#if 0 //Mel: Original
int terawins_i2c_write(int address, const unsigned char* buf, int count)
{
	int i,x=0;

	terawins_i2c_start();
	terawins_i2c_outb(address & 0xFE);
	if (terawins_i2c_getack()){
	    for (i=0; i<count; i++){
	    	terawins_i2c_outb(buf[i]);
	        if (!terawins_i2c_getack()){
	        	x=2;
	                break;
	        }
	    }
	}else{
		x=1;
	}
	
	terawins_i2c_stop();
	return x;
}

#else //Mel modified
int terawins_i2c_write(int address, const unsigned char* buf, int count)
{
	int i,x=0;	
    int retry=0;

_AddrRetry:
	terawins_i2c_start();
	terawins_i2c_outb(address & 0xFE);
	if (terawins_i2c_getack())
	{
	    for (i=0; i<count; i++)
	    {
	    	terawins_i2c_outb(buf[i]);
	        if (!terawins_i2c_getack()) {
	        	x=2;
	            break;
	        }
	    }
	}else {
		x=1;
	}
	terawins_i2c_stop();

    if(x==1 && retry==0)
    {
        x=0;
        retry=1;
        address += 0x10;
        
        if(gI2CAddrWA==0) {
            printk("############################################\n");
            printk("!!!!!!!!!!! Retry I2C Address(0x%x)\n", address);
            printk("############################################\n");
            gI2CAddrWA=1;
        }

        //printk("Retry I2C_Addr(0x%x)\n", address);
        goto _AddrRetry;
    }
	return x;
}
#endif

/* -------------------------------------------------------------------------- */
static spinlock_t sk_i2c_lock = SPIN_LOCK_UNLOCKED;
static unsigned long sk_i2c_flag;
int terawins_set_cmd(uint8 dev_addr, uint8 reg_addr, uint8 reg_val)
{
	unsigned char _buf[2] = { reg_addr, reg_val};
	int i;
    //printk("TCON: P[%X] R[%X] V[%X]\n", dev_addr, reg_addr, reg_val);

	//-- Flowbee 2008-01-30 critical area. Using spinlock.
	spin_lock_irqsave(&sk_i2c_lock, sk_i2c_flag);
	i = terawins_i2c_write(dev_addr, _buf, 2);
	spin_unlock_irqrestore(&sk_i2c_lock, sk_i2c_flag);
	if(i != 0){
		TW_MSG("Terawins I2C write(%X:%X-%X) error(%d)!!\n", dev_addr, reg_addr, reg_val, i);
	}
	return i;
}

/* -------------------------------------------------------------------------- */
int terawins_set_page(unsigned char _page_num, int *_page)
{
	int ret_val = 0;
	while(*(_page) != END_CMD){
        printk("TabPg P[%02x] R[%02x] V[%02x]\n", _page_num, (unsigned char)(*(_page)), (unsigned char)(*(_page + 1)));
		ret_val = terawins_set_cmd(_page_num, (unsigned char)(*(_page)), (unsigned char)(*(_page + 1)));
		if((ret_val == 2) || (ret_val == 1)){
			TW_MSG("Write to TeraWins Error!! page:%x %x %x\n", _page_num, (unsigned char)(*(_page)), (unsigned char)(*(_page + 1)));
			return ret_val;
		}
		_page += 2;
		terawins_delay(0x10);
	}
	return ret_val;
}

#if 0	//-- unused function
/* -------------------------------------------------------------------------- */
static int terawins_i2c_read(int address, unsigned char* buf, int count, int msb)
{
    int i,x=1;

	terawins_i2c_start();
	terawins_i2c_outb(address | 0x01);
	if (terawins_i2c_getack()) {
		for (i=0; i<count; i++)
			buf[i] = terawins_i2c_inb(msb, 0);
	}
	else
        x=0;
	
	terawins_i2c_stop();
	return x;
}

/* -------------------------------------------------------------------------- */
static unsigned char terawins_get_cmd(uint8 dev_addr, uint8 reg_addr)
{
	unsigned char _buf = reg_addr;
    int i,x=1;	
	int count = 1;
	int msb = 1;
	/* Step1: write */
	terawins_i2c_start();
	terawins_i2c_outb(dev_addr & 0xFE);
	if (terawins_i2c_getack()){
    	terawins_i2c_outb(_buf);
        if (!terawins_i2c_getack()){
        	x=2;
        }
	}else{
		x=1;
	}
    /* Step2: restart and read. */
	terawins_i2c_start();
	terawins_i2c_outb(dev_addr | 0x01);
	if (terawins_i2c_getack()) {
		for (i=0; i<count; i++)
			_buf = terawins_i2c_inb(msb, 0);
	}
	else
        x=0;
	
	terawins_delay(TERAWINS_TIME_2);	
	terawins_i2c_stop();
	terawins_delay(TERAWINS_TIME_2);
	return _buf;
}
#endif

/*******************************************************************************
			Terawins functions
*******************************************************************************/
/* -------------------------------------------------------------------------- */
void terawins_on_off(uint8 _on)
{
	if(_on == 1){
		//-- turn on
		terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xC2, 0x00);
		terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0x91, 0x07);
		terawins_delay(0x200000);
	}
	else{
		//-- turn off
		terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xC2, 0x12);
		terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0x91, 0x87);
	}
}

/* -------------------------------------------------------------------------- */
int terawins_set_resolution(uint8 idx)
{
	int ret_val = 0;

    printk("terawins_set_resolution(%d)\n", idx);

	/* Set command to terawins T-CON. */
	//-- Setting page0 registers.
	ret_val = terawins_set_page(TERAWINS_PAGE0_ADDR, TerawinsCmdTbl[idx * 2]);
	//-- Setting page1 registers.
	ret_val |= terawins_set_page(TERAWINS_PAGE1_ADDR, TerawinsCmdTbl[idx * 2 + 1]);

    printk("terawins_set_resolution(%d) done\n", idx);
	return ret_val;
}

/* -------------------------------------------------------------------------- */
void terawins_pwm_on_off(uint8 _on)
{
	int i = 0;
	if(_on == 1){
		//-- Turn on PWM
        //Mel: original
		//i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xe8, 0x10);
        i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xe8, 0x13); //206Hz
	}
	else{
		//-- Turn off PWM
        //Mel: original
		//i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xe8, 0x00);
		i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xe8, 0x03);
	}
	if(i != 0){
		TW_MSG("Set PWM on/off fail !\n");
	}
}

/* -------------------------------------------------------------------------- */
void terawins_backlight_on_off(uint8 _on)
{
	int i = 0;
//MEL port for spk pop noise---
	QMI_MUTE_OUTPUT;
	QMI_MUTE_LO;
	terawins_delay(200);	
//-----------------------------
	if(_on == 1){
		//-- Turn on terawins backlight
		i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xec, 0xe2);
	}
	else{
		//-- Turn off terawins backlight
		i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xec, 0xc0);			
	}
	if(i != 0){
		TW_MSG("Terawins backlight on/off fail !\n");
	}
	QMI_MUTE_HI; ///MEL port for spk pop noise
}

#if (CONFIG_SKY_IO_QMI_COMPANY_V1 || CONFIG_SKY_IO_QMI_COMPANY_V2)
/* -------------------------------------------------------------------------- */
int set_qmi_brightness(uint8 dev_addr, uint8 reg_val)
{
	int i;

	reg_val &= 0x7F; 		//0~127
	
	if( reg_val == 127 ){	//Set backlight off (GPIO[3]=0) or (GPIO[15]=0)
	     QMI_BL_OUTPUT;
	     QMI_BL_LO;
	     TW_MSG("Set Backlight Off\n");
	     return 0;
	}
	else{				//Set backlight on (GPIO[3]=1) or (GPIO[15]=1)
	     QMI_BL_OUTPUT;
          QMI_BL_HI;
		//Patch AD5247 value for IIC address = 0x5c
		if (dev_addr == QMI_AD5247_ADDR_5C){
	          switch(reg_val)
     	     {
          		case 100://20%
          			reg_val = 82;
          			break;
	          	case 87: //30%
					reg_val = 78;
          			break;
          		case 75: //40%
					reg_val = 75;
	          		break;
     	     	case 62: //50%
					reg_val = 71;
          			break;
	          	case 50: //60%
					reg_val = 67;
          			break;
	          	case 37: //70%
					reg_val = 64;
          			break;
          		case 25: //80%
					reg_val = 59;
     	     		break;
          		case 12: //90%
					reg_val = 52;
	          		break;
     	     	default:
          		     break;
	          }
		}
	}
	spin_lock_irqsave(&sk_i2c_lock, sk_i2c_flag);
	i = terawins_i2c_write(dev_addr, &reg_val, 1);
	spin_unlock_irqrestore(&sk_i2c_lock, sk_i2c_flag);
	
	return i;
}
#endif //CONFIG_SKY_IO_QMI_COMPANY_V1 || CONFIG_SKY_IO_QMI_COMPANY_V2

#if CONFIG_SKY_IO_QMI_COMPANY_V1
/* -------------------------------------------------------------------------- */
int set_qmi_contrast(uint8 reg_val)
{     
	if(reg_val == 255){			//Set contrast high (GPIO[13]=0,GPIO[14]=1)
          QMI_CT1_OUTPUT;
	     QMI_CT1_LO;
	     QMI_CT2_OUTPUT;
	     QMI_CT2_HI;
	     //TW_MSG("Set QMI Blue LED High\n");
     }
	else if(reg_val == 254){		//Set contrast middle (GPIO[13]=1,GPIO[14]=0)
	     QMI_CT1_OUTPUT;
	     QMI_CT1_HI;
	     QMI_CT2_OUTPUT;
	     QMI_CT2_LO;
	     //TW_MSG("Set QMI Blue LED Middle\n");
     }
	else if(reg_val == 253){		//Set contrast off (GPIO[13]=0,GPIO[14]=0)
	     QMI_CT1_OUTPUT;
	     QMI_CT1_LO;
	     QMI_CT2_OUTPUT;
	     QMI_CT2_LO;
	     //TW_MSG("Set QMI Blue LED Off\n");
     }
	else{					//Invalid value
		return -1;
	}
	return 0;
}

/* -------------------------------------------------------------------------- */
void get_gpio_status(uint32* gpio)
{     
     vtval = GIO_R32(GPIO_DIR);
	vtval = GIO_R32(GPIO_DIR);
	vtval &= (~QMI_BLACK_WHITE_MASK);
	GIO_W32(GPIO_DIR, vtval);
     vtval = GIO_R32(GPIO_DATA);
     vtval = GIO_R32(GPIO_DATA);
     //TW_MSG("Get QMI GPIO Status = 0x%08x\n",(vtval & QMI_BLACK_WHITE_MASK));
     *gpio = (vtval & QMI_BLACK_WHITE_MASK);
}

/* -------------------------------------------------------------------------- */
void set_lcd_on_off(uint8 lcd_on_off)
{     
    //Mel port for pop Noise--
	QMI_MUTE_OUTPUT;
	QMI_MUTE_LO;	
	terawins_delay(200);
    //------------------------
	if(lcd_on_off){			//Set lcd on(GPIO[15]=1)
		QMI_LCD_OUTPUT;
		QMI_LCD_HI;
		//TW_MSG("Set QMI LCD On\n");
	}
	else{					//Set lcd off(GPIO[15]=0)
		QMI_LCD_OUTPUT;
		QMI_LCD_LO;
		//TW_MSG("Set QMI LCD Off\n");
	}
	QMI_MUTE_HI; //Mel port for pop Noise
}
#endif //CONFIG_SKY_IO_QMI_COMPANY_V1

/* -------------------------------------------------------------------------- */
void terawins_set_pwm(uint8 _val)
{
	int i = 0;	
    static uint8 pwm_last=0;

//-----------------------------------------------
// Mel : avoid LCD backlight refresh when booting
    if(_val==1 || (pwm_last==0x1 && _val==0x60) )
    {
        if(_val==1) pwm_last=1;
        printk("PWM[0x%x]DoNothing\n", _val);
        return;
    }
    pwm_last=_val;
//-----------------------------------------------

	#if CONFIG_SKY_IO_QMI_COMPANY_V1
	i = set_qmi_brightness(QMI_AD5247_ADDR_2E, _val);
	if(i != 0){
		i = set_qmi_brightness(QMI_AD5247_ADDR_5C, _val);
		if(i != 0){
			TW_MSG("Set Brightness fail !\n");
		}
	}
	#elif CONFIG_SKY_IO_QMI_COMPANY_V2
	i = set_qmi_brightness(QMI_AD5247_ADDR_5C, _val);
	if(i != 0){
		TW_MSG("Set Brightness fail !\n");
	}
	#else
	i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xe9, _val);	
	if(i != 0){
		TW_MSG("Set PWM fail !\n");
	}
	#endif //CONFIG_SKY_IO_QMI_COMPANY_V1 || CONFIG_SKY_IO_QMI_COMPANY_V2
}

/* -------------------------------------------------------------------------- */
void terawins_set_brightness(uint8 _val)
{
	int i = 0;
	//TW_MSG("\nBrightness !!\n");
	//-- Set pwm freq.
	i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0x69, _val);
	if(i != 0){
		TW_MSG("Set Brightness fail !\n");
	}
}

/* -------------------------------------------------------------------------- */
void terawins_set_contrast(uint8 _val)
{
	int i = 0;
	//TW_MSG("\nContrast !!\n");
	//-- Set pwm freq.
	#if CONFIG_SKY_IO_QMI_COMPANY_V1
	i = set_qmi_contrast(_val);
	#elif CONFIG_SKY_IO_QMI_COMPANY_V2
	TW_MSG("Not supported !\n");
	#else
	i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0x68, _val);
	#endif //CONFIG_SKY_IO_QMI_COMPANY_V1
	
	if(i != 0){
		TW_MSG("Set Contrast fail !\n");
	}
}

/* -------------------------------------------------------------------------- */
int init_terawins_tcon(void)
{
	int *p0, *p1;

	TW_MSG("\nInit Terawins T-CON !!\n");
	/* Set default commands to terawins T-CON. */
	//-- Setting page0 registers.
	p0 = TerawinsCmdTbl[TERAWINS_INIT * 2];
	terawins_set_page(TERAWINS_PAGE0_ADDR, p0);
	//-- Setting page1 registers.
	p1 = TerawinsCmdTbl[TERAWINS_INIT * 2 + 1];
	terawins_set_page(TERAWINS_PAGE1_ADDR, p1);
	return 0;
}


/*******************************************************************************
			Terawins OSD functions.
*******************************************************************************/
/* -------------------------------------------------------------------------- */
int terawins_set_osd_cmd(unsigned char cfg_idx, unsigned char cfg_data)
{
	int ret_val = 0;
	//-- set osd control register address.
	ret_val = terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_CFG_INDEX, cfg_idx);
	
	//-- set osd control register data.
	ret_val |= terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_CFG_DATA, cfg_data);

	if(ret_val != 0){
		TW_MSG("\nTeraWins set osd cmd error!\n");
	}
	
	return ret_val;
}

/* -------------------------------------------------------------------------- */
int terawins_set_osd_ram_addr(unsigned short ram_addr)
{
	int ret_val = 0;
	
	//-- set osd ram address.
	ret_val = terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_RAM_ADDR_LO, (unsigned char)(ram_addr & 0x00FF));
	ret_val |= terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_RAM_ADDR_HI, (unsigned char)((ram_addr & 0xFF00) >> 8));

	if(ret_val != 0){
		//ret_val = terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_RAM_ADDR_LO, (unsigned char)(ram_addr & 0x00FF));
		//ret_val |= terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_RAM_ADDR_HI, (unsigned char)((ram_addr & 0xFF00) >> 8));
	}
	return ret_val;
}

/* -------------------------------------------------------------------------- */
int terawins_set_osd_ram_data(unsigned char ram_data)
{
	int ret_val = 0;

	//-- set osd ram data.
	ret_val = terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_RAM_DATA, ram_data);

	if(ret_val != 0){
		//ret_val = terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_RAM_DATA, ram_data);
	}
	
	return ret_val;
}

/* -------------------------------------------------------------------------- */
void terawins_set_osd_alpha_blending(unsigned char mode, unsigned char _val)
{
	terawins_set_osd_cmd(ALPHA_BLENDING_CTRL, (((mode&0x03) << 6) | (_val&0x0f)));
}

/* -------------------------------------------------------------------------- */
void terawins_set_osd_blinking(unsigned char blk_freq, unsigned char duty)
{
	terawins_set_osd_cmd(BLINKING_CTRL, (((blk_freq&0x03) << 2) | (duty&0x03)));
}

/* -------------------------------------------------------------------------- */
void terawins_enable_osd(unsigned char _en)
{
	if(_en == 1){
		terawins_set_osd_cmd(OSD_CTRL, 0x80);
	}
	else{
		terawins_set_osd_cmd(OSD_CTRL, 0x00);
	}
}

/* -------------------------------------------------------------------------- */
void terawins_set_osd_line_data(uint16 _addr, uint8 *_str)
{
	uint8 i = 0;

	//-- Set start address of osd sram.
	terawins_set_osd_ram_addr(_addr);

	//-- Set osd data(64 bytes per line)
	for(i = 0; i < (2*WORDS_PER_ROW); i ++){
		terawins_set_osd_ram_data(_str[i]);
	}
}

/* -------------------------------------------------------------------------- */
void clear_osd_line(struct _terawins_osd_info* _osd)
{
	uint8 i = 0, j = 0;
	
	//-- Set start ram address in terawins sram.
	osd_line_data.ram_addr = _osd->ram_addr;

	for(i = 0; i < ((2*WORDS_PER_ROW) - 1); i ++){
		osd_line_data.str[i] = 0x00;
	}
	osd_line_data.str[i] = _osd->row_attr;

	for(j = 0; j < _osd->n_row; j ++){
		terawins_set_osd_line_data(osd_line_data.ram_addr, osd_line_data.str);
		osd_line_data.ram_addr += WORDS_PER_ROW;
	}
}

/* -------------------------------------------------------------------------- */
void clear_osd(int _osd_idx)
{
#if 1

#else
	unsigned int i;
	if(_osd_idx >= OI_ALL){
		for(i = 0; i < OI_ALL; i++){
			clear_osd_line(terawins_osd[i]);
		}
	}
	else{
		clear_osd_line(terawins_osd[_osd_idx]);
	}
#endif	
}

/* -------------------------------------------------------------------------- */
void terawins_set_osd_line_info(struct _terawins_osd_info* _osd)
{

}

/* -------------------------------------------------------------------------- */
void terawins_set_osd_line(char *_str, struct _terawins_osd_info *_info)
{
	unsigned char i = 0;
	int _ack = 0;


	for(i = 0; i < 30; i ++){
		if(_str[i] != 0){
			//-- low byte first.(char index.)
			_ack |= terawins_set_osd_ram_data(_str[i]);
			//-- high byte.(char attribute.)
			_ack |= terawins_set_osd_ram_data(_info->char_attr);
		}
		else{
			//-- low byte first.(char index.)
			_ack |= terawins_set_osd_ram_data(0x00);
			//-- high byte.(char attribute.)
			_ack |= terawins_set_osd_ram_data(0x00);
		}
	}

	//-- It must be the blank char at the 31th word of the line info.
	_ack |= terawins_set_osd_ram_data(00);
	_ack |= terawins_set_osd_ram_data(00);

	//-- Row attr.
	_ack |= terawins_set_osd_ram_data(00);
	_ack |= terawins_set_osd_ram_data(_info->row_attr);

	if(_ack != 0){
		TW_MSG("\nSet osd line error !!\n");
	}
}

/* -------------------------------------------------------------------------- */
void terawins_show_speed(char *_t)
{
#if 1
	char _osd[30] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
	uint8 i = 0;
	int j = 0;
	struct _terawins_osd_info *_info = &video_speed;

//	TW_MSG("\n--- show speed: %x %x %x ---\n", _info->ram_addr, _info->char_attr, _info->row_attr);

	if(_info->on == 0){
		return;
	}

	//--Set start ram address in terawins sram.
	j = terawins_set_osd_ram_addr(_info->ram_addr);

	if(j != 0){
		return;
	}
	while( (_t[i] != '\0') && (i < 30) ){
		_osd[i] = _t[i];
		i++;
	}
	terawins_set_osd_line(_osd, _info);
#else
	char _osd[30] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
	uint8 i = 0;
	int j = 0;
	struct _terawins_osd_info *_info = terawins_osd[OI_VIDEO_SPEED];	

	if(_info->on == 0){
		return;
	}

	//--Set start ram address in terawins sram.
	j = terawins_set_osd_ram_addr(_info->ram_addr);

	if(j != 0){
		return;
	}
	while( (_t[i] != '\0') && (i < 30) ){
		_osd[i] = _t[i];
		i++;
	}
	terawins_set_osd_line(_osd, _info);
#endif
}


/* -------------------------------------------------------------------------- */
void terawins_show_time(char *_t)
{
#if 1

#else
	char _osd[30] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
	uint8 i = 0;
	int j = 0;
	struct _terawins_osd_info *_info = terawins_osd[OI_VIDEO_TIME];		

	if(_info->on == 0){
		return;
	}

	//--Set start ram address in terawins sram.
	j = terawins_set_osd_ram_addr(_info->ram_addr);
	if(j != 0){
		return;
	}
	while( (_t[i] != '\0') && (i < 30) ){
		_osd[i] = _t[i];
		i++;
	}
	terawins_set_osd_line(_osd, _info);
#endif	
}


/* -------------------------------------------------------------------------- */
void terawins_show_subtitle(char _s[][31])
{
#if 1
	char _osd[30] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
	uint8 i = 0, j = 0;
	uint16 r_addr;
	struct _terawins_osd_info *_info = &subtitle;	

//	TW_MSG("\n--- show subtitle: %x %x %x ---\n", _info->ram_addr, _info->char_attr, _info->row_attr);

	if(_info->on == 0){
		return;
	}

	//--Set start ram address in terawins sram.
	terawins_set_osd_ram_addr(_info->ram_addr);

	for(i = 0; i < _info->n_row; i++){
		for( ; j > 0; j--){
			_osd[j] = 0;
		}
		j = 0;
		while( (_s[i][j] != '\0') && (j < 30) ){
			_osd[j] = _s[i][j];
			j++;
		}
		terawins_set_osd_line(_osd, _info);
	}
#else
	char _osd[30] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
	uint8 i = 0, j = 0;
	uint16 r_addr;
	struct _terawins_osd_info *_info = terawins_osd[OI_SUBTITLE];	

	if(_info->on == 0){
		return;
	}

	//--Set start ram address in terawins sram.
	terawins_set_osd_ram_addr(_info);

	for(i = 0; i < terawins_osd[OI_SUBTITLE]->n_row; i++){
		for( ; j > 0; j--){
			_osd[j] = 0;
		}
		j = 0;
		while( (_s[i][j] != '\0') && (j < 30) ){
			_osd[j] = _s[i][j];
			j++;
		}
		terawins_set_osd_line(_osd, _info);
	}
#endif
}


/* -------------------------------------------------------------------------- */
void terawins_show_mute_vol(uint32 _idx)
{
#if 1

#else
	char _osd[30] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
	unsigned int i, start, end;
	struct _terawins_osd_info *_info = terawins_osd[OI_MUTE_VOL];

	if(_info->on == 0){
		return;
	}

	//--Set start ram address in terawins sram.
	terawins_set_osd_ram_addr(_info);

	if(_idx == OSD_MUTE){
		start = MUTE_START;
		for(i = 0; i < MUTE_NUM; i++){
			_osd[start ++] = mute_icon[i];
		}
	}
	else{
		start = VOL_START;
		for(i = 0; i < VOL_NUM; i ++){
			_osd[start ++] = vol_icon[i];
		}
		for(i = 0; i < _idx; i ++){
			_osd[start ++] = FONT_VOL_BAR;
		}
		end = (TOTAL_VOL_BAR - _idx);
		for(i = 0; i < end; i ++){
			_osd[start ++] = FONT_VOL_EMPTY_BAR;
		}
	}
	
	terawins_set_osd_line(_osd, _info);
#endif	
}

/* -------------------------------------------------------------------------- */
void terawins_show_icon(uint32 _idx)
{
#if 1

#else
	char _osd[30] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
	unsigned int i, j, k, h_end, v_end, h_start;
	char *_p;
	uint16 r_addr;
	struct _terawins_osd_info *_info = terawins_osd[OI_ICON];	

	if(_info->on == 0){
		return;
	}

	//--Set start ram address in terawins sram.
	terawins_set_osd_ram_addr(_info);
	
	if(_idx == 0x00){
		_p = valid_icon;
		_info->f_color = FG_COLOR12;
	}
	else if(_idx == 0x01){
		_p = invalid_icon;
		_info->f_color = FG_COLOR10;
	}
	else{
		TW_MSG("\nInput icon num error !\n");
		return;
	}
	//-- Set ICON color
	terawins_set_osd_line_info(_info);	
	
	k = 0;
	for(j = 0; j < v_end; j ++){
		for(i = 0 ; i < h_end; i ++){
			_osd[(h_start + i)] = _p[k ++];
		}
		terawins_set_osd_line(_osd, _info);
	}
#endif	
}

/* -------------------------------------------------------------------------- */
void terawins_clear_icon(void)
{
	//clear_osd(OI_ICON);	
}

/* -------------------------------------------------------------------------- */
void terawins_clear_subtitle(void)
{
//	TW_MSG("\n--- clear_subtitle ---\n");
	clear_osd_line(&subtitle);
	//clear_osd(OI_SUBTITLE);
}

/* -------------------------------------------------------------------------- */
void terawins_clear_mute_vol(void)
{
	//clear_osd(OI_MUTE_VOL);
}

/* -------------------------------------------------------------------------- */
void terawins_clear_time(void)
{
	//clear_osd(OI_VIDEO_TIME);
}

/* -------------------------------------------------------------------------- */
void terawins_clear_speed(void)
{
//	TW_MSG("\n--- clear_speed ---\n");
	clear_osd_line(&video_speed);
	//clear_osd(OI_VIDEO_SPEED);
}


/* -------------------------------------------------------------------------- */
void terawins_clear_all_osd_item(void)
{
	//clear_osd(OI_ALL);
}

/* -------------------------------------------------------------------------- */
void calculate_start_line(void)
{

}

/* -------------------------------------------------------------------------- */
uint8 calculate_start_row(struct _terawins_osd_info* _osd[])
{

}

/* -------------------------------------------------------------------------- */
void terawins_set_time_on_off(uint32 _on)
{

}

/* -------------------------------------------------------------------------- */
char terawins_get_time_on_off(void)
{
	//return video_time.on;
	return 0;
}

/* -------------------------------------------------------------------------- */
void init_osd_info(void)
{

}

/* -------------------------------------------------------------------------- */
void init_terawins_osd(void)
{

}

/*----------------------------------------------------------------------------*/
void terawins_parse_resolution(unsigned long _w, unsigned long _h)
{
	unsigned char tcon_dim = 0;			// define terawins t-con output resolution.

//	TW_MSG("\n--- tcon parse resolution ---\n");
	
	if(_w <= 640){
	//-- width(640)
		if(_h <= 480){
			//-- 640x480
			tcon_dim = TERAWINS_640x480;
		}
		else if(_h > 576){
			//-- 800x600
			tcon_dim = TERAWINS_800x600;
		}
		else{
			//-- 720x576
			tcon_dim = TERAWINS_720x576;
		}
	}
	else if(_w > 720){
		//-- 800x600
		tcon_dim = TERAWINS_800x600;
	}
	else{
	//-- width(720)
		if(_h <= 480){
			//-- 720x480
			tcon_dim = TERAWINS_720x480;
		}
		else if(_h > 576){
			//-- 800x600
			tcon_dim = TERAWINS_800x600;
		}
		else{
			//-- 720x576
			tcon_dim = TERAWINS_720x576;
		}
	}
	terawins_set_resolution(tcon_dim);
}


/* -------------------------------------------------------------------------- */
void test_terawins(unsigned int _val)
{

#if 0	//-- Test terawins command
	unsigned int i;
	//-- Set cmd.
	i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, (uint8)((_val&0xff00)>>8), (uint8)(_val&0x00ff));	
#endif


#if 0	//--Test the ICON
	if(_val == 0x00){
		terawins_show_icon(0);
	}
	else if(_val == 0x01){		// show icon
		terawins_show_icon(1);
	}
	else{		// clear icon
		terawins_clear_icon();
	}
#endif

#if 0	//-- Test pwm on/off
	if(_val == 0){
		TW_MSG("0ff");
		terawins_pwm_on_off(_val);
	}
	else if(_val == 1){
		TW_MSG("0n");
		terawins_pwm_on_off(_val);
	}
	else{
		TW_MSG("Retry !!");
	}
#endif


#if 0	//-- Switching tables.
	if(_val == 0){
		terawins_set_resolution(TERAWINS_640x480);
	}
	else if(_val == 1){
		terawins_set_resolution(TERAWINS_720x480);
	}
	else if(_val == 2){
		terawins_set_resolution(TERAWINS_720x576);
	}
	else if(_val == 3){
		terawins_set_resolution(TERAWINS_800x600);
	}
	else{
		TW_MSG("Not Support !");
	}
#endif


}

#endif	//-- #if CONFIG_TERAWINS_TCON

