/*
 *	terawins_tcon.h - Register and variable definitions for TeraWins T-CON.
 */

#ifndef _TERAWINS_TCON_H_
#define _TERAWINS_TCON_H_

//** Debug
#define TERAWINS_DBG
#ifdef TERAWINS_DBG
#define	TW_MSG(format, arg...)		printk(format, ## arg)
#else
#define TW_MSG
#endif

//** Terawins T-CON device address.
#define TERAWINS_PAGE0_ADDR		0x40
#define TERAWINS_PAGE1_ADDR		0x42

enum TerawinsDim
{
	TERAWINS_INIT,
	TERAWINS_640x480,
	TERAWINS_720x480,
	TERAWINS_720x576,
	TERAWINS_800x600,
	TERAWINS_END,
};

#define SCALE_TABLE_NUM				32

//** GPIO function
#define GIO_BASE_ADDR				0xC1FC0000
#define EGIO_BASE_ADDR				0xC1FC0008

#define EGIO_W32(reg, val32)		(*((volatile unsigned long *)(EGIO_BASE_ADDR + reg)) = val32)
#define EGIO_R32(reg)				(*((volatile unsigned long *)(EGIO_BASE_ADDR + reg)))

#define GIO_W32(reg, val32)			(*((volatile unsigned long *)(GIO_BASE_ADDR + reg)) = val32)
#define GIO_R32(reg)				(*((volatile unsigned long *)(GIO_BASE_ADDR + reg)))

/* Definitions for I2C function. */
typedef enum{
	GPIO_DIR = 0x0,
	GPIO_DATA = 0x4,
}SKY_GPIO_REG;

#if 1
#define I2C_SCL_MASK				(0x00000001<<CONFIG_SKY_I2C_SCL)
#define I2C_SDA_MASK				(0x00000001<<CONFIG_SKY_I2C_SDA)

#else
//-- Select EVB board.
#if CONFIG_SOUND_PLATFORM == 0
#define I2C_SCL_MASK				0x00000010		// GPIO 4
#define I2C_SDA_MASK				0x00000020		// GPIO 5
#else
#define I2C_SCL_MASK				0x00000001		// GPIO 0
#define I2C_SDA_MASK				0x00000002		// GPIO 1
#endif

#endif

//-- SCLK
#define SCL_INPUT \
{\
	vtval = GIO_R32(GPIO_DIR);\
	vtval = GIO_R32(GPIO_DIR);\
	vtval &= (~I2C_SCL_MASK);\
	GIO_W32(GPIO_DIR, vtval);\
}
#define SCL_OUTPUT \
{ \
	vtval = GIO_R32(GPIO_DIR);\
	vtval = GIO_R32(GPIO_DIR);\
	vtval |= (I2C_SCL_MASK);\
	GIO_W32(GPIO_DIR, vtval);\
}
#define SCL_HI \
{ \
	vtval = GIO_R32(GPIO_DATA);\
	vtval = GIO_R32(GPIO_DATA);\
	vtval |= (I2C_SCL_MASK);\
	GIO_W32(GPIO_DATA, vtval);\
}
#define SCL_LO \
{ \
	vtval = GIO_R32(GPIO_DATA);\
	vtval = GIO_R32(GPIO_DATA);\
	vtval &= (~I2C_SCL_MASK);\
	GIO_W32(GPIO_DATA, vtval);\
}
#define SCL \
{ \
	vtval = GIO_R32(GPIO_DATA);\
	vtval = GIO_R32(GPIO_DATA);\
	vtval &= (I2C_SCL_MASK);\
}

//-- SDA
#define SDA_INPUT \
{\
	vtval = GIO_R32(GPIO_DIR);\
	vtval = GIO_R32(GPIO_DIR);\
	vtval &= (~I2C_SDA_MASK);\
	GIO_W32(GPIO_DIR, vtval);\
}
#define SDA_OUTPUT \
{ \
	vtval = GIO_R32(GPIO_DIR);\
	vtval = GIO_R32(GPIO_DIR);\
	vtval |= I2C_SDA_MASK;\
	GIO_W32(GPIO_DIR, vtval);\
}
#define SDA_HI \
{ \
	vtval = GIO_R32(GPIO_DATA);\
	vtval = GIO_R32(GPIO_DATA);\
	vtval |= I2C_SDA_MASK;\
	GIO_W32(GPIO_DATA, vtval);\
}
#define SDA_LO \
{ \
	vtval = GIO_R32(GPIO_DATA);\
	vtval = GIO_R32(GPIO_DATA);\
	vtval &= (~I2C_SDA_MASK);\
	GIO_W32(GPIO_DATA, vtval);\
}
#define SDA \
{ \
	vtval = GIO_R32(GPIO_DATA);\
	vtval = GIO_R32(GPIO_DATA);\
	vtval &= (I2C_SDA_MASK);\
}

#define QMI_AD5247_ADDR_2E	0x2e			//IIC address
#define QMI_AD5247_ADDR_5C    0x5c			//IIC address

#if CONFIG_SKY_IO_QMI_COMPANY_V1
#define QMI_MUTE_MASK         0x00000008     //BIT3  for mute pin // 0: Mute , 1: UnMute
#define QMI_BL_MASK           0x00000008     //BIT3 for QMI/PF115 backlight
#define QMI_CT1_MASK          0x00002000     //BIT13 for blue led1
#define QMI_CT2_MASK          0x00004000     //BIT14 for blue led2
#define QMI_LCD_MASK          0x00008000     //BIT15 for lcd on/off
#define QMI_BLACK_WHITE_MASK  0x00001002     //BIT1 and BIT12
#endif //CONFIG_SKY_IO_QMI_COMPANY_V1

#if CONFIG_SKY_IO_QMI_COMPANY_V2
#define QMI_BL_MASK           0x00008000     //BIT15 for QMI/PF151 backlight
#endif //CONFIG_SKY_IO_QMI_COMPANY_V2

#if (CONFIG_SKY_IO_QMI_COMPANY_V1 || CONFIG_SKY_IO_QMI_COMPANY_V2)
#define QMI_BL_INPUT \
{\
     vtval = GIO_R32(GPIO_DIR); \
	vtval = GIO_R32(GPIO_DIR); \
	vtval &= (~QMI_BL_MASK); \
	GIO_W32(GPIO_DIR, vtval); \
}

#define QMI_BL_OUTPUT \
{\
     vtval = GIO_R32(GPIO_DIR); \
	vtval = GIO_R32(GPIO_DIR); \
	vtval |= (QMI_BL_MASK); \
	GIO_W32(GPIO_DIR, vtval); \
}

#define QMI_BL_HI \
{\
     vtval = GIO_R32(GPIO_DATA); \
     vtval = GIO_R32(GPIO_DATA); \
	vtval |= (QMI_BL_MASK); \
	GIO_W32(GPIO_DATA, vtval); \
}

#define QMI_BL_LO \
{\
     vtval = GIO_R32(GPIO_DATA); \
     vtval = GIO_R32(GPIO_DATA); \
	vtval &= (~QMI_BL_MASK); \
	GIO_W32(GPIO_DATA, vtval); \
}
#endif //CONFIG_SKY_IO_QMI_COMPANY_V1 || CONFIG_SKY_IO_QMI_COMPANY_V2

#if CONFIG_SKY_IO_QMI_COMPANY_V1
#define QMI_CT1_INPUT \
{\
     vtval = GIO_R32(GPIO_DIR); \
	vtval = GIO_R32(GPIO_DIR); \
	vtval &= (~QMI_CT1_MASK); \
	GIO_W32(GPIO_DIR, vtval); \
}

#define QMI_CT1_OUTPUT \
{\
     vtval = GIO_R32(GPIO_DIR); \
	vtval = GIO_R32(GPIO_DIR); \
	vtval |= (QMI_CT1_MASK); \
	GIO_W32(GPIO_DIR, vtval); \
}

#define QMI_CT1_HI \
{\
     vtval = GIO_R32(GPIO_DATA); \
     vtval = GIO_R32(GPIO_DATA); \
	vtval |= (QMI_CT1_MASK); \
	GIO_W32(GPIO_DATA, vtval); \
}

#define QMI_CT1_LO \
{\
     vtval = GIO_R32(GPIO_DATA); \
     vtval = GIO_R32(GPIO_DATA); \
	vtval &= (~QMI_CT1_MASK); \
	GIO_W32(GPIO_DATA, vtval); \
}

#define QMI_CT2_INPUT \
{\
     vtval = GIO_R32(GPIO_DIR); \
	vtval = GIO_R32(GPIO_DIR); \
	vtval &= (~QMI_CT2_MASK); \
	GIO_W32(GPIO_DIR, vtval); \
}

#define QMI_CT2_OUTPUT \
{\
     vtval = GIO_R32(GPIO_DIR); \
	vtval = GIO_R32(GPIO_DIR); \
	vtval |= (QMI_CT2_MASK); \
	GIO_W32(GPIO_DIR, vtval); \
}

#define QMI_CT2_HI \
{\
     vtval = GIO_R32(GPIO_DATA); \
     vtval = GIO_R32(GPIO_DATA); \
	vtval |= (QMI_CT2_MASK); \
	GIO_W32(GPIO_DATA, vtval); \
}

#define QMI_CT2_LO \
{\
     vtval = GIO_R32(GPIO_DATA); \
     vtval = GIO_R32(GPIO_DATA); \
	vtval &= (~QMI_CT2_MASK); \
	GIO_W32(GPIO_DATA, vtval); \
}

#define QMI_LCD_INPUT \
{\
     vtval = GIO_R32(GPIO_DIR); \
	vtval = GIO_R32(GPIO_DIR); \
	vtval &= (~QMI_LCD_MASK); \
	GIO_W32(GPIO_DIR, vtval); \
}

#define QMI_LCD_OUTPUT \
{\
     vtval = GIO_R32(GPIO_DIR); \
	vtval = GIO_R32(GPIO_DIR); \
	vtval |= (QMI_LCD_MASK); \
	GIO_W32(GPIO_DIR, vtval); \
}

#define QMI_LCD_HI \
{\
     vtval = GIO_R32(GPIO_DATA); \
     vtval = GIO_R32(GPIO_DATA); \
	vtval |= (QMI_LCD_MASK); \
	GIO_W32(GPIO_DATA, vtval); \
}

#define QMI_LCD_LO \
{\
     vtval = GIO_R32(GPIO_DATA); \
     vtval = GIO_R32(GPIO_DATA); \
	vtval &= (~QMI_LCD_MASK); \
	GIO_W32(GPIO_DATA, vtval); \
}
#endif //CONFIG_SKY_IO_QMI_COMPANY_V1
#if CONFIG_SKY_IO_QMI_COMPANY_V3 //Mel port for SPK pop noise
#define QMI_MUTE_MASK         0x00000008     //BIT3  for mute pin // 0: Mute , 1: UnMute
#define QMI_MUTE_OUTPUT \
{\
     vtval = GIO_R32(GPIO_DIR); \
	vtval = GIO_R32(GPIO_DIR); \
	vtval |= (QMI_MUTE_MASK); \
	GIO_W32(GPIO_DIR, vtval); \
}

#define QMI_MUTE_HI \
{\
     vtval = GIO_R32(GPIO_DATA); \
     vtval = GIO_R32(GPIO_DATA); \       
	vtval |= (QMI_MUTE_MASK); \
	GIO_W32(GPIO_DATA, vtval); \
}

#define QMI_MUTE_LO \
{\
     vtval = GIO_R32(GPIO_DATA); \
     vtval = GIO_R32(GPIO_DATA); \
	vtval &= (~QMI_MUTE_MASK); \
	GIO_W32(GPIO_DATA, vtval); \
}
#endif

#if 1	// 600 KHz
#define TERAWINS_TIME_1 			1
#define TERAWINS_TIME_2 			1
#else	
#define TERAWINS_TIME_1 			100 // 200 // 82KHz
#define TERAWINS_TIME_2 			100
#endif

/* Register definitions for TeraWins OSD control. */
enum{
	OSD_CFG_INDEX 		= 0xA0,
	OSD_CFG_DATA 		= 0xA1,
	OSD_RAM_ADDR_LO 	= 0xA2,
	OSD_RAM_ADDR_HI 	= 0xA3,
	OSD_RAM_DATA 		= 0xA4,
};

enum TERAWINS_OSD_CMD{
	OSD_CTRL			= 0x00,
	CHAR_DLY1			= 0x01,
	CHAR_DLY2			= 0x02,
	CHAR_DLY3			= 0x03,
	CHAR_FONT_SIZE		= 0x04,
	CHAR_RAM_START_ADDR	= 0x05,
	CHAR_BS_CTRL		= 0x06,
	CHAR_BS_COLOR		= 0x07,
	CHAR_HEIGHT_SCALING	= 0x08,
	BLINKING_CTRL		= 0x09,
	BMAP_WIN_SIZE_UPPER	= 0x0A,
	BMAP_WIN_SIZE_WIDTH	= 0x0B,
	BMAP_WIN_SIZE_HEIGHT= 0x0C,	
	BMAP_DOT_ENLARGE	= 0x0D,
	OSD_COLOR_LUT_PORT	= 0x0E,
	CHAR_RAM_ACCESS_CTRL= 0x0F,
	W1_START_ROW_NUM	= 0x10,
	W1_END_ROW_NUM		= 0x11,
	W1_START_COL_NUM	= 0x12,
	W1_END_COL_NUM		= 0x13,	
	W1_SHADOW_SIZE		= 0x14,
	CHAR2BP_IDX_BASE	= 0x19,
	CHAR2BP_BASEADDR_LO	= 0x1A,	
	CHAR2BP_BASEADDR_HI	= 0x1B,	
	ALPHA_BLENDING_CTRL	= 0x1C,	
	REVISION_ID			= 0x1D,
	CHAR_RAM_STOP_ADDR	= 0x1E,
};

struct _terawins_reg
{
	unsigned char osd_ctrl;
	unsigned char dly1;
	unsigned char dly2;
	unsigned char dly3;
	unsigned char font_size;
	unsigned char ram_start;
	unsigned char bs_ctrl;
	unsigned char bs_color;	
	
	unsigned char h_scaling;
	unsigned char blink_ctrl;
	unsigned char bmap_u;
	unsigned char bmap_w;
	unsigned char bmap_h;
	unsigned char bmap_large;
	unsigned char lut_port;
	unsigned char acc_ctrl;	

	unsigned char start_row;
	unsigned char end_row;
	unsigned char start_col;
	unsigned char end_col;
	unsigned char sha_size;
	unsigned char c2bp_idx;
	unsigned char c2bp_addrlo;
	unsigned char c2bp_addrhi;	

	unsigned char alpha_ctrl;	
	unsigned char rev_id;
	unsigned char ram_stop;	
};


/* Data structure for OSD info. */
struct _terawins_line_info
{
	int *char_idx;
	unsigned char char_attr;		// bit[7:5]=BG, bit[4]=blink, bit[3:0]=FG, 
	unsigned char row_attr;			// bit[7]=RGAP_BG(1:show background color, 0:transparent),
									// bit[6:2]=RGAP (Row gap/spacing heigh (4*RGAP).)
									// bit[1]=CHS(char height be double)
									// bit[0]=CWS(char width be double)
};

struct _terawins_window_info
{
	unsigned char row_start;
	unsigned char row_end;
	unsigned char col_start;
	unsigned char col_end;
	unsigned char shadow_size;
};

struct _terawins_menu_info
{
	unsigned short h_start;
	unsigned short v_start;
	unsigned char row_index;
	unsigned char num_rows;
};

//-- Flowbee 2007-12-13
struct _terawins_osd_info
{
	char on;							// osd on
	char n_row;							// num of row
	unsigned short s_line;				// start line of the frame (multiples of 16)
	char r_num;							// row num of the osd area
	unsigned short ram_addr;			// ram address
	char blink;							// blink
	char f_color;						// foreground color
	char b_color;						// background color
	char d_line;						// double size of line	
	unsigned char char_attr;
	unsigned char row_attr;
};

enum{
	FONT_10x16 = 0,
	FONT_12x18,
	FONT_16x24,
	FONT_END,
};

struct _font_info
{
	unsigned long font_size;	// register val
	unsigned char width;		// font width
	unsigned char height;		// font height
	unsigned char font_nums;	// total font nums
};

#define TERAWINS_OSD_RAM_SIZE				(6*1024)	// byte
#define CHAR_PER_LINE						30
#define BYTES_PER_LINE_INFO					64

#define FONT_WIDTH							font_info[font_index].width
#define FONT_HEIGHT							font_info[font_index].height
#define FONT_WIDTH_BYTES					((FONT_WIDTH + 7) / 8)

#define CHAR_START_ADDR(num_of_font)		( (((((FONT_WIDTH_BYTES*FONT_HEIGHT*num_of_font)/32)+1)*32)+63)/64 )
#define CHAR_START_RAM_ADDR(num_of_font)	( CHAR_START_ADDR(num_of_font) * 32 )


/* Define the Foreground color and Background color. */
enum FG_color{
	FG_COLOR0 = 0,
	FG_COLOR1,
	FG_COLOR2,
	FG_COLOR3,
	FG_COLOR4,
	FG_COLOR5,
	FG_COLOR6,
	FG_COLOR7,
	FG_COLOR8,
	FG_COLOR9,
	FG_COLOR10,
	FG_COLOR11,
	FG_COLOR12,
	FG_COLOR13,
	FG_COLOR14,
	FG_COLOR15,	
	FG_MAX,
};

enum BG_color{
	BG_COLOR0 = 0,
	BG_COLOR1,
	BG_COLOR2,
	BG_COLOR3,
	BG_COLOR4,
	BG_COLOR5,
	BG_COLOR6,
	BG_COLOR7,
	BG_MAX,
};

//-- OSD define
/* -------------------------------------------------------------------------- */
#define WORDS_PER_ROW				32
#define FONTS_PER_ROW				30
#define WIDTH_PER_ROW				(FONTS_PER_ROW * FONT_WIDTH)

#define LCD_W						displays[CONFIG_ARCPGU_DISPTYPE].xres
#define LCD_H						displays[CONFIG_ARCPGU_DISPTYPE].yres

#if (CONFIG_ARCPGU_DISPTYPE == 3)
	#define TERAWINS_W				2176
#elif (CONFIG_ARCPGU_DISPTYPE == 4)
	#define TERAWINS_W				1500
#elif (CONFIG_ARCPGU_DISPTYPE == 5)
	#define TERAWINS_W				1500
#elif (CONFIG_ARCPGU_DISPTYPE == 6)
	#define TERAWINS_W				3456	
#else
	#define TERAWINS_W				1500
#endif

#define LCD_START_X					((LCD_W - WIDTH_PER_ROW) / 2)
#define TERAWINS_START_X			((TERAWINS_W * LCD_START_X) / LCD_W)

enum OSDIndex 
{
	OI_VIDEO_SPEED,
	OI_ICON,
	OI_SUBTITLE,
	OI_MUTE_VOL,
	OI_VIDEO_TIME,
	OI_ALL,
};


//-- Define ICON
#define ICON_START					13
#define ICON_NUM					3

//-- Define MUTE and VOL
#define MUTE_START					13
#define MUTE_NUM					4

#define VOL_START					8
#define VOL_NUM						4

#define FONT_VOL_BAR				0x12	// font index
#define FONT_VOL_EMPTY_BAR			0x13	// font index
#define TOTAL_VOL_BAR				10


/* Function declarations. */
int terawins_set_cmd(uint8 dev_addr, uint8 reg_addr, uint8 reg_val);
int terawins_set_osd_cmd(uint8 cfg_idx, uint8 cfg_data);
int terawins_set_osd_ram_addr(uint16 ram_addr);
int terawins_set_osd_ram_data(uint8 ram_data);
void terawins_set_osd_line_data(uint16 ram_addr, uint8 *str);
int init_terawins_tcon(void);
void init_terawins_osd(void);
void terawins_on_off(uint8 _on);
int terawins_set_resolution(uint8 idx);
void terawins_pwm_on_off(uint8 _on);
void terawins_backlight_on_off(uint8 _on);
void terawins_set_pwm(uint8 _val);
void terawins_set_brightness(uint8 _val);
void terawins_set_contrast(uint8 _val);
void terawins_set_osd_alpha_blending(uint8 mode, uint8 val);
void terawins_set_osd_blinking(uint8 blk_freq, uint8 duty);
void terawins_enable_osd(uint8 _en);
void terawins_parse_resolution(unsigned long _w, unsigned long _h);

void terawins_show_mute_vol(uint32 idx);
void terawins_clear_mute_vol(void);
void terawins_show_icon(uint32 _idx);
void terawins_clear_icon(void);
void terawins_show_subtitle(char _s[][31]);
void terawins_clear_subtitle(void);
void terawins_clear_all_osd_item(void);
void terawins_show_time(char *_t);
void terawins_clear_time(void);
void terawins_show_speed(char *_t);
void terawins_clear_speed(void);
void terawins_set_time_on_off(uint32 _on);
char terawins_get_time_on_off(void);

#if (CONFIG_SKY_IO_QMI_COMPANY_V1 || CONFIG_SKY_IO_QMI_COMPANY_V2)
int set_qmi_brightness(uint8 dev_addr, uint8 reg_val);
#endif 

#if CONFIG_SKY_IO_QMI_COMPANY_V1
void get_gpio_status(uint32* gpio);
void set_lcd_on_off(uint8 lcd_on_off);
int set_qmi_contrast(uint8 reg_val);
#endif

void test_terawins(unsigned int _val);

#endif

