/************************************************************************
 *		This file include primitive functions which can control the
 *		TeraWins T-CON.
 ************************************************************************/
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/tty.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/ioport.h>
/* Flowbee@960327 */
#include <linux/interrupt.h>
#include <asm/arcregs.h>
#include <asm/uaccess.h>

#include "arc_pgu.h"
#include "terawins_tcon.h"


#define TERAWINS_DBG
#ifdef TERAWINS_DBG
#define	TW_MSG					printk//printk(KERNEL_INFO "FB: " m "\n")
#else
#define TW_MSG
#endif

//-- Global variable
static unsigned long val = 0;
static volatile unsigned long vtval; 	
unsigned long tval;

/*******************************************************************************
		Terawins T-CON
*******************************************************************************/
#if CONFIG_TERAWINS_TCON


//#define CONFIG_54MHz			(1)
/* 10" panel (800x480(37M) => 1024x600) */
#if (CONFIG_ARCPGU_DISPTYPE == 3)
//-- 16:9
int TerawinsInitCmdPage0[][2]=
{	/* Addr, Val */
    {  0x30, 0x42  },
    {  0x31, 0x70  },
    {  0x32, 0x3f  },
    {  0x33, 0xbf  },
    {  0x34, 0x00  },
    {  0x35, 0xbc  },
    {  0x36, 0x34  },
    {  0x37, 0x40  },
    {  0x38, 0x00  },
    {  0x39, 0x10  },
    {  0x3a, 0x00  },
    {  0x3b, 0x00  },
    {  0x3c, 0x00  },
    {  0x3d, 0x00  },
    {  0x3e, 0x00  },
    {  0x3f, 0x10  },

    {  0x50, 0x00  },
   
    {  0x60, 0x00  },
    {  0x61, 0x88  },
    {  0x62, 0x18  },
    {  0x63, 0x04  },//High Pass Filter   {  0x63, 0x0f  }, //mod by Eric 2007-09-20
    {  0x64, 0x04  },//Low Pass Filter    {  0x64, 0x04  },
    {  0x65, 0x08  },
    {  0x66, 0x08  },
    {  0x67, 0x1e  },
	{  0x68, 0x7d  },//Contrast    {  0x68, 0x55  },//pwm
    {  0x69, 0x81  },//Bright    {  0x69, 0x73  },//    {  0x69, 0x72  },
    {  0x6a, 0x00  },
    {  0x6b, 0x7f  },
    {  0x6c, 0x80  },
    {  0x6d, 0x00  },
    {  0x6e, 0x10  },
    {  0x6f, 0x00  },
    
    {  0x72, 0x9f  },//    {  0x72, 0x00  },
    {  0x73, 0x18  },//    {  0x73, 0x19  },
    {  0x74, 0x99  },
    {  0x75, 0x19  },
    {  0x76, 0x00  },
    {  0x77, 0x00  },
    {  0x78, 0x00  },
    {  0x79, 0x00  },
    {  0x7a, 0x00  },
    {  0x7b, 0x00  },
    {  0x7c, 0x00  },
    {  0x7d, 0x10  },
    {  0x7e, 0x80  },
    {  0x7f, 0x80  },

	/* Flowbee@960608. */
    {  0x85, 0x5f  },
    
    {  0x90, 0x04  },//Dither_video    {  0x90, 0x05  },//Still    {  0x90, 0x04  },
    {  0x91, 0x07  },
    {  0x92, 0x00  },
    {  0x93, 0x00  },
    {  0x94, 0x00  },
    {  0x95, 0x00  },
    {  0x96, 0x00  },
    {  0x97, 0x00  },
    {  0x98, 0x00  },
    {  0x99, 0x02  },
    {  0x9a, 0x40  },
    {  0x9b, 0x00  },
    {  0x9c, 0x00  },
    {  0x9d, 0x10  },
    {  0x9e, 0x80  },
    {  0x9f, 0x80  },

    {  0xb0, 0x20  },
    {  0xb1, 0x00  },
    {  0xb2, 0x0f  },
    {  0xb3, 0x00  },
    {  0xb4, 0x00  },
    {  0xb5, 0x04  },
    {  0xb6, 0x58  },
    {  0xb7, 0x02  },
    {  0xb8, 0x3a  },
    {  0xb9, 0x04  },
    {  0xba, 0x58  },
    {  0xbb, 0x02  },
    {  0xbc, 0x20  },
    {  0xbd, 0x00  },
    {  0xbe, 0x04  },
    {  0xbf, 0x00  },

	{  0xc0, 0x01  },//RGB666 
    {  0xc1, 0x08  },//    {  0xc1, 0x00  },
    {  0xc2, 0x00  },
    {  0xc3, 0x00  },
    {  0xc4, 0x00  },
    {  0xc5, 0x60  },
    {  0xc6, 0x05  },
    {  0xc7, 0x00  },
    {  0xc8, 0x45  },
    {  0xc9, 0x03  },
    {  0xca, 0x02  },
    {  0xcb, 0x00  },
    {  0xcc, 0x00  },
    {  0xcd, 0x00  },
    {  0xce, 0x00  },
    {  0xcf, 0x00  },
    
    {  0xd0, 0x00  },
    {  0xd1, 0x00  },
    {  0xd2, 0x00  },
    {  0xd3, 0x00  },
    {  0xd4, 0x00  },
    {  0xd5, 0x00  },
    {  0xd6, 0x00  },
    {  0xd7, 0x00  },
    {  0xd8, 0x00  },
    {  0xd9, 0x00  },
    {  0xda, 0x00  },
    {  0xdb, 0x00  },
    {  0xdc, 0x00  },
    {  0xdd, 0x04  },
    {  0xde, 0x58  },
    {  0xdf, 0x02  },
    
    {  0xe0, 0xb0  },
    {  0xe1, 0xc0  },
    {  0xe2, 0x00  },
    {  0xe3, 0x00  },
    {  0xe4, 0x00  },
    {  0xe5, 0x00  },
    {  0xe6, 0x07  },
    {  0xe7, 0x80  },
    {  0xe8, 0x00  },
    {  0xe9, 0x7d  },
    {  0xea, 0x07  },
    {  0xeb, 0x80  },
    {  0xec, 0xc0  },
    {  0xed, 0x80  },
    {  0xee, 0x00  },
    {  0xef, 0x00  },
	{  END_CMD, END_CMD  },		
};

int TerawinsInitCmdPage1[][2]=
{	/* Addr, Val */
    {  0x20, 0x00  },
    {  0x21, 0xff  },
    {  0x22, 0x03  },
    {  0x23, 0x00  },
    {  0x24, 0x21  },
    {  0x25, 0x00  },
    {  0x26, 0x00  },
    {  0x27, 0x00  },
    {  0x28, 0x00  },
    {  0x29, 0x00  },
    {  0x2a, 0x00  },
    {  0x2b, 0x01  },
    {  0x2c, 0x12  },
    {  0x2d, 0x05  },
    {  0x2e, 0x68  },
    {  0x2f, 0x01  },
    
    {  0x30, 0x00  },
    {  0x31, 0x0f  },
    {  0x32, 0x00  },
    {  0x33, 0x0f  },
    {  0x34, 0x00  },
    {  0x35, 0x00  },
    {  0x36, 0x00  },
    {  0x37, 0x00  },
    {  0x38, 0x00  },
    {  0x39, 0x00  },
    {  0x3a, 0x00  },
    {  0x3b, 0x00  },
    {  0x3c, 0x00  },
    {  0x3d, 0x00  },
    {  0x3e, 0x00  },
    {  0x3f, 0x00  },

    {  0xb0, 0x95  },
    {  0xb1, 0x00  },
    {  0xb2, 0xcc  },
    {  0xb3, 0x95  },
    {  0xb4, 0x32  },
    {  0xb5, 0x68  },
    {  0xb6, 0x95  },
    {  0xb7, 0x81  },
    {  0xb8, 0xc0  },
    {  0xb9, 0x00  },
    {  0xba, 0x00  },
    {  0xbb, 0x00  },
    {  0xbc, 0x00  },
    {  0xbd, 0x00  },
    {  0xbe, 0x00  },
    {  0xbf, 0x00  },

    {  0xd0, 0x30  },
    {  0xd1, 0x00  },
    {  0xd2, 0x10  },
    {  0xd3, 0x01  },
    {  0xd4, 0x01  },
    {  0xd5, 0x4c  },//    {  0xd5, 0x49  },
    {  0xd6, 0x00  },
    {  0xd7, 0x20  },
    {  0xd8, 0x03  },
    {  0xd9, 0x2a  },
    {  0xda, 0x00  },
    {  0xdb, 0xe0  },
    {  0xdc, 0x01  },
    {  0xdd, 0x00  },
    {  0xde, 0x03  },//    {  0xde, 0x02  },
    {  0xdf, 0x4c  },
	{  END_CMD, END_CMD  },		
};

	#if 1
int Terawins640x480Page0[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};

int Terawins640x480Page1[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};


int Terawins720x480Page0[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};

int Terawins720x480Page1[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};


int Terawins720x576Page0[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};

int Terawins720x576Page1[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};


int Terawins800x600Page0[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};

int Terawins800x600Page1[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};
	#else
//-- 4:3
int Terawins640x480Page0[][2]=
{	/* Addr, Val */
    {  0x72, 0x47  },
    {  0x73, 0x1c  },
    {  0x74, 0x86  },
    {  0x75, 0x19  },
    {  0x85, 0x5e  },
    {  0xb2, 0x10  },
    {  0xb8, 0x36  },
    {  0xc8, 0x37  },
    {  0xc9, 0x02  },
    {  0xd8, 0x3c  },
    {  0xdd, 0x04  },
	{  END_CMD, END_CMD  },		
};

int Terawins640x480Page1[][2]=
{	/* Addr, Val */
    {  0xd5, 0x49  },
    {  0xd7, 0x80  },
    {  0xd8, 0x03  },
	{  END_CMD, END_CMD  },		
};


int Terawins720x480Page0[][2]=
{	/* Addr, Val */
    {  0x72, 0x00  },
    {  0x73, 0x1a  },
    {  0x74, 0x86  },
    {  0x75, 0x19  },
    {  0x85, 0x5e  },
    {  0xb2, 0x10  },
    {  0xb8, 0x36  },
    {  0xc8, 0x37  },
    {  0xc9, 0x02  },
    {  0xd8, 0x14  },
    {  0xdd, 0x04  },

	{  END_CMD, END_CMD  },		
};

int Terawins720x480Page1[][2]=
{	/* Addr, Val */
    {  0xd5, 0x49  },
    {  0xd7, 0x80  },
    {  0xd8, 0x03  },
 	{  END_CMD, END_CMD  },		
};


int Terawins720x576Page0[][2]=
{	/* Addr, Val */
    {  0x72, 0x3f  },
    {  0x73, 0x21  },
    {  0x74, 0x86  },
    {  0x75, 0x19  },
    {  0x85, 0x5e  },
    {  0xb2, 0x10  },
    {  0xb8, 0x36  },
    {  0xc8, 0x37  },
    {  0xc9, 0x02  },
    {  0xd8, 0x86  },
    {  0xdd, 0x03  },

	{  END_CMD, END_CMD  },		
};

int Terawins720x576Page1[][2]=
{	/* Addr, Val */
    {  0xd5, 0x49  },
    {  0xd7, 0x20  },
    {  0xd8, 0x03  },

	{  END_CMD, END_CMD  },		
};

//-- 16:9
int Terawins800x600Page0[][2]=
{	/* Addr, Val */
    {  0x72, 0x9f  },
    {  0x73, 0x18  },
    {  0x74, 0x99  },
    {  0x75, 0x19  },
    {  0x85, 0x5f  },
    {  0xb2, 0x0f  },
    {  0xb8, 0x3a  },
    {  0xc8, 0x45  },
    {  0xc9, 0x03  },
    {  0xd8, 0x00  },
    {  0xdd, 0x04  },
	{  END_CMD, END_CMD  },		
};

int Terawins800x600Page1[][2]=
{	/* Addr, Val */
    {  0xd5, 0x4c  },
    {  0xd7, 0x20  },
    {  0xd8, 0x03  },
	{  END_CMD, END_CMD  },		
};
	#endif

/* Innolux 8" panel 800x480(37M) */
#elif (CONFIG_ARCPGU_DISPTYPE == 4)
//-- 16:9
int TerawinsInitCmdPage0[][2]=
{	/* Addr, Val */
    {  0x30, 0x03  },
    {  0x31, 0x70  },
    {  0x32, 0x1b  },
    {  0x33, 0x3f  },
    {  0x34, 0x00  },
    {  0x35, 0xbc  },
    {  0x36, 0xbc  },
    {  0x37, 0x40  },
    {  0x38, 0x50  },
    {  0x39, 0x10  },
    {  0x3a, 0x20  },
    {  0x3b, 0x03  },
    {  0x3c, 0x00  },
    {  0x3d, 0x00  },
    {  0x3e, 0x00  },
    {  0x3f, 0x00  },

    {  0x50, 0x00  },
   
    {  0x60, 0x00  },
    {  0x61, 0x88  },
    {  0x62, 0x18  },
    {  0x63, 0x04  },//High Pass Filter   {  0x63, 0x0f  }, //mod by Eric 2007-09-20
    {  0x64, 0x04  },//Low Pass Filter    {  0x64, 0x04  },
    {  0x65, 0x08  },
    {  0x66, 0x88  },
    {  0x67, 0x1e  },
	{  0x68, 0x7d  },//Contrast    {  0x68, 0x55  },//pwm

	#if CONFIG_UDRL_INVERSE
	{  0x69, 0x81  },//Bright    {  0x69, 0x73  },//    {  0x69, 0x72  },
	#else
	{  0x69, 0x6c  },//Bright    {  0x69, 0x73  },//    {  0x69, 0x72  },
	#endif
	
    {  0x6a, 0x00  },
    {  0x6b, 0x7f  },
    {  0x6c, 0x82  },
    {  0x6d, 0x00  },
    {  0x6e, 0x10  },
    {  0x6f, 0x00  },
    
    {  0x72, 0x00  },//    {  0x72, 0x00  },
    {  0x73, 0x20  },//    {  0x73, 0x19  },
    {  0x74, 0x00  },
    {  0x75, 0x20  },
    {  0x76, 0x00  },
    {  0x77, 0x00  },
    {  0x78, 0x00  },
    {  0x79, 0x00  },
    {  0x7a, 0x00  },
    {  0x7b, 0x00  },
    {  0x7c, 0x00  },
    {  0x7d, 0x10  },
    {  0x7e, 0x80  },
    {  0x7f, 0x80  },

	/* Flowbee@960608. */
    {  0x85, 0x5b  },
    
    {  0x90, 0x04  },//Dither_video    {  0x90, 0x05  },//Still    {  0x90, 0x04  },
    {  0x91, 0x07  },
    {  0x92, 0x00  },
    {  0x93, 0x00  },
    {  0x94, 0x00  },
    {  0x95, 0x00  },
    {  0x96, 0x00  },
    {  0x97, 0x00  },
    {  0x98, 0x00  },
    {  0x99, 0x02  },
    {  0x9a, 0x40  },
    {  0x9b, 0x00  },
    {  0x9c, 0x00  },
    {  0x9d, 0x10  },
    {  0x9e, 0x80  },
    {  0x9f, 0x80  },

    {  0xb0, 0x20  },
    {  0xb1, 0x00  },
    {  0xb2, 0x0f  },
    {  0xb3, 0x00  },
    {  0xb4, 0x25  },
    {  0xb5, 0x03  },
    {  0xb6, 0xe0  },
    {  0xb7, 0x02  },
    {  0xb8, 0x64  },
    {  0xb9, 0x04  },
    {  0xba, 0x0d  },
    {  0xbb, 0x02  },
    {  0xbc, 0x20  },
    {  0xbd, 0x00  },
    {  0xbe, 0x04  },
    {  0xbf, 0x00  },

	{  0xc0, 0x01  },//RGB666 
    {  0xc1, 0x08  },
    {  0xc2, 0x00  },
    {  0xc3, 0x00  },
    {  0xc4, 0x00  },
    {  0xc5, 0x60  },
    {  0xc6, 0x05  },
    {  0xc7, 0x00  },
    {  0xc8, 0x39  },
    {  0xc9, 0x03  },
    {  0xca, 0x02  },
    {  0xcb, 0x00  },
    {  0xcc, 0x00  },
    {  0xcd, 0x00  },
    {  0xce, 0x00  },
    {  0xcf, 0x00  },
    
    {  0xd0, 0x00  },
    {  0xd1, 0x00  },
    {  0xd2, 0x00  },
    {  0xd3, 0x00  },
    {  0xd4, 0x00  },
    {  0xd5, 0x00  },
    {  0xd6, 0x00  },
    {  0xd7, 0x00  },
    {  0xd8, 0x00  },
    {  0xd9, 0x00  },
    {  0xda, 0x00  },
    {  0xdb, 0x00  },
    {  0xdc, 0x25  },
    {  0xdd, 0x03  },
    {  0xde, 0xe0  },
    {  0xdf, 0x02  },
    
    {  0xe0, 0xb1  },

    #if	CONFIG_FASHION_STAR	// Up/Down
    {  0xe1, 0xc0  },//Fashion Star
	#else
		#if CONFIG_UDRL_INVERSE
	    {  0xe1, 0xf0  },//Skymedi
		#else
	    {  0xe1, 0x00  },//Billionton
	    #endif
	#endif
	
    {  0xe2, 0x00  },
    {  0xe3, 0x00  },
    {  0xe4, 0x00  },
    {  0xe5, 0x00  },
    {  0xe6, 0x07  },
    {  0xe7, 0x80  },
    {  0xe8, 0x10  },
    {  0xe9, 0x7d  },
    {  0xea, 0x07  },
    {  0xeb, 0x80  },
    {  0xec, 0xc0  },
    {  0xed, 0x80  },
    {  0xee, 0x00  },
    {  0xef, 0x00  },
	{  END_CMD, END_CMD  },		
};

int TerawinsInitCmdPage1[][2]=
{	/* Addr, Val */
    {  0x20, 0x28  },//    {  0x20, 0x20  },
    {  0x21, 0x7b  },
    {  0x22, 0xc5  },
    {  0x23, 0x03  },
    {  0x24, 0x0c  },
    {  0x25, 0xc5  },
    {  0x26, 0x03  },
    {  0x27, 0xb9  },
    {  0x28, 0x03  },
    {  0x29, 0x29  },
    {  0x2a, 0x00  },
    {  0x2b, 0x06  },//Still    {  0x2b, 0x06  },//Video    {  0x2b, 0x07  },
    {  0x2c, 0x18  },
    {  0x2d, 0x03  },
    {  0x2e, 0x00  },
    {  0x2f, 0x00  },
    
    {  0x30, 0x00  },
    {  0x31, 0x90  },
    {  0x32, 0x03  },
    {  0x33, 0x35  },
    {  0x34, 0x00  },
    {  0x35, 0x02  },
    {  0x36, 0x80  },
    {  0x37, 0x02  },
    {  0x38, 0x00  },
    {  0x39, 0x00  },
    {  0x3a, 0x00  },
    {  0x3b, 0x00  },
    {  0x3c, 0x00  },
    {  0x3d, 0x00  },
    {  0x3e, 0x00  },
    {  0x3f, 0x00  },

    {  0xb0, 0x95  },
    {  0xb1, 0x00  },
    {  0xb2, 0xe0  },//    {  0xb2, 0xe0  },
    {  0xb3, 0x95  },//    {  0xb3, 0x91  },//    {  0xb3, 0x8e  },//    {  0xb3, 0x74  },
    {  0xb4, 0x00  },//    {  0xb4, 0x32  },
    {  0xb5, 0x68  },
    {  0xb6, 0x95  },
    {  0xb7, 0x81  },
    {  0xb8, 0xc0  },
    {  0xb9, 0x00  },
    {  0xba, 0x00  },
    {  0xbb, 0x00  },
    {  0xbc, 0x00  },
    {  0xbd, 0x00  },
    {  0xbe, 0x00  },
    {  0xbf, 0x00  },

    {  0xd0, 0x30  },
    {  0xd1, 0x00  },
    {  0xd2, 0x08  },//    {  0xd2, 0x0a  },
    {  0xd3, 0x01  },
    {  0xd4, 0x01  },
    {  0xd5, 0x44  },//Still    {  0xd5, 0x37  },//Video    {  0xd5, 0x3b  },
    {  0xd6, 0x00  },
    {  0xd7, 0x60  },//Still    {  0xd7, 0x25  },//Video    {  0xd7, 0x20  },
    {  0xd8, 0x03  },//    {  0xd8, 0x02  },
    {  0xd9, 0x2a  },//    {  0xd9, 0x08  },
    {  0xda, 0x00  },
    {  0xdb, 0xe0  },
    {  0xdc, 0x01  },
    {  0xdd, 0x00  },
    {  0xde, 0x02  },
    {  0xdf, 0x0c  },
	{  END_CMD, END_CMD  },		
};

int Terawins640x480Page0[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};

int Terawins640x480Page1[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};


int Terawins720x480Page0[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};

int Terawins720x480Page1[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};


int Terawins720x576Page0[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};

int Terawins720x576Page1[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};


int Terawins800x600Page0[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};

int Terawins800x600Page1[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};

/* Innolux 800x600 LCD panel. */
#elif (CONFIG_ARCPGU_DISPTYPE == 5)
// 37 MHz RGB666
int TerawinsInitCmdPage0[][2]=
{	/* Addr, Val */
    {  0x30, 0x03  },
    {  0x31, 0x70  },//RGB666
    {  0x32, 0x3f  },
    {  0x33, 0x3f  },
    {  0x34, 0x00  },
    {  0x35, 0xbc  },
    {  0x36, 0xbc  },//    {  0x36, 0x34  },
    {  0x37, 0x40  },
    {  0x38, 0x50  },
    {  0x39, 0x10  },
    {  0x3a, 0x20  },
    {  0x3b, 0x03  },
    {  0x3c, 0x00  },
    {  0x3d, 0x00  },
    {  0x3e, 0x00  },
    {  0x3f, 0x00  },//    {  0x3f, 0x10  },

	/* Flowbee@960608. */
    {  0x50, 0x00  },//    {  0x50, 0x30  },
   
    {  0x60, 0x00  },
    {  0x61, 0x88  },
    {  0x62, 0x18  },
    {  0x63, 0x04  },//High Pass Filter   {  0x63, 0x0f  }, //mod by Eric 2007-09-20
    {  0x64, 0x04  },//    {  0x64, 0x03  },//Low Pass Filter    {  0x64, 0x04  },
    {  0x65, 0x08  },
    {  0x66, 0x88  },
    {  0x67, 0x1e  },
    #if	CONFIG_FASHION_STAR
	{  0x68, 0x80  },
    {  0x69, 0x88  },
	#else
	{  0x68, 0x7d  },//	{  0x68, 0x7d  },//Contrast    {  0x68, 0x55  },//pwm
		#if CONFIG_UDRL_INVERSE
		{  0x69, 0x81  },//Bright    {  0x69, 0x73  },//    {  0x69, 0x72  },
		#else
		{  0x69, 0x6c  },//Bright    {  0x69, 0x73  },//    {  0x69, 0x72  },
		#endif
	#endif
    {  0x6a, 0x00  },
    {  0x6b, 0x7f  },
    {  0x6c, 0x82  },
    {  0x6d, 0x00  },
    {  0x6e, 0x10  },
    {  0x6f, 0x00  },
    
    {  0x72, 0x0b  },//Still    {  0x72, 0x0b  },//Video    {  0x72, 0xd1  },
    {  0x73, 0x20  },//Still    {  0x73, 0x20  },//Video    {  0x73, 0x1f  },
    {  0x74, 0x00  },
    {  0x75, 0x20  },
    {  0x76, 0x00  },
    {  0x77, 0x00  },
    {  0x78, 0x00  },
    {  0x79, 0x00  },
    {  0x7a, 0x00  },
    {  0x7b, 0x00  },
    {  0x7c, 0x00  },
    {  0x7d, 0x10  },
    {  0x7e, 0x80  },
    {  0x7f, 0x80  },

	/* Flowbee@960608. */
	#if CONFIG_54MHz
    {  0x85, 0x14  },//27MHz    {  0x85, 0x1f  },//54MHz    {  0x85, 0x14  },
	#else
    {  0x85, 0x1f  },//27MHz    {  0x85, 0x1f  },//54MHz    {  0x85, 0x14  },
	#endif
    
    {  0x90, 0x04  },//Dither_video    {  0x90, 0x05  },//Still    {  0x90, 0x04  },
    {  0x91, 0x87  },//on    {  0x91, 0x07  },//off    {  0x91, 0x87  },
    {  0x92, 0x00  },
    {  0x93, 0x00  },
    {  0x94, 0x00  },
    {  0x95, 0x00  },
    {  0x96, 0x00  },
    {  0x97, 0x00  },
    {  0x98, 0x00  },
    {  0x99, 0x02  },
    {  0x9a, 0x40  },
    {  0x9b, 0x00  },
    {  0x9c, 0x00  },
    {  0x9d, 0x10  },
    {  0x9e, 0x80  },
    {  0x9f, 0x80  },

    {  0xb0, 0x20  },
    {  0xb1, 0x00  },
    #if CONFIG_54MHz
    {  0xb2, 0x17  },//27MHz    {  0xb2, 0x0e  },//54MHz    {  0xb2, 0x17  },
	#else
    {  0xb2, 0x0e  },//27MHz    {  0xb2, 0x0e  },//54MHz    {  0xb2, 0x17  },
	#endif
    {  0xb3, 0x00  },
    {  0xb4, 0x25  },
    {  0xb5, 0x03  },
    {  0xb6, 0x59  },//960828    {  0xb6, 0x58  },
    {  0xb7, 0x02  },
    #if CONFIG_54MHz
    {  0xb8, 0xcb  },//27MHz    {  0xb8, 0x86  },//54MHz    {  0xb8, 0xcb  },
    {  0xb9, 0x04  },//27MHz    {  0xb9, 0x05  },//27MHz    {  0xb9, 0x04  },
	#else
    {  0xb8, 0xff  },//27MHz    {  0xb8, 0x86  },//54MHz    {  0xb8, 0xcb  },
    {  0xb9, 0x04  },//27MHz    {  0xb9, 0x05  },//27MHz    {  0xb9, 0x04  },
	#endif
    {  0xba, 0x6d  },
    {  0xbb, 0x02  },
    {  0xbc, 0x20  },
    {  0xbd, 0x00  },
    {  0xbe, 0x04  },
    {  0xbf, 0x00  },

	{  0xc0, 0x01  },//RGB666 
    {  0xc1, 0x08  },
    {  0xc2, 0x12  },//on    {  0xc2, 0x00  },//off    {  0xc2, 0x12  },
    {  0xc3, 0x00  },
    {  0xc4, 0x00  },
    {  0xc5, 0x60  },
    {  0xc6, 0x05  },
    {  0xc7, 0x00  },
    #if CONFIG_54MHz
    {  0xc8, 0x19  },//27MHz    {  0xc8, 0x3c  },//54MHz    {  0xc8, 0x19  },
	#else
    {  0xc8, 0x36  },//27MHz    {  0xc8, 0x3c  },//54MHz    {  0xc8, 0x19  },
	#endif
    {  0xc9, 0x02  },
    {  0xca, 0x02  },
    {  0xcb, 0x00  },
    {  0xcc, 0x00  },
    {  0xcd, 0x00  },
    {  0xce, 0x00  },
    {  0xcf, 0x00  },
    
    {  0xd0, 0x00  },
    {  0xd1, 0x00  },
    {  0xd2, 0x00  },
    {  0xd3, 0x00  },
    {  0xd4, 0x00  },
    {  0xd5, 0x00  },
    {  0xd6, 0x00  },
    {  0xd7, 0x00  },
    {  0xd8, 0x00  },
    {  0xd9, 0x00  },
    {  0xda, 0x00  },
    {  0xdb, 0x00  },
    {  0xdc, 0x25  },
    {  0xdd, 0x03  },
    {  0xde, 0x58  },
    {  0xdf, 0x02  },
    
    {  0xe0, 0xb1  },
    #if	CONFIG_FASHION_STAR	// Up/Down
    {  0xe1, 0xc0  },//Fashion Star
	#else
		#if CONFIG_UDRL_INVERSE
	    {  0xe1, 0xf0  },//Skymedi
		#else
	    {  0xe1, 0x00  },//Billionton
	    #endif
	#endif
    {  0xe2, 0x00  },
    {  0xe3, 0x00  },
    {  0xe4, 0x00  },
    {  0xe5, 0x00  },
    {  0xe6, 0x07  },
    {  0xe7, 0x80  },
    {  0xe8, 0x00  },// pwm
    {  0xe9, 0x7d  },// pwm
    {  0xea, 0x07  },
    {  0xeb, 0x80  },
    {  0xec, 0xc0  },
    {  0xed, 0x00  },//    {  0xed, 0x10  },
    {  0xee, 0x00  },
    {  0xef, 0x00  },
	{  END_CMD, END_CMD  },		
};

int TerawinsInitCmdPage1[][2]=
{	/* Addr, Val */
    {  0x20, 0x28  },
    {  0x21, 0x7b  },
    {  0x22, 0xc5  },
    {  0x23, 0x03  },
    {  0x24, 0x0c  },
    {  0x25, 0xc5  },
    {  0x26, 0x03  },
    {  0x27, 0xb9  },
    {  0x28, 0x03  },
    {  0x29, 0x29  },
    {  0x2a, 0x00  },
    {  0x2b, 0x06  },//Still    {  0x2b, 0x06  },//Video    {  0x2b, 0x07  },
    {  0x2c, 0x18  },
    {  0x2d, 0x03  },
    {  0x2e, 0x00  },
    {  0x2f, 0x00  },
    
    {  0x30, 0x00  },
    {  0x31, 0x90  },
    {  0x32, 0x03  },
    {  0x33, 0x35  },
    {  0x34, 0x00  },
    {  0x35, 0x02  },
    {  0x36, 0x80  },
    {  0x37, 0x02  },
    {  0x38, 0x00  },
    {  0x39, 0x00  },
    {  0x3a, 0x00  },
    {  0x3b, 0x00  },
    {  0x3c, 0x00  },
    {  0x3d, 0x00  },
    {  0x3e, 0x00  },
    {  0x3f, 0x00  },

    {  0xb0, 0x95  },
    {  0xb1, 0x00  },
    {  0xb2, 0xe8  },//    {  0xb2, 0xe0  },
    {  0xb3, 0x95  },//    {  0xb3, 0x91  },//    {  0xb3, 0x8e  },//    {  0xb3, 0x74  },
    #if CONFIG_FASHION_STAR
    {  0xb4, 0x14  },
	#else
    {  0xb4, 0x00  },//    {  0xb4, 0x32  },
	#endif
    {  0xb5, 0x68  },
    {  0xb6, 0x95  },
    {  0xb7, 0x81  },
    {  0xb8, 0xc0  },
    {  0xb9, 0x00  },
    {  0xba, 0x00  },
    {  0xbb, 0x00  },
    {  0xbc, 0x00  },
    {  0xbd, 0x00  },
    {  0xbe, 0x00  },
    {  0xbf, 0x00  },

    {  0xd0, 0x30  },
    {  0xd1, 0x00  },
    {  0xd2, 0x08  },//    {  0xd2, 0x0a  },
    {  0xd3, 0x01  },
    {  0xd4, 0x01  },
    {  0xd5, 0x37  },//Still    {  0xd5, 0x37  },//Video    {  0xd5, 0x3b  },
    {  0xd6, 0x00  },
    {  0xd7, 0x25  },//Still    {  0xd7, 0x25  },//Video    {  0xd7, 0x20  },
    {  0xd8, 0x03  },//    {  0xd8, 0x02  },
    {  0xd9, 0x11  },//960824    {  0xd9, 0x12  },
    {  0xda, 0x00  },
    {  0xdb, 0x58  },
    {  0xdc, 0x02  },
    {  0xdd, 0x00  },
    {  0xde, 0x03  },//    {  0xde, 0x02  },
    {  0xdf, 0x0c  },
	{  END_CMD, END_CMD  },		
};

#if CONFIG_54MHz	//-- 54MHz

int Terawins640x480Page0[][2]=
{	/* Addr, Val */
    {  0x63, 0x04  },//high pass filter
    {  0x72, 0x93  },
    {  0x73, 0x19  },
    {  0x74, 0x00  },
    {  0x75, 0x1a  },    
    {  0x85, 0x2a  },
    {  0x90, 0x05  },//Dither    
    {  0xb2, 0x56  },
    {  0xb8, 0xa0  },
    {  0xb9, 0x04  },
    {  0xc8, 0x1e  },    
	{  END_CMD, END_CMD  },
};

int Terawins640x480Page1[][2]=
{	/* Addr, Val */
    {  0x2b, 0x07  },
    {  0xd5, 0x8b  },
    {  0xd7, 0x15  },    
    {  0xd9, 0x14  },
	{  END_CMD, END_CMD  },
};


int Terawins720x480Page0[][2]=
{	/* Addr, Val */
    {  0x63, 0x04  },//high pass filter
    {  0x72, 0x95  },
    {  0x73, 0x1c  },
    {  0x74, 0x00  },        
    {  0x75, 0x1d  },
    {  0x85, 0xe0  },
    {  0x90, 0x05  },//Dither
    {  0xb2, 0x15  },
    {  0xb8, 0xd2  },
    {  0xb9, 0x04  },
    {  0xc8, 0x1c  },        
	{  END_CMD, END_CMD  },
};

int Terawins720x480Page1[][2]=
{	/* Addr, Val */
    {  0x2b, 0x07  },
    {  0xd5, 0x64  },
    {  0xd7, 0x21  },
    {  0xd9, 0x0c  },
	{  END_CMD, END_CMD  },
};


int Terawins720x576Page0[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};

int Terawins720x576Page1[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};


int Terawins800x600Page0[][2]=
{	/* Addr, Val */
    {  0x63, 0x04  },//High Pass Filter   {  0x63, 0x0f  }, //mod by Eric
    {  0x72, 0x0b  },
    {  0x73, 0x20  },
    {  0x74, 0x00  },    
    {  0x75, 0x20  },
    {  0x90, 0x04  },//No dither
    {  0x85, 0x14  },
    {  0xb2, 0x17  },
    {  0xb8, 0xcb  },
    {  0xb9, 0x04  },
    {  0xc8, 0x19  },        
	{  END_CMD, END_CMD  },
};

int Terawins800x600Page1[][2]=
{	/* Addr, Val */
    {  0x2b, 0x06  },
    {  0xd5, 0x37  },//    {  0xd5, 0x37  },//    {  0xd5, 0x3b  },
    {  0xd7, 0x25  },    
    {  0xd9, 0x12  },
	{  END_CMD, END_CMD  },
};

#else	//-- 27MHz

int Terawins640x480Page0[][2]=
{	/* Addr, Val */
    {  0x72, 0x93  },
    {  0x73, 0x19  },
    {  0x75, 0x19  },
    {  0x85, 0x24  },
    {  0x90, 0x05  },//Dither    
    {  0xb2, 0x63  },
    {  0xb8, 0xe4  },//    {  0xb8, 0x4f  },
    {  0xb9, 0x03  },//    {  0xb9, 0x04  },
	{  END_CMD, END_CMD  },
};

int Terawins640x480Page1[][2]=
{	/* Addr, Val */
    {  0x2b, 0x07  },
    {  0xd5, 0x88  },
    {  0xd7, 0x20  },    
    {  0xd9, 0x19  },
	{  END_CMD, END_CMD  },
};


int Terawins720x480Page0[][2]=
{	/* Addr, Val */
    {  0x72, 0x79  },
    {  0x73, 0x1c  },
    {  0x75, 0x1d  },
    {  0x85, 0x00  },
    {  0x90, 0x05  },//Dither
    {  0xb2, 0x40  },
    {  0xb8, 0x82  },//    {  0xb8, 0xfe  },
    {  0xb9, 0x04  },//    {  0xb9, 0x04  },
	{  END_CMD, END_CMD  },
};

int Terawins720x480Page1[][2]=
{	/* Addr, Val */
    {  0x2b, 0x07  },
    {  0xd5, 0x64  },
    {  0xd7, 0x20  },
    {  0xd9, 0x0d  },
	{  END_CMD, END_CMD  },
};


int Terawins720x576Page0[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};

int Terawins720x576Page1[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};


int Terawins800x600Page0[][2]=
{	/* Addr, Val */
    {  0x72, 0x0b  },//    {  0x72, 0x0b  },//    {  0x72, 0xd1  },
    {  0x73, 0x20  },//    {  0x73, 0x20  },//    {  0x73, 0x1f  },
    {  0x75, 0x20  },
    {  0x90, 0x04  },//No dither
    {  0x85, 0x1f  },//27MHz    {  0x85, 0x1f  },//54MHz    {  0x85, 0x14  },
    {  0xb2, 0x0e  },//27MHz    {  0xb2, 0x0e  },//54MHz    {  0xb2, 0x17  },
    {  0xb8, 0xff  },//27MHz    {  0xb8, 0x86  },//54MHz    {  0xb8, 0xcb  },
    {  0xb9, 0x04  },//27MHz    {  0xb9, 0x05  },//54MHz    {  0xb9, 0x04  },
	{  END_CMD, END_CMD  },
};

int Terawins800x600Page1[][2]=
{	/* Addr, Val */
    {  0x2b, 0x06  },
    {  0xd5, 0x37  },//    {  0xd5, 0x37  },//    {  0xd5, 0x3b  },
    {  0xd7, 0x25  },    
    {  0xd9, 0x11  },//960824    {  0xd9, 0x12  },
	{  END_CMD, END_CMD  },
};

#endif


#else	// TV

int TerawinsInitCmdPage0[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};

int TerawinsInitCmdPage1[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};

int Terawins640x480Page0[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};

int Terawins640x480Page1[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};


int Terawins720x480Page0[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};

int Terawins720x480Page1[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};


int Terawins720x576Page0[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};

int Terawins720x576Page1[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};


int Terawins800x600Page0[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};

int Terawins800x600Page1[][2]=
{	/* Addr, Val */
	{  END_CMD, END_CMD  },		
};

#endif

int *TerawinsCmdTbl[TERAWINS_END*2]=
{
	(&(TerawinsInitCmdPage0[0][0])),
	(&(TerawinsInitCmdPage1[0][0])),
	(&(Terawins640x480Page0[0][0])),
	(&(Terawins640x480Page1[0][0])),
	(&(Terawins720x480Page0[0][0])),
	(&(Terawins720x480Page1[0][0])),
	(&(Terawins720x576Page0[0][0])),
	(&(Terawins720x576Page1[0][0])),
	(&(Terawins800x600Page0[0][0])),
	(&(Terawins800x600Page1[0][0])),
};


/* --------------------------------------------------------------------- */
inline void terawins_delay(unsigned long count)
{
	unsigned long _x; 
	do { 
		for(_x=0;_x< count;_x++){
			__asm__ __volatile__("nop");
			//__builtin_arc_nop();
		}
	} while(0);
}

/* --------------------------------------------------------------------- */
void terawins_i2c_reset(void)
{
	SCL_HI;
	SCL_OUTPUT;
	SDA_HI;
	SDA_OUTPUT;	

	terawins_delay(TERAWINS_TIME_1);
}

/* --------------------------------------------------------------------- */
void terawins_i2c_start(void)
{
	SDA_HI;
	SDA_OUTPUT;
	SCL_HI;
	SCL_OUTPUT;
	terawins_delay(TERAWINS_TIME_1);	
	SDA_LO;
	terawins_delay(TERAWINS_TIME_1);	
	SCL_LO;
}

/* --------------------------------------------------------------------- */
void terawins_i2c_stop(void)
{
	SDA_LO;
	SCL_HI;
	terawins_delay(TERAWINS_TIME_1);
	SDA_HI;
}

/* --------------------------------------------------------------------- */
void terawins_i2c_ack(int bit)
{
	/* Here's the deal. The slave is slow, and sometimes needs to wait
       before it can receive the acknowledge. Therefore it forces the clock
       low until it is ready. We need to poll the clock line until it goes
       high before we release the ack. */
    
	SCL_LO;      /* Set the clock low */
	if ( bit )
	{
		SDA_HI;
	}
	else
	{
		SDA_LO;
	}
    
	SCL_INPUT;   /* Set the clock to input */
	#if 0
	while(!SCL);  /* and wait for the slave to release it */      
	#else
	do{
		SCL;
	}while(!vtval);
	#endif
	terawins_delay(TERAWINS_TIME_1);
	SCL_OUTPUT;
	SCL_LO;
	terawins_delay(TERAWINS_TIME_2);	
}

/* --------------------------------------------------------------------- */
int terawins_i2c_getack(void)
{
	int ret = 1;

	SDA_INPUT;
	terawins_delay(TERAWINS_TIME_2);
	SCL_HI;
	#if 0
	if (SDA)
	    ret = 0;
	#else
	SDA;
	if (vtval)
	    ret = 0;

	#endif
	
	terawins_delay(TERAWINS_TIME_2);

	/* pull up SDA first, and set SDA to OUTPUT */
	//Flowbee
//	SDA_LO;   
	SDA_HI;
	SDA_OUTPUT;  
	  
	SCL_LO;

	return ret;
}

/* --------------------------------------------------------------------- */
void terawins_i2c_outb(unsigned char byte)
{
    int i;
   /* clock out each bit, MSB first */
   for ( i=0x80; i; i>>=1 ) {
      	if ( i & byte ){
        	SDA_HI;
      	}
      	else{
        	SDA_LO;
      	}
	terawins_delay(TERAWINS_TIME_1);
      	SCL_HI;
	terawins_delay(TERAWINS_TIME_1);
	SCL_LO;
   }

   SDA_HI;
}

/* --------------------------------------------------------------------- */
unsigned char terawins_i2c_inb(int msb, int ack)
{
	int i;
	unsigned char byte = 0;

	if(msb){
	/* clock in each bit, MSB first */
		for ( i=0x80; i; i>>=1 ) {
			terawins_delay(TERAWINS_TIME_1);
			SDA_HI;
			SDA_INPUT;   /* And set to input */
			SCL_HI;
			terawins_delay(TERAWINS_TIME_1);
			#if 0
			if ( SDA )
				byte |= i;

			#else
			SDA;
			if (vtval)
				byte |= i;

			#endif

			SCL_LO;
			SDA_OUTPUT;
			terawins_delay(TERAWINS_TIME_1);
		}
	}else{
	/* clock in each bit, LSB first */
		for(i=0; i<8; i++){
			SDA_HI;
			SDA_INPUT;   /* And set to input */
			SCL_HI;
			terawins_delay(TERAWINS_TIME_1);		
			#if 0
			if ( SDA )
				byte |= (1 << i);

			#else
			SDA;
			if (vtval)
				byte |= (1 << i);

			#endif

			SCL_LO;
			SDA_OUTPUT;
			terawins_delay(TERAWINS_TIME_1);			
		}
	}
   
	terawins_i2c_ack(ack);
	return byte;
}

/* --------------------------------------------------------------------- */
int terawins_i2c_write(int address, const unsigned char* buf, int count)
{
	int i,x=0;	
	terawins_i2c_start();
	terawins_i2c_outb(address & 0xFE);
	if (terawins_i2c_getack()){
	    for (i=0; i<count; i++){
	    	terawins_i2c_outb(buf[i]);
	        if (!terawins_i2c_getack()){
	        	x=2;
	                break;
	        }
	    }
	}else{
		x=1;
	}
	
	terawins_i2c_stop();
	return x;
}

#if 0	//-- unused function
/* --------------------------------------------------------------------- */
static int terawins_i2c_read(int address, unsigned char* buf, int count, int msb)
{
    int i,x=1;

	terawins_i2c_start();
	terawins_i2c_outb(address | 0x01);
	if (terawins_i2c_getack()) {
		for (i=0; i<count; i++)
			buf[i] = terawins_i2c_inb(msb, 0);
	}
	else
        x=0;
	
	terawins_i2c_stop();
	return x;
}
#endif

/* --------------------------------------------------------------------- */
int terawins_set_cmd(uint8 dev_addr, uint8 reg_addr, uint8 reg_val)
{
	unsigned char _buf[2] = { reg_addr, reg_val};
	int i;

	i = terawins_i2c_write(dev_addr, _buf, 2);
	if(i != 0){
		//TW_MSG("Terawins I2C write error !!\n");
	}
	return i;
}

#if 0	//-- unused function
/* --------------------------------------------------------------------- */
static unsigned char terawins_get_cmd(uint8 dev_addr, uint8 reg_addr)
{
	unsigned char _buf = reg_addr;
    int i,x=1;	
	int count = 1;
	int msb = 1;
	/* Step1: write */
	terawins_i2c_start();
	terawins_i2c_outb(dev_addr & 0xFE);
	if (terawins_i2c_getack()){
    	terawins_i2c_outb(_buf);
        if (!terawins_i2c_getack()){
        	x=2;
        }
	}else{
		x=1;
	}
    /* Step2: restart and read. */
	terawins_i2c_start();
	terawins_i2c_outb(dev_addr | 0x01);
	if (terawins_i2c_getack()) {
		for (i=0; i<count; i++)
			_buf = terawins_i2c_inb(msb, 0);
	}
	else
        x=0;
	
	terawins_delay(TERAWINS_TIME_2);	
	terawins_i2c_stop();
	terawins_delay(TERAWINS_TIME_2);
	return _buf;
}
#endif

/* --------------------------------------------------------------------- */
int terawins_set_page(unsigned char _page_num, int *_page)
{
	int ret_val = 0;
	while(*(_page) != END_CMD){
		ret_val = terawins_set_cmd(_page_num, (unsigned char)(*(_page)), (unsigned char)(*(_page + 1)));
		if((ret_val == 2) || (ret_val == 1)){
			TW_MSG("Write to TeraWins Error!! page:%x %x %x\n", _page_num, (unsigned char)(*(_page)), (unsigned char)(*(_page + 1)));
			return ret_val;
		}
		_page += 2;
		terawins_delay(0x10);
	}
	return ret_val;
}

/* --------------------------------------------------------------------- */
int terawins_on_off(uint8 _on)
{
	if(_on == 1){
		//-- turn on
		terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xC2, 0x00);
		terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0x91, 0x07);
		terawins_delay(0x200000);
	}
	else{
		//-- turn off
		terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xC2, 0x12);
		terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0x91, 0x87);
	}
	return 0;
}

/* --------------------------------------------------------------------- */
int terawins_set_resolution(uint8 idx)
{
	int ret_val = 0;
	int *p0, *p1;

	if (idx == TERAWINS_640x480){
		p0 = TerawinsCmdTbl[TERAWINS_640x480 * 2];
		p1 = TerawinsCmdTbl[TERAWINS_640x480 * 2 + 1];
	}
	else if(idx == TERAWINS_720x480){
		p0 = TerawinsCmdTbl[TERAWINS_720x480 * 2];
		p1 = TerawinsCmdTbl[TERAWINS_720x480 * 2 + 1];
	}
	else if(idx == TERAWINS_720x576){
		p0 = TerawinsCmdTbl[TERAWINS_720x576 * 2];
		p1 = TerawinsCmdTbl[TERAWINS_720x576 * 2 + 1];
	}
	else if(idx == TERAWINS_800x600){
		p0 = TerawinsCmdTbl[TERAWINS_800x600 * 2];
		p1 = TerawinsCmdTbl[TERAWINS_800x600 * 2 + 1];
	}
	else{
		TW_MSG("Input dimension not support !!\n");
		return -1;
	}
	/* Set command to terawins T-CON. */
	//-- Setting page0 registers.
	ret_val = terawins_set_page(TERAWINS_PAGE0_ADDR, p0);
	//-- Setting page1 registers.
	ret_val |= terawins_set_page(TERAWINS_PAGE1_ADDR, p1);

	return ret_val;
}

/* --------------------------------------------------------------------- */
void terawins_pwm_on_off(uint8 _on)
{
	int i = 0;
	if(_on == 1){
		//-- Turn on PWM
		i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xe8, 0x10);
	}
	else{
		//-- Turn off PWM
		i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xe8, 0x00);			
	}
	if(i != 0){
		TW_MSG("Set PWM on/off fail !\n");
	}
}

/* --------------------------------------------------------------------- */
void terawins_backlight_on_off(uint8 _on)
{
	int i = 0;
	if(_on == 1){
		//-- Turn on terawins backlight
		i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xec, 0xe2);
	}
	else{
		//-- Turn off terawins backlight
		i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xec, 0xc0);			
	}
	if(i != 0){
		TW_MSG("Terawins backlight on/off fail !\n");
	}
}

/* --------------------------------------------------------------------- */
void terawins_set_pwm(uint8 _val)
{
	int i = 0;
	//TW_MSG("\nPWM !!\n");
	//-- Set pwm duty cycle.
	i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xe9, _val);	
	if(i != 0){
		TW_MSG("Set PWM fail !\n");
	}
}

/* --------------------------------------------------------------------- */
void terawins_set_brightness(uint8 _val)
{
	int i = 0;
	//TW_MSG("\nBrightness !!\n");
	//-- Set pwm freq.
	i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0x69, _val);
	if(i != 0){
		TW_MSG("Set Brightness fail !\n");
	}
}

/* --------------------------------------------------------------------- */
void terawins_set_contrast(uint8 _val)
{
	int i = 0;
	//TW_MSG("\nContrast !!\n");
	//-- Set pwm freq.
	i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0x68, _val);
	if(i != 0){
		TW_MSG("Set Contrast fail !\n");
	}
}

/* --------------------------------------------------------------------- */
int init_terawins_tcon(void)
{
	int *p0, *p1;

	TW_MSG("\nInit Terawins T-CON !!\n");
	/* Set default commands to terawins T-CON. */
	//-- Setting page0 registers.
	p0 = TerawinsCmdTbl[TERAWINS_INIT * 2];
	terawins_set_page(TERAWINS_PAGE0_ADDR, p0);
	//-- Setting page1 registers.
	p1 = TerawinsCmdTbl[TERAWINS_INIT * 2 + 1];
	terawins_set_page(TERAWINS_PAGE1_ADDR, p1);
	return 0;
}

/* --------------------------------------------------------------------- */
/*
 *	Terawins OSD functions.
 */
 
//-- Font table (12 x 18)
/* --------------------------------------------------------------------- */
// sandglass 2x2, invalid 3x3
//-- SandGrass: 0x00 ~ 0x03
static unsigned char font_000[]={ //
0xF0,0xFF,0xF0,0xFF,0xF0,0xFF,0xF0,0x39,0xF0,0x39,0xF0,0x39,0xF0,0x38,0xF0,0x1C,0x70,0x1E,
0x70,0x0E,0x30,0x07,0x30,0x03,0x30,0x03,0xB0,0x01,0xF0,0x00,0xF0,0x00,0xF0,0x00,0xF0,0x00
};
static unsigned char font_001[]={ //
0xF0,0xFF,0xF0,0xFF,0xF0,0xFF,0xC0,0xFF,0xC0,0xFF,0xC0,0xFF,0xC0,0xFF,0x80,0xFF,0x80,0xFF,
0x00,0xFF,0x00,0xFE,0x00,0xFC,0x00,0xFC,0x00,0xF8,0x00,0xF0,0x00,0xF0,0x00,0xF0,0x00,0xF0
};
static unsigned char font_002[]={ //(null)
0xF0,0x00,0xF0,0x00,0xF0,0x00,0xF0,0x00,0xB0,0x01,0x30,0x03,0x30,0x03,0x70,0x06,0x70,0x0E,
0xF0,0x1C,0xF0,0x1C,0xF0,0x38,0xF0,0x39,0xF0,0x39,0xC0,0x39,0xF0,0xFF,0xF0,0xFF,0xF0,0xFF
};
static unsigned char font_003[]={ //(null)
0x00,0xF0,0x00,0xF0,0x00,0xF0,0x00,0xF0,0x00,0xF8,0x00,0xFC,0x00,0xFC,0x00,0xFE,0x00,0xFF,
0x80,0xFF,0x80,0xFF,0xC0,0xF9,0xC0,0xF9,0xC0,0x81,0xC0,0x01,0xF0,0xFF,0xF0,0xFF,0xF0,0xFF
};

//-- tri: 0x04 ~ 0x07
static unsigned char font_004[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x10,0x00,0x30,0x00,0x70,0x00,0x70,0x00,
0xE0,0x00,0xE0,0x00,0xC0,0x01,0xC0,0x01,0x90,0x03,0x90,0x03,0x10,0x07,0x10,0x07,0x10,0x0E
};
static unsigned char font_005[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x00,0xC0,0x00,0xE0,0x00,0xE0,
0x00,0x70,0x00,0x70,0x00,0x38,0x00,0x38,0x00,0x9C,0x00,0x9C,0x00,0x8E,0x00,0x8E,0x00,0x87
};
static unsigned char font_006[]={ //(null)
0x10,0x1E,0x10,0x1C,0x00,0x3C,0x10,0x38,0x10,0x78,0x00,0x70,0xF0,0xFF,0xF0,0xFF,0xF0,0xFF,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
};
static unsigned char font_007[]={ //(null)
0x80,0x87,0x80,0x83,0xC0,0x03,0xC0,0x81,0xE0,0x81,0xE0,0x00,0xF0,0xFF,0xF0,0xFF,0xF0,0xFF,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
};
static unsigned char font_008[]={ //(null)
0x00,0xE0,0x00,0xE0,0x00,0xF0,0x00,0xF8,0x00,0xFC,0x00,0xFE,0x00,0xFE,0x00,0xFF,0x00,0xFF,
0x00,0xF7,0x80,0xE3,0x80,0xC3,0x80,0x03,0x80,0x03,0xF0,0xFF,0xF0,0xFF,0xF0,0xFF,0xF0,0xFF
};
/* --------------------------------------------------------------------- */
//-- Invalid sign: 0x09 ~ 0x11
static unsigned char font_009[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x10,0x00,0x70,0x00,0xF0,0x01,0xF0,0x03,0xF0,0x07,0xF0,0x0F,0xE0,0x1F,0xC0,0x3F,0x80,0x3F
};
static unsigned char font_00A[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0xF0,0xFF,0xF0,0xFF,0xF0,0xFF,0xF0,0xFF,0xF0,0xFF,0x10,0x80,0x00,0x00,0x00,0x00,0x10,0x00
};
static unsigned char font_00B[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x80,0x00,0xE0,0x00,0xF8,0x00,0xFC,0x00,0xFE,0x00,0xFF,0x80,0x7F,0xC0,0xFF,0xC0,0xFF
};
static unsigned char font_00C[]={ //(null)
0x00,0x7F,0x00,0x7E,0x00,0xFC,0x00,0xFC,0x00,0xF8,0x00,0xF8,0x00,0xF8,0x00,0xF8,0x00,0xF8,
0x00,0xF8,0x00,0xF8,0x10,0xF8,0x30,0xF8,0x70,0xF8,0xF0,0xFC,0xF0,0xFD,0xF0,0x7F,0xF0,0x7F
};
static unsigned char font_00D[]={ //(null)
0x30,0x00,0x70,0x00,0xF0,0x00,0xF0,0x01,0xF0,0x03,0xF0,0x07,0xF0,0x0F,0xF0,0x1F,0xF0,0x3F,
0xE0,0x7F,0xC0,0xFF,0x80,0xFF,0x00,0xFF,0x00,0xFE,0x00,0xFC,0x00,0xF8,0x00,0xF0,0x00,0xE0
};
static unsigned char font_00E[]={ //(null)
0xE0,0xFF,0xE0,0xFF,0xF0,0xFF,0xF0,0xFB,0xF0,0xF1,0xF0,0xE1,0xF0,0xC1,0xF0,0x81,0xF0,0x01,
0xF0,0x01,0xF0,0x01,0xF0,0x01,0xF0,0x01,0xF0,0x01,0xF0,0x03,0xF0,0x03,0xE0,0x07,0xE0,0x0F
};
static unsigned char font_00F[]={ //(null)
0xF0,0x3F,0xF0,0x3F,0xF0,0x1F,0xF0,0x0F,0xF0,0x07,0xF0,0x03,0xF0,0x01,0x70,0x00,0x10,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
};
static unsigned char font_010[]={ //(null)
0x00,0xC0,0x00,0x80,0x00,0x00,0x10,0x80,0xF0,0xFF,0xF0,0xFF,0xF0,0xFF,0xF0,0xFF,0xF0,0xFF,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
};
static unsigned char font_011[]={ //(null)
0xC0,0x1F,0xC0,0x3F,0x80,0x7F,0x00,0xFF,0x00,0xFE,0x00,0xFC,0x00,0xF8,0x00,0xE0,0x00,0x80,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
};

/* --------------------------------------------------------------------- */
static unsigned char font_012[]={ //Volume bar
0x00,0x00,0x00,0x00,0xC0,0x3F,0xC0,0x3F,0xC0,0x3F,0xC0,0x3F,0xC0,0x3F,0xC0,0x3F,0xC0,0x3F,
0xC0,0x3F,0xC0,0x3F,0xC0,0x3F,0xC0,0x3F,0xC0,0x3F,0xC0,0x3F,0xC0,0x3F,0x00,0x00,0x00,0x00
};
static unsigned char font_013[]={ //Volume empty bar
0x00,0x00,0x00,0x00,0xC0,0x3F,0x40,0x20,0x40,0x20,0x40,0x20,0x40,0x20,0x40,0x20,0x40,0x20,
0x40,0x20,0x40,0x20,0x40,0x20,0x40,0x20,0x40,0x20,0x40,0x20,0xC0,0x3F,0x00,0x00,0x00,0x00
};
static unsigned char font_014[]={ //(null)
0x00,0x00,0x60,0x00,0xE0,0x00,0xA0,0x41,0x30,0x23,0x20,0x7E,0x60,0x4C,0xA0,0x4C,0x20,0x4F,
0x20,0x4F,0xA0,0x4C,0x60,0x7E,0x20,0x13,0xB0,0x21,0xE0,0x40,0x60,0x00,0x00,0x00,0x00,0x00
};
static unsigned char font_015[]={ //Volume
0x00,0x00,0x60,0x00,0xe0,0x00,0xa0,0x01,0x20,0x03,0x20,0x7e,0x20,0x4c,0x20,0x4c,0x20,0x4c,
0x20,0x4c,0x20,0x4c,0x20,0x7e,0x20,0x03,0xa0,0x01,0xe0,0x00,0x60,0x00,0x00,0x00,0x00,0x00
};
static unsigned char font_016[]={ //Volume
0x00,0x00,0x00,0x00,0x00,0x04,0x00,0x02,0x00,0x11,0x80,0x08,0x80,0x44,0x80,0x24,0x80,0x24,
0x80,0x44,0x80,0x08,0x00,0x11,0x00,0x02,0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
};
static unsigned char font_017[]={ //Brightness ICON
0x00,0x00,0x10,0x00,0x10,0x00,0x10,0x03,0xb0,0x03,0xf0,0x01,0xc0,0x00,0x80,0x01,0x80,0x0f,
0x80,0x0f,0x80,0x01,0xc0,0x00,0xf0,0x01,0xb0,0x03,0x10,0x03,0x10,0x00,0x10,0x00,0x00,0x00
};
static unsigned char font_018[]={ //Brightness ICON
0x00,0x00,0x00,0x80,0x00,0x80,0x00,0x8c,0x00,0xdc,0x00,0xf8,0x00,0x30,0x00,0x18,0x00,0x1f,
0x00,0x1f,0x00,0x18,0x00,0x30,0x00,0xf8,0x00,0xdc,0x00,0x8c,0x00,0x80,0x00,0x80,0x00,0x00
};
static unsigned char font_019[]={ //Contrast ICON
0x00,0x00,0x00,0x00,0x30,0x00,0xf0,0x00,0xc0,0x01,0x00,0x03,0x00,0x03,0x00,0x06,0x00,0x06,
0x00,0x06,0x00,0x06,0x00,0x03,0x00,0x03,0xc0,0x01,0xf0,0x00,0x30,0x00,0x00,0x00,0x00,0x00
};
static unsigned char font_01A[]={ //Contrast ICON
0x00,0x00,0x00,0x00,0x00,0xc0,0x00,0xf0,0x00,0xf8,0x00,0xfc,0x00,0xfc,0x00,0xfe,0x00,0xfe,
0x00,0xfe,0x00,0xfe,0x00,0xfc,0x00,0xfc,0x00,0xf8,0x00,0xf0,0x00,0xc0,0x00,0x00,0x00,0x00
};
static unsigned char font_01B[]={ //Phase ICON
0x00,0x00,0x00,0x00,0xf0,0x3f,0xf0,0x3f,0x00,0x30,0xa0,0x32,0xa0,0x32,0xa0,0x32,0xa0,0x32,
0xa0,0x32,0xa0,0x32,0xa0,0x32,0xa0,0x32,0x00,0x30,0xf0,0x3f,0xf0,0x3f,0x00,0x00,0x00,0x00
};
static unsigned char font_01C[]={ //Phase ICON
0x00,0x00,0x00,0x00,0xc0,0xff,0xc0,0xff,0xc0,0x00,0xc0,0x44,0xc0,0x44,0xc0,0x44,0xc0,0x44,
0xc0,0x44,0xc0,0x44,0xc0,0x44,0xc0,0x44,0xc0,0x00,0xc0,0xff,0xc0,0xff,0x00,0x00,0x00,0x00
};
static unsigned char font_01D[]={ //
0x00,0x00,0x00,0x00,0xF0,0x01,0x00,0x02,0x30,0x04,0x30,0x08,0x80,0x09,0x80,0x09,0x10,0x08,
0x60,0x08,0xC0,0x06,0xF0,0x01,0x00,0x03,0x00,0x06,0x00,0x0c,0x00,0x08,0x00,0x00,0x00,0x00
};
static unsigned char font_01E[]={ //
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xE0,0x00,0x10,0x00,0x08,0x00,0x64,0x00,0x62,0x00,0x01,  
0x00,0x19,0x00,0x19,0x00,0x01,0x00,0x8D,0x00,0x41,0x00,0x22,0x00,0x1C,0x00,0x00,0x00,0x00
};
static unsigned char font_01F[]={ //Color ICON
0x00,0x00,0x00,0x00,0xf0,0x3f,0xf0,0x3f,0x00,0x30,0x30,0x30,0x20,0x30,0x30,0x30,0x20,0x33,
0x00,0x34,0x80,0x35,0x80,0x34,0x00,0x33,0x00,0x30,0xf0,0x3f,0xf0,0x3f,0x00,0x00,0x00,0x00
};
static unsigned char font_020[]={ //Color ICON -> change to space.
//0x00,0x00,0x00,0x00,0xc0,0xff,0xc0,0xff,0xc0,0x00,0xc0,0x80,0xc0,0x40,0xc0,0x80,0xc0,0x5c,
//0xc0,0x12,0xc0,0x1c,0xc0,0x12,0xc0,0x1c,0xc0,0x00,0xc0,0xff,0xc0,0xff,0x00,0x00,0x00,0x00
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
};
static unsigned char font_021[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,
0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x00,0x00,0x00,0x00,0x06,0x00,0x06,0x00,0x00,0x00,0x00
};
static unsigned char font_022[]={ //VPOS ICON
0x00,0x00,0xf0,0x03,0x00,0x02,0xf0,0x3f,0xf0,0x3f,0x00,0x30,0x00,0x32,0xf0,0x33,0x00,0x30,
0x00,0x30,0x00,0x30,0x00,0x30,0x00,0x30,0x00,0x30,0xf0,0x3f,0xf0,0x3f,0x00,0x00,0x00,0x00
};
static unsigned char font_023[]={ //VPOS ICON
0x00,0x00,0x00,0xfc,0x00,0x04,0xc0,0xff,0xc0,0xff,0xc0,0x00,0xc0,0x04,0xc0,0xfc,0xc0,0x00,
0xc0,0x00,0xc0,0x00,0xc0,0x00,0xc0,0x00,0xc0,0x00,0xc0,0xff,0xc0,0xff,0x00,0x00,0x00,0x00
};
static unsigned char font_024[]={ //Pixel Clock ICON
0x00,0x00,0xf0,0x3f,0xf0,0x3f,0x00,0x30,0xf0,0x37,0x00,0x34,0x40,0x34,0xc0,0x34,0xf0,0x35,
0xc0,0x34,0x40,0x34,0x00,0x34,0xf0,0x37,0x00,0x30,0xf0,0x3f,0xf0,0x3f,0x00,0x00,0x00,0x00
};
static unsigned char font_025[]={ //Pixel Clock ICON
0x00,0x00,0xc0,0xff,0xc0,0xff,0xc0,0x00,0xc0,0xfe,0xc0,0x02,0xc0,0x22,0xc0,0x32,0xc0,0xfa,
0xc0,0x32,0xc0,0x22,0xc0,0x02,0xc0,0xfe,0xc0,0x00,0xc0,0xff,0xc0,0xff,0x00,0x00,0x00,0x00
};
static unsigned char font_026[]={ //HPOS ICON
0x00,0x00,0x00,0x00,0xf0,0x3f,0xf0,0x3f,0x00,0x30,0x00,0x30,0x00,0x30,0x00,0x30,0x00,0x30,
0x00,0x30,0x00,0x30,0x00,0x30,0x00,0x30,0x00,0x30,0xf0,0x3f,0xf0,0x3f,0x00,0x00,0x00,0x00
};
static unsigned char font_027[]={ //The same as font_060
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x00,0x30,0x00,0x30,0x00,0x30,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
};
static unsigned char font_028[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x03,0x00,0x06,0x00,0x0C,0x00,0x0C,0x00,0x0C,
0x00,0x0C,0x00,0x0C,0x00,0x0C,0x00,0x0C,0x00,0x0C,0x00,0x06,0x00,0x03,0x00,0x00,0x00,0x00
};
static unsigned char font_029[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0C,0x00,0x06,0x00,0x03,0x00,0x03,0x00,0x03,
0x00,0x03,0x00,0x03,0x00,0x03,0x00,0x03,0x00,0x03,0x00,0x06,0x00,0x0C,0x00,0x00,0x00,0x00
};
static unsigned char font_02A[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x20,0x22,0x40,0x12,0x80,0x0A,
0x00,0x07,0xE0,0x3F,0x00,0x07,0x80,0x0A,0x40,0x12,0x20,0x22,0x00,0x00,0x00,0x00,0x00,0x00
};
static unsigned char font_02B[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,
0xE0,0x7F,0xE0,0x7F,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x00,0x00,0x00,0x00,0x00
};
static unsigned char font_02C[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x07,0x00,0x07,0x00,0x03,0x00,0x03,0x00,0x06,0x00,0x00,0x00,0x00
};
static unsigned char font_02D[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0xE0,0x7F,0xE0,0x7F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
};
static unsigned char font_02E[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x06,0x00,0x06,0x00,0x00,0x00,0x00
};
static unsigned char font_02F[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x03,0x00,0x03,0x00,0x03,0x00,0x06,0x00,0x06,
0x00,0x06,0x00,0x0C,0x00,0x0C,0x00,0x0C,0x00,0x18,0x00,0x18,0x00,0x18,0x00,0x00,0x00,0x00
};
static unsigned char font_030[]={ //Digital 0
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x1f,0xc0,0x3f,0xe0,0x70,0xe0,0x61,0x60,0x63,
0x60,0x66,0x60,0x6c,0x60,0x78,0x60,0x70,0xe0,0x60,0xc0,0x3f,0x80,0x1f,0x00,0x00,0x00,0x00
};
static unsigned char font_031[]={ //Digital 1
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x06,0x00,0x0e,0x00,0x1e,0x00,0x06,0x00,0x06,
0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x80,0x1f,0x80,0x1f,0x00,0x00,0x00,0x00
};
static unsigned char font_032[]={ //Digital 2
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x0f,0xc0,0x1f,0xe0,0x38,0x60,0x30,0xe0,0x00,
0xc0,0x01,0x80,0x03,0x00,0x07,0x00,0x0e,0x00,0x1c,0xe0,0x3f,0xe0,0x3f,0x00,0x00,0x00,0x00
};
static unsigned char font_033[]={ //Digital 3
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x1f,0xc0,0x3f,0xe0,0x70,0x60,0x60,0x60,0x00,
0xc0,0x03,0xc0,0x03,0x60,0x00,0x60,0x60,0xe0,0x70,0xc0,0x3f,0x80,0x1f,0x00,0x00,0x00,0x00
};
static unsigned char font_034[]={ //Digital 4
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xc0,0x03,0xc0,0x07,0xc0,0x0e,0xc0,0x1c,0xc0,0x38,
0xc0,0x70,0xc0,0x60,0xc0,0x60,0xe0,0x7f,0xe0,0x7f,0xc0,0x00,0xc0,0x00,0x00,0x00,0x00,0x00
};
static unsigned char font_035[]={ //Digital 5
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xe0,0x7f,0xe0,0x7f,0x00,0x60,0x00,0x60,0x80,0x7f,
0xc0,0x7f,0xe0,0x00,0x60,0x00,0x60,0x60,0xe0,0x70,0xc0,0x3f,0x80,0x1f,0x00,0x00,0x00,0x00
};
static unsigned char font_036[]={  //Digital 6
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x1f,0xc0,0x3f,0xe0,0x70,0x60,0x60,0x00,0x60,
0x80,0x6f,0xc0,0x7f,0xe0,0x70,0x60,0x60,0xe0,0x70,0xc0,0x3f,0x80,0x1f,0x00,0x00,0x00,0x00
};
static unsigned char font_037[]={ //Digital 7
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xe0,0x7f,0xe0,0x7f,0xe0,0x00,0xc0,0x01,0x80,0x03,
0x00,0x07,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x00,0x00,0x00
};
static unsigned char font_038[]={ //Digital 8
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x1f,0xc0,0x3f,0xe0,0x70,0x60,0x60,0xc0,0x30,
0xc0,0x3f,0xe0,0x7f,0xe0,0x70,0x60,0x60,0xe0,0x70,0xc0,0x3f,0x80,0x1f,0x00,0x00,0x00,0x00
};
static unsigned char font_039[]={ //Digital 9
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x1f,0xc0,0x3f,0xe0,0x70,0x60,0x60,0xe0,0x70,
0xe0,0x3f,0xe0,0x1f,0x60,0x00,0x60,0x60,0xe0,0x70,0xc0,0x3f,0x80,0x1f,0x00,0x00,0x00,0x00
};
static unsigned char font_03A[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x07,0x00,0x07,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x07,0x00,0x07,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
};
static unsigned char font_03B[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x07,0x00,0x07,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x07,0x00,0x07,0x00,0x03,0x00,0x06,0x00,0x00,0x00,0x00
};
static unsigned char font_03C[]={ //(null)
0x00,0x00,0x00,0x00,0xC0,0x00,0x80,0x01,0x00,0x03,0x00,0x06,0x00,0x0C,0x00,0x18,0x00,0x30,
0x00,0x60,0x00,0x30,0x00,0x18,0x00,0x0C,0x00,0x06,0x00,0x03,0x80,0x01,0xC0,0x00,0x00,0x00
};
static unsigned char font_03D[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xE0,0x7F,0xE0,0x7F,
0x00,0x00,0x00,0x00,0xE0,0x7F,0xE0,0x7F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
};
static unsigned char font_03E[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x30,0x00,0x18,0x00,0x0C,0x00,0x06,0x00,0x03,0x80,0x01,0xC0,0x00,
0x60,0x00,0xC0,0x00,0x80,0x01,0x00,0x03,0x00,0x06,0x00,0x0C,0x00,0x18,0x00,0x30,0x00,0x00
};
static unsigned char font_03F[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0F,0x80,0x19,0xC0,0x30,0xC0,0x30,0xC0,0x00,
0x80,0x01,0x00,0x03,0x00,0x06,0x00,0x06,0x00,0x00,0x00,0x00,0x00,0x06,0x00,0x00,0x00,0x00
};
static unsigned char font_040[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xC0,0x1F,0x60,0x70,0x30,0xC0,0x10,0x8F,0x90,0x99,
0x90,0xB1,0x90,0xB1,0xB0,0xB3,0xE0,0x9F,0x00,0x80,0xE0,0xE0,0x80,0x3F,0x00,0x00,0x00,0x00
};
static unsigned char font_041[]={ // A
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x06,0x00,0x0f,0x80,0x1f,0xc0,0x39,0xe0,0x70,
0x60,0x60,0xe0,0x7f,0xe0,0x7f,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x00,0x00,0x00,0x00
};
static unsigned char font_042[]={ // B
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x7f,0xc0,0x7f,0xe0,0x30,0x60,0x30,0xc0,0x30,
0xc0,0x3f,0xe0,0x3f,0xe0,0x30,0x60,0x30,0xe0,0x30,0xc0,0x7f,0x80,0x7f,0x00,0x00,0x00,0x00
};
static unsigned char font_043[]={ // C
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x1f,0xc0,0x3f,0xe0,0x70,0x60,0x60,0x00,0x60,
0x00,0x60,0x00,0x60,0x00,0x60,0x60,0x60,0xe0,0x70,0xc0,0x3f,0x80,0x1f,0x00,0x00,0x00,0x00
};
static unsigned char font_044[]={ // D
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x7f,0xc0,0x7f,0xe0,0x60,0x60,0x60,0x60,0x60,
0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0xe0,0x60,0xc0,0x7f,0x80,0x7f,0x00,0x00,0x00,0x00
};
static unsigned char font_045[]={ // E
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xe0,0x7f,0xe0,0x7f,0x00,0x60,0x00,0x60,0x00,0x60,
0xc0,0x7f,0xc0,0x7f,0x00,0x60,0x00,0x60,0x00,0x60,0xe0,0x7f,0xe0,0x7f,0x00,0x00,0x00,0x00
};
static unsigned char font_046[]={ // F
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xe0,0x7f,0xe0,0x7f,0x00,0x60,0x00,0x60,0x00,0x60,
0xc0,0x7f,0xc0,0x7f,0x00,0x60,0x00,0x60,0x00,0x60,0x00,0x60,0x00,0x60,0x00,0x00,0x00,0x00
};
static unsigned char font_047[]={ // G
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x1f,0xc0,0x3f,0xe0,0x70,0x60,0x60,0x00,0x60,
0xe0,0x63,0xe0,0x63,0x60,0x60,0xe0,0x60,0xe0,0x71,0xe0,0x3f,0x60,0x1f,0x00,0x00,0x00,0x00
};
static unsigned char font_048[]={ // H
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,
0xe0,0x7f,0xe0,0x7f,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x00,0x00,0x00,0x00
};
static unsigned char font_049[]={ // I
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x1f,0x80,0x1f,0x00,0x06,0x00,0x06,0x00,0x06,
0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x80,0x1f,0x80,0x1f,0x00,0x00,0x00,0x00
};
static unsigned char font_04A[]={ // J
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xe0,0x0f,0xe0,0x0f,0x80,0x01,0x80,0x01,0x80,0x01,
0x80,0x01,0x80,0x01,0x80,0x01,0x80,0x61,0x80,0x73,0x00,0x3f,0x00,0x1e,0x00,0x00,0x00,0x00
};
static unsigned char font_04B[]={ // K
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xc0,0x60,0xc0,0x61,0x80,0x63,0x00,0x67,0x00,0x6e,
0x00,0x7c,0x00,0x7c,0x00,0x6e,0x00,0x67,0x80,0x63,0xc0,0x61,0xc0,0x60,0x00,0x00,0x00,0x00
};
static unsigned char font_04C[]={ // L
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x60,0x00,0x60,0x00,0x60,0x00,0x60,0x00,0x60,
0x00,0x60,0x00,0x60,0x00,0x60,0x00,0x60,0x00,0x60,0xe0,0x7f,0xe0,0x7f,0x00,0x00,0x00,0x00
};
static unsigned char font_04D[]={ // M
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xe0,0x70,0xe0,0x79,0xe0,0x7f,0x60,0x6f,0x60,0x66,
0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x00,0x00,0x00,0x00
};
static unsigned char font_04E[]={ // N
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x60,0x60,0x60,0x60,0x60,0x70,0x60,0x78,0x60,0x7c,
0x60,0x6e,0x60,0x67,0xe0,0x63,0xe0,0x61,0xe0,0x60,0x60,0x60,0x60,0x60,0x00,0x00,0x00,0x00
};
static unsigned char font_04F[]={ // O
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x1f,0xc0,0x3f,0xe0,0x70,0x60,0x60,0x60,0x60,
0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0xe0,0x70,0xc0,0x3f,0x80,0x1f,0x00,0x00,0x00,0x00
};
static unsigned char font_050[]={ // P
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x7f,0xc0,0x7f,0xe0,0x60,0x60,0x60,0xe0,0x60,
0xc0,0x7f,0x80,0x7f,0x00,0x60,0x00,0x60,0x00,0x60,0x00,0x60,0x00,0x60,0x00,0x00,0x00,0x00
};
static unsigned char font_051[]={ // Q
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x1f,0xc0,0x3f,0xe0,0x70,0x60,0x60,0x60,0x60,
0x60,0x60,0x60,0x60,0x60,0x67,0xe0,0x73,0xc0,0x3f,0xe0,0x1e,0x60,0x00,0x00,0x00,0x00,0x00
};
static unsigned char font_052[]={ // R
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x7f,0xc0,0x7f,0xe0,0x60,0x60,0x60,0xe0,0x60,
0xc0,0x7f,0x80,0x7f,0x00,0x6e,0x00,0x67,0x80,0x63,0xc0,0x61,0xe0,0x60,0x00,0x00,0x00,0x00
};
static unsigned char font_053[]={ // S
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x1f,0xc0,0x3f,0xe0,0x70,0x60,0x60,0x00,0x70,
0x80,0x3f,0xc0,0x1f,0xe0,0x00,0x60,0x60,0xe0,0x70,0xc0,0x3f,0x80,0x1f,0x00,0x00,0x00,0x00
};
static unsigned char font_054[]={ // T
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xe0,0x7f,0xe0,0x7f,0x00,0x06,0x00,0x06,0x00,0x06,
0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x00,0x00,0x00
};
static unsigned char font_055[]={ // U
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,
0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0xe0,0x70,0xc0,0x3f,0x80,0x1f,0x00,0x00,0x00,0x00
};
static unsigned char font_056[]={ // V
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,
0x60,0x60,0x60,0x60,0xe0,0x70,0xc0,0x39,0x80,0x1f,0x00,0x0f,0x00,0x06,0x00,0x00,0x00,0x00
};
static unsigned char font_057[]={ // W
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,
0x60,0x60,0x60,0x66,0x60,0x6f,0xe0,0x7f,0xe0,0x79,0xe0,0x70,0x60,0x60,0x00,0x00,0x00,0x00
};
static unsigned char font_058[]={ // X
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x60,0x60,0x60,0x60,0xe0,0x70,0xc0,0x39,0x80,0x1f,
0x00,0x0f,0x00,0x0f,0x80,0x1f,0xc0,0x39,0xe0,0x70,0x60,0x60,0x60,0x60,0x00,0x00,0x00,0x00
};
static unsigned char font_059[]={ // Y
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x60,0x60,0x60,0x60,0xe0,0x70,0xc0,0x30,0xc0,0x39,
0x80,0x19,0x80,0x1f,0x00,0x0f,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x00,0x00,0x00
};
static unsigned char font_05A[]={ // Z
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xe0,0x7f,0xe0,0x7f,0xe0,0x00,0xc0,0x01,0x80,0x03,
0x00,0x07,0x00,0x0e,0x00,0x1c,0x00,0x38,0x00,0x70,0xe0,0x7f,0xe0,0x7f,0x00,0x00,0x00,0x00
};
static unsigned char font_05B[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3C,0x00,0x30,0x00,0x30,0x00,0x30,0x00,0x30,0x00,0x30,
0x00,0x30,0x00,0x30,0x00,0x30,0x00,0x30,0x00,0x30,0x00,0x30,0x00,0x3C,0x00,0x00,0x00,0x00
};
static unsigned char font_05C[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x18,0x00,0x18,0x00,0x18,0x00,0x0C,0x00,0x0C,0x00,0x0C,
0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x03,0x00,0x03,0x00,0x03,0x00,0x00,0x00,0x00,0x00,0x00
};
static unsigned char font_05D[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0xC0,0x03,0xC0,0x00,0xC0,0x00,0xC0,0x00,0xC0,0x00,0xC0,0x00,
0xC0,0x00,0xC0,0x00,0xC0,0x00,0xC0,0x00,0xC0,0x00,0xC0,0x00,0xC0,0x03,0x00,0x00,0x00,0x00
};
static unsigned char font_05E[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x06,0x00,0x0F,0x80,0x19,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
};
static unsigned char font_05F[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xC0,0x3F,0x00,0x00,0x00,0x00
};
static unsigned char font_060[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x00,0x30,0x00,0x30,0x00,0x30,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
};
static unsigned char font_061[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3F,0x80,0x61,
0x80,0x01,0x80,0x01,0x80,0x3F,0x80,0x61,0x80,0x61,0x80,0x61,0xC0,0x3F,0x00,0x00,0x00,0x00
};
static unsigned char font_062[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x00,0x30,0x00,0x30,0x00,0x3F,0x80,0x39,
0xC0,0x30,0xC0,0x30,0xC0,0x30,0xC0,0x30,0xC0,0x30,0x80,0x39,0x00,0x3F,0x00,0x00,0x00,0x00
};
static unsigned char font_063[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0F,0x80,0x19,
0xC0,0x30,0x00,0x30,0x00,0x30,0x00,0x30,0xC0,0x30,0x80,0x19,0x00,0x0F,0x00,0x00,0x00,0x00
};
static unsigned char font_064[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xC0,0x00,0xC0,0x00,0xC0,0x00,0xC0,0x0F,0xC0,0x19,
0xC0,0x30,0xC0,0x30,0xC0,0x30,0xC0,0x30,0xC0,0x30,0xC0,0x19,0xC0,0x0F,0x00,0x00,0x00,0x00
};
static unsigned char font_065[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0F,0x80,0x19,
0xC0,0x30,0xC0,0x30,0xC0,0x3F,0x00,0x30,0xC0,0x30,0x80,0x19,0x00,0x0F,0x00,0x00,0x00,0x00
};
static unsigned char font_066[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x03,0x00,0x06,0x00,0x06,0x80,0x1F,0x00,0x06,
0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x00,0x00,0x00
};
static unsigned char font_067[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xC0,0x0F,0xC0,0x19,0xC0,0x30,0xC0,0x30,
0xC0,0x30,0xC0,0x30,0xC0,0x30,0xC0,0x19,0xC0,0x0F,0xC0,0x00,0xC0,0x00,0x80,0x01,0x00,0x1F
};
static unsigned char font_068[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x00,0x30,0x00,0x30,0x00,0x3F,0x80,0x39,
0xC0,0x30,0xC0,0x30,0xC0,0x30,0xC0,0x30,0xC0,0x30,0xC0,0x30,0xC0,0x30,0x00,0x00,0x00,0x00
};
static unsigned char font_069[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x06,0x00,0x00,0x00,0x00,0x00,0x06,0x00,0x06,
0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x00,0x00,0x00
};
static unsigned char font_06A[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x06,0x00,0x00,0x00,0x00,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,
0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x0C
};
static unsigned char font_06B[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x00,0x30,0x00,0x30,0x80,0x31,0x00,0x33,
0x00,0x36,0x00,0x3C,0x00,0x3C,0x00,0x36,0x00,0x33,0x80,0x31,0xC0,0x30,0x00,0x00,0x00,0x00
};
static unsigned char font_06C[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,
0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x00,0x00,0x00
};
static unsigned char font_06D[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xC0,0xF9,0x60,0xEF,
0x30,0xC6,0x30,0xC6,0x30,0xC6,0x30,0xC6,0x30,0xC6,0x30,0xC6,0x30,0xC6,0x00,0x00,0x00,0x00
};
static unsigned char font_06E[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3F,0x80,0x39,
0xC0,0x30,0xC0,0x30,0xC0,0x30,0xC0,0x30,0xC0,0x30,0xC0,0x30,0xC0,0x30,0x00,0x00,0x00,0x00
};
static unsigned char font_06F[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0F,0x80,0x19,
0xC0,0x30,0xC0,0x30,0xC0,0x30,0xC0,0x30,0xC0,0x30,0x80,0x19,0x00,0x0F,0x00,0x00,0x00,0x00
};
static unsigned char font_070[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3F,0x80,0x39,0xC0,0x30,0xC0,0x30,
0xC0,0x30,0xC0,0x30,0xC0,0x30,0x80,0x39,0x00,0x3F,0x00,0x30,0x00,0x30,0x00,0x30,0x00,0x30
};
static unsigned char font_071[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xC0,0x0F,0xC0,0x19,0xC0,0x30,0xC0,0x30,
0xC0,0x30,0xC0,0x30,0xC0,0x30,0xC0,0x19,0xC0,0x0F,0xC0,0x00,0xC0,0x00,0xC0,0x00,0xC0,0x00
};
static unsigned char font_072[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x00,0x00,0x0B,0x00,0x0E,
0x00,0x0C,0x00,0x0C,0x00,0x0C,0x00,0x0C,0x00,0x0C,0x00,0x0C,0x00,0x0C,0x00,0x00,0x00,0x00
};
static unsigned char font_073[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1F,0x80,0x31,
0x00,0x30,0x00,0x30,0x00,0x1F,0x80,0x01,0x80,0x01,0x80,0x31,0x00,0x1F,0x00,0x00,0x00,0x00
};
static unsigned char font_074[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x06,0x00,0x06,0x00,0x06,0x80,0x1F,0x00,0x06,
0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x80,0x06,0x00,0x03,0x00,0x00,0x00,0x00
};
static unsigned char font_075[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xC0,0x30,0xC0,0x30,
0xC0,0x30,0xC0,0x30,0xC0,0x30,0xC0,0x30,0xC0,0x30,0xC0,0x19,0xC0,0x0F,0x00,0x00,0x00,0x00
};
static unsigned char font_076[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xC0,0x30,0xC0,0x30,
0x80,0x19,0x80,0x19,0x00,0x0F,0x00,0x0F,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x00,0x00,0x00
};
static unsigned char font_077[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0xC0,0x30,0xC6,
0x60,0x66,0x60,0x66,0xC0,0x3F,0xC0,0x3F,0x80,0x19,0x80,0x19,0x80,0x19,0x00,0x00,0x00,0x00
};
static unsigned char font_078[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xC0,0x30,0x80,0x19,
0x80,0x19,0x00,0x0F,0x00,0x06,0x00,0x0F,0x80,0x19,0x80,0x19,0xC0,0x30,0x00,0x00,0x00,0x00
};
static unsigned char font_079[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xC0,0x30,0xC0,0x30,
0x80,0x19,0x80,0x19,0x00,0x0F,0x00,0x0F,0x00,0x06,0x00,0x06,0x00,0x0C,0x00,0x0C,0x00,0x38
};
static unsigned char font_07A[]={ //(null)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x3F,0x80,0x01,
0x00,0x03,0x00,0x06,0x00,0x0C,0x00,0x0C,0x00,0x18,0x00,0x30,0x80,0x3F,0x00,0x00,0x00,0x00
};
static unsigned char font_07B[]={ //Enter
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x60,0x00,0x60,0x00,0x60,0x00,
0x60,0x10,0x60,0x30,0xE0,0x7F,0xE0,0x7F,0x00,0x30,0x00,0x10,0x00,0x00,0x00,0x00,0x00,0x00
};

unsigned char *AllFonts[]={
font_000,font_001,font_002,font_003,font_004,font_005,font_006,font_007,font_008,font_009,	
font_00A,font_00B,font_00C,font_00D,font_00E,font_00F,font_010,font_011,font_012,font_013,
font_014,font_015,font_016,font_017,font_018,font_019,font_01A,font_01B,font_01C,font_01D,
font_01E,font_01F,font_020,font_021,font_022,font_023,font_024,font_025,font_026,font_027,
font_028,font_029,font_02A,font_02B,font_02C,font_02D,font_02E,font_02F,font_030,font_031,
font_032,font_033,font_034,font_035,font_036,font_037,font_038,font_039,font_03A,font_03B,
font_03C,font_03D,font_03E,font_03F,font_040,font_041,font_042,font_043,font_044,font_045,
font_046,font_047,font_048,font_049,font_04A,font_04B,font_04C,font_04D,font_04E,font_04F,
font_050,font_051,font_052,font_053,font_054,font_055,font_056,font_057,font_058,font_059,
font_05A,font_05B,font_05C,font_05D,font_05E,font_05F,font_060,font_061,font_062,font_063,
font_064,font_065,font_066,font_067,font_068,font_069,font_06A,font_06B,font_06C,font_06D,
font_06E,font_06F,font_070,font_071,font_072,font_073,font_074,font_075,font_076,font_077,
font_078,font_079,font_07A,font_07B, NULL,
};


/* --------------------------------------------------------------------- */
static unsigned short terawins_osd_color_table[16] = 
{
	/* OSD color LUT. */
	//-- Null, Red, Green, Blue
	0x0000,  // 0: Transparant
  	0x0000,
	0x0fff,  // 2: White
	0x0fff,
	
  	0x0c11,  // 4: Red
	0x0c11,  
	0x00f0,  // 6: Green
	0x00f0,
  	
  	0x011c,  // 8: Blue
  	0x011c,  
  	0x0f40,  // A: Orange
  	0x0f40,
  	
  	0x0ddd,  // C: Gray
  	0x0ddd,
  	0x0000,	 // E: Black  
  	0x0000,  
};

/* --------------------------------------------------------------------- */
static int terawins_set_osd_cmd(unsigned char cfg_idx, unsigned char cfg_data)
{
	int ret_val = 0;
	//-- set osd control register address.
	ret_val = terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_CFG_INDEX, cfg_idx);
	
	//-- set osd control register data.
	ret_val |= terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_CFG_DATA, cfg_data);

	if(ret_val != 0){
		TW_MSG("\nTeraWins set osd cmd error!\n");
	}
	
	return ret_val;
}

/* --------------------------------------------------------------------- */
static int terawins_set_osd_ram_addr(unsigned short ram_addr)
{
	int ret_val = 0;
	
	//-- set osd ram address.
	ret_val = terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_RAM_ADDR_LO, (unsigned char)(ram_addr & 0x00FF));
	ret_val |= terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_RAM_ADDR_HI, (unsigned char)((ram_addr & 0xFF00) >> 8));

	if(ret_val != 0){
		//ret_val = terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_RAM_ADDR_LO, (unsigned char)(ram_addr & 0x00FF));
		//ret_val |= terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_RAM_ADDR_HI, (unsigned char)((ram_addr & 0xFF00) >> 8));
	}
	return ret_val;
}

/* --------------------------------------------------------------------- */
static int terawins_set_osd_ram_data(unsigned char ram_data)
{
	int ret_val = 0;

	//-- set osd ram data.
	ret_val = terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_RAM_DATA, ram_data);

	if(ret_val != 0){
		//ret_val = terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_RAM_DATA, ram_data);
	}
	
	return ret_val;
}

/* --------------------------------------------------------------------- */
static int terawins_load_osd_color(unsigned short *color_table)
{
	int ret_val = 0;
	unsigned long i;

	/* Load OSD color LUT. */	
	//-- Configure OSD color LUT.
	ret_val = terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_CFG_INDEX, OSD_COLOR_LUT_PORT);	

	//-- Load data.
	for(i = 0; i < 0x10; i++){
		ret_val |= terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_CFG_DATA, (unsigned char)(color_table[i] & 0x00FF));
		ret_val |= terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_CFG_DATA, (unsigned char)((color_table[i] & 0xFF00) >> 8));
	}

	if(ret_val != 0){
		TW_MSG("\nTeraWins load osd color error!\n");
	}

	return ret_val;	
}

/* --------------------------------------------------------------------- */
//struct _terawins_reg terawins_reg;
struct _terawins_line_info terawins_line_info;
//struct _terawins_window_info terawins_window_info;
struct _terawins_menu_info terawins_menu_info;


static unsigned char font_index = FONT_12x18;	// index of font type.
struct _font_info font_info[] = {
//-- font_size, width, height, font_nums
	{0x0000,    10,    16,     0},	// 10x16
	{0x0009,    12,    18,     0},	// 12x18
	{0x0013,    16,    24,     0},	// 16x24
};

/* --------------------------------------------------------------------- */
void terawins_clear_osd_ram(void)
{
	unsigned long i;
	unsigned long word_nums;

	word_nums = (TERAWINS_OSD_RAM_SIZE / 2);

//	TW_MSG("\nword_nums = %d, %x", word_nums, word_nums);
	
	//-- Clear OSD ram.(clear word.)
	terawins_set_osd_ram_addr(0x0000);
	for(i = 0; i < word_nums; i++){
		//low byte
		terawins_set_osd_ram_data(0x00);
		//high byte
		terawins_set_osd_ram_data(0x00);
	}
}

/* --------------------------------------------------------------------- */
unsigned char terawins_load_osd_fonts(unsigned char** _font_array)
{
	unsigned char i = 0;
	unsigned char j = 0;	
	unsigned char bytes_per_font;
	unsigned char *p;
	int _ack = 0;

	bytes_per_font = FONT_WIDTH_BYTES * FONT_HEIGHT;
	terawins_set_osd_ram_addr(0x0000);
	
	while(_font_array[i] != NULL){
		p = _font_array[i];
		for(j = 0; j < (bytes_per_font); j++){
			_ack |= terawins_set_osd_ram_data(p[j]);
		}
		i ++;
	}
	// flowbee 1007-09-10 dbg
	if(_ack != 0){
		TW_MSG("\nLoad Fonts error, font_num = %d\n", i);
	}
	
	return i;
}

/* --------------------------------------------------------------------- */
void terawins_set_osd_window_info(
				unsigned char r_start, unsigned char r_end,
				unsigned short c_start, unsigned short c_end,
				unsigned char win_en, unsigned char win_int, unsigned char win_color,
				unsigned char sha_en, unsigned char sha_gray, unsigned char sha_color,
				unsigned char sha_w, unsigned char sha_h,
				struct _terawins_window_info *_info )
{
	_info->row_start 	= ((win_en & 0x01) << 7) | ((win_int & 0x01) << 6) | (r_start & 0x3f);
	_info->row_end		= ((sha_en & 0x01) << 7) | ((sha_gray & 0x01) << 6) | (r_end & 0x3f);
	_info->col_start	= ((win_color & 0x07) << 5) | (c_start & 0x1f);
	_info->col_end		= ((sha_color & 0x07) << 5) | (c_end & 0x1f);
	_info->shadow_size	= ((sha_h & 0x0f) << 4) | (sha_w & 0x0f);
}

/* --------------------------------------------------------------------- */
void terawins_set_osd_menu_info(unsigned short h_start, 
										   unsigned short v_start,
										   unsigned char row_index,
										   unsigned char n_rows,	
						 				   struct _terawins_menu_info *_info)
{
	_info->h_start		= h_start;
	_info->v_start 		= v_start;
	_info->row_index	= row_index;
	_info->num_rows		= n_rows;
}

/* --------------------------------------------------------------------- */
void terawins_set_osd_line_info(int idx[30], unsigned char c_blink,
						 	unsigned char c_fg_c, unsigned char c_bg_c, 
						 	unsigned char rgap_bg, unsigned char rgap, 
						 	unsigned char r_chs, unsigned char r_cws,
						 	struct _terawins_line_info *_info)
{
	_info->char_idx		= idx;
	_info->char_attr 	= ((c_bg_c & 0x07) << 5) | ((c_blink & 0x01) << 4) | (c_fg_c & 0x0F);
	_info->row_attr 	= ((rgap_bg & 0x01) << 7) | ((rgap & 0x1F) << 2) 
							| ((r_chs & 0x01) << 1) | (r_cws & 0x01);
}

/* --------------------------------------------------------------------- */
void terawins_set_osd_window(struct _terawins_window_info *_info)
{
	/* Setup window info. */
	//TW_MSG("\nWindow Info:\n");
	terawins_set_osd_cmd(W1_START_ROW_NUM, _info->row_start);
	terawins_set_osd_cmd(W1_END_ROW_NUM, _info->row_end);
	terawins_set_osd_cmd(W1_START_COL_NUM, _info->col_start);
	terawins_set_osd_cmd(W1_END_COL_NUM, _info->col_end);	
	terawins_set_osd_cmd(W1_SHADOW_SIZE, _info->shadow_size);	
}

/* --------------------------------------------------------------------- */
void terawins_set_osd_line(struct _terawins_line_info *_info)
{
	unsigned char i = 0;
	int _ack = 0;
#if 0
	/* Fill line info. */
	//-- 30 chars each line.
	for(i = 0; i < 30; i ++){
		if(_info->char_idx[i] != -1){
			//-- low byte first.(char index.)
			_ack |= terawins_set_osd_ram_data(_info->char_idx[i]);
			//-- high byte.(char attribute.)
			_ack |= terawins_set_osd_ram_data(_info->char_attr);
		}
		else{
			//-- low byte first.(char index.)
			_ack |= terawins_set_osd_ram_data(0x00);
			//-- high byte.(char attribute.)
			_ack |= terawins_set_osd_ram_data(0x00);
		}
	}

	//-- It must be the blank char at the 31th word of the line info.
	_ack |= terawins_set_osd_ram_data(00);
	_ack |= terawins_set_osd_ram_data(00);

	//-- Row attr.
	_ack |= terawins_set_osd_ram_data(00);
	_ack |= terawins_set_osd_ram_data(_info->row_attr);
#else
	/* Fill line info. */
	//-- 30 chars each line.
	for(i = 0; i < 30; i ++){
		if(_info->char_idx[i] != -1){
			//-- low byte first.(char index.)
			if( (_ack = terawins_set_osd_ram_data(_info->char_idx[i])) != 0 ){
				TW_MSG("\nSet osd line error !!\n");
				return;
			}
			//-- high byte.(char attribute.)
			if( (_ack = terawins_set_osd_ram_data(_info->char_attr))  != 0 ){
				TW_MSG("\nSet osd line error !!\n");
				return;
			}
		}
		else{
			//-- low byte first.(char index.)
			if( (_ack = terawins_set_osd_ram_data(0x00)) != 0 ){
				TW_MSG("\nSet osd line error !!\n");
				return;
			}
			//-- high byte.(char attribute.)
			if( (_ack = terawins_set_osd_ram_data(0x00)) != 0 ){
				TW_MSG("\nSet osd line error !!\n");
				return;
			}
		}
	}

	//-- It must be the blank char at the 31th word of the line info.
	if( (_ack = terawins_set_osd_ram_data(0x00)) != 0 ){
		TW_MSG("\nSet osd line error !!\n");
		return;
	}
	if( (_ack = terawins_set_osd_ram_data(0x00)) != 0 ){
		TW_MSG("\nSet osd line error !!\n");
		return;
	}

	//-- Row attr.
	if( (_ack = terawins_set_osd_ram_data(0x00)) != 0 ){
		TW_MSG("\nSet osd line error !!\n");
		return;
	}
	if( (_ack = terawins_set_osd_ram_data(_info->row_attr)) != 0 ){
		TW_MSG("\nSet osd line error !!\n");
		return;
	}
#endif
	if(_ack != 0){
		TW_MSG("\nSet osd line error !!\n");
	}
}

/* --------------------------------------------------------------------- */
void terawins_clear_osd_line(unsigned char _num)
{
	unsigned char i, j;

	terawins_set_osd_ram_addr(CHAR_START_RAM_ADDR(font_info[font_index].font_nums));
	for(j = 0; j < _num; j ++){
		for(i = 0; i < 32; i++){
			//-- low byte first.(char index.)
			terawins_set_osd_ram_data(0x00);
			//-- high byte.(char attribute.)
			terawins_set_osd_ram_data(0x00);
		}
	}
}

/* --------------------------------------------------------------------- */
void terawins_set_osd_menu(struct _terawins_menu_info *_info)
{
	unsigned char h_lbyte, h_hbyte;
	unsigned char v_lbyte, v_hbyte;	
	unsigned char start_addr, end_addr;
	
	/* Set terawins osd menu info. */
	//-- Set start address.
	h_lbyte = (unsigned char)(_info->h_start & 0x00FF);
	h_hbyte = (unsigned char)((_info->h_start & 0x0700) >> 8);
	v_lbyte = (unsigned char)(_info->v_start & 0x00FF);
	v_hbyte = (unsigned char)((_info->v_start & 0x0700) >> 8);
	terawins_set_osd_cmd(CHAR_DLY1, (h_hbyte | (v_hbyte << 4)));
	terawins_set_osd_cmd(CHAR_DLY2, h_lbyte);
	terawins_set_osd_cmd(CHAR_DLY3, v_lbyte);

//	TW_MSG("\nCHAR_DLY1: %02x", (h_hbyte | (v_hbyte << 4)));
//	TW_MSG("\nCHAR_DLY2: %02x", h_lbyte);
//	TW_MSG("\nCHAR_DLY3: %02x", v_lbyte);	

	//-- Set end address of menu info.
	if(_info->num_rows < 1){
		_info->num_rows = 0;
		TW_MSG("\nOSD input row error !!");
	}

	start_addr = CHAR_START_ADDR(font_info[font_index].font_nums) + _info->row_index;
	terawins_set_osd_cmd(CHAR_RAM_START_ADDR, start_addr);
	
	end_addr = start_addr + (_info->num_rows - 1);
	terawins_set_osd_cmd(CHAR_RAM_STOP_ADDR, end_addr);
//	TW_MSG("\nCHAR_RAM_START_ADDR: %02x", CHAR_START_ADDR(font_info[font_index].font_nums));
//	TW_MSG("\nCHAR_RAM_STOP_ADDR: %02x", end_addr);	
}

/* --------------------------------------------------------------------- */
void terawins_set_osd_alpha_blending(unsigned char mode, unsigned char _val)
{
	terawins_set_osd_cmd(ALPHA_BLENDING_CTRL, (((mode&0x03) << 6) | (_val&0x0f)));
}

/* --------------------------------------------------------------------- */
void terawins_set_osd_blinking(unsigned char blk_freq, unsigned char duty)
{
	terawins_set_osd_cmd(BLINKING_CTRL, (((blk_freq&0x03) << 2) | (duty&0x03)));
}

/* --------------------------------------------------------------------- */
void terawins_enable_osd_win(struct _terawins_window_info *_info)
{
	_info->row_start |= 0x80;
	/* Enable terawins osd window at char mode. */
	terawins_set_osd_cmd(W1_START_ROW_NUM, _info->row_start);
}

/* --------------------------------------------------------------------- */
void terawins_disable_osd_win(struct _terawins_window_info *_info)
{
	/* Disable terawins osd window. */
	_info->row_start &= ~(0x80);
	/* Enable terawins osd window at char mode. */
	terawins_set_osd_cmd(W1_START_ROW_NUM, _info->row_start);
}

/* --------------------------------------------------------------------- */
void terawins_enable_osd(void)
{
	/* Enable terawins osd at char mode. */
	terawins_set_osd_cmd(OSD_CTRL, 0x80);
}

/* --------------------------------------------------------------------- */
void terawins_disable_osd(void)
{
	/* Disable terawins osd. */
	terawins_set_osd_cmd(OSD_CTRL, 0x00);
}

/* --------------------------------------------------------------------- */
#define WORDS_PER_ROW				32
#define FONTS_PER_ROW				30
#define FONT_W						12
#define FONT_H						18
#define WIDTH_PER_ROW				(FONTS_PER_ROW * FONT_W)

#if (CONFIG_ARCPGU_DISPTYPE == 3)
	#define LCD_W					800
	#define LCD_H					480
	#define TERAWINS_W				1500
#elif (CONFIG_ARCPGU_DISPTYPE == 4)
	#define LCD_W					800
	#define LCD_H					480
	#define TERAWINS_W				1500
#else
	#define LCD_W					800
	#define LCD_H					600
	#define TERAWINS_W				1500
#endif

#define TERAWINS_H					LCD_H

#define LCD_START_X					((LCD_W - WIDTH_PER_ROW) / 2)
#define TERAWINS_START_X			((TERAWINS_W * LCD_START_X) / LCD_W)

// ICON
#define ICON_ROWS					(3)
#define FONT_H_PER_ICON_ROW			(1)
#define ICON_LINES					(ICON_ROWS * FONT_H_PER_ICON_ROW * FONT_H)
// SUBTITLE
#define SUBTITLE_ROWS				(3)
#define FONT_H_PER_SUBTITLE_ROW		(2)
#define SUBTITLE_LINES				(SUBTITLE_ROWS * FONT_H_PER_SUBTITLE_ROW * FONT_H)
// MUTE_VOL
#define MUTE_VOL_ROWS				(1)
#define FONT_H_PER_MUTE_VOL_ROW		(1)
#define MUTE_VOL_LINES				(MUTE_VOL_ROWS * FONT_H_PER_MUTE_VOL_ROW * FONT_H)
// PLAY TIME
#define TIME_ROWS					(1)
#define FONT_H_PER_TIME_ROW			(2)
#define TIME_LINES					(TIME_ROWS * FONT_H_PER_TIME_ROW * FONT_H)
// GAP
#define FONT_H_PER_GAP_ROW			(1)


#define LCD_START_Y					((LCD_H - ICON_LINES) / 2)	

#define TOTAL_ICON_COLS				(2)
#define TOTAL_TIME_COLS				(19)
#define TOTAL_MUTE_COLS				(4)
#define TOTAL_VOL_COLS				(14)
#define TOTAL_SUBTITLE_COLS			(30)

#define ICON_START_COL				( (FONTS_PER_ROW - TOTAL_ICON_COLS) / 2 )
#define TIME_START_COL				( (FONTS_PER_ROW - TOTAL_TIME_COLS) / 2 )
#define MUTE_START_COL				( (FONTS_PER_ROW - TOTAL_MUTE_COLS) / 2 )
#define VOL_START_COL				( (FONTS_PER_ROW - TOTAL_VOL_COLS) / 2 )
#define SUBTITLE_START_COL			( (FONTS_PER_ROW - TOTAL_SUBTITLE_COLS) / 2 )


/* --------------------------------------------------------------------- */
uint8 icon_start_row = 0, time_start_row = 0, mute_vol_start_row = 0, subtitle_start_row = 0;
uint8 total_rows = 0;
uint16 terawins_start_y = 0;
int16 gap_start_row = 0, gap_rows = 0, gap_lines = 0;
void terawins_set_display_range(void)
{

	gap_lines = (LCD_H - LCD_START_Y) - (ICON_LINES + SUBTITLE_LINES + MUTE_VOL_LINES + TIME_LINES);

	//-- Modify gap line be multiples of FONT_H
	if(gap_lines < 0){
		TW_MSG("\nGap lines < 0 in terawins_set_display_range()\n");
		return;
	}
	else if((gap_lines % FONT_H) == 0){
		terawins_start_y = LCD_START_Y;
	}
	else{
		terawins_start_y = LCD_START_Y - (FONT_H - (gap_lines % FONT_H));
		gap_lines = gap_lines + (FONT_H - (gap_lines % FONT_H));
	}
	
	//-- Calculate the start row of each item.
	gap_rows = gap_lines / FONT_H;
	total_rows = gap_rows + ICON_ROWS + SUBTITLE_ROWS + MUTE_VOL_ROWS + TIME_ROWS;

	icon_start_row 		= 0;
	gap_start_row 		= icon_start_row + ICON_ROWS;
	subtitle_start_row 	= gap_start_row + gap_rows;
	mute_vol_start_row 	= subtitle_start_row + SUBTITLE_ROWS;
	time_start_row 		= mute_vol_start_row + MUTE_VOL_ROWS;

	terawins_set_osd_menu_info(TERAWINS_START_X, terawins_start_y, icon_start_row, total_rows, &terawins_menu_info);
	terawins_set_osd_menu(&terawins_menu_info);
}

/* --------------------------------------------------------------------- */
void terawins_show_osd_item(uint8 idx)
{

}

/* --------------------------------------------------------------------- */
void terawins_show_icon(uint32 _idx)
{
	int _icon0[30] = {-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, ' ', ' ',
					' ', -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1};

	//--Set start ram address in terawins sram.
	terawins_set_osd_ram_addr(CHAR_START_RAM_ADDR(font_info[font_index].font_nums) 
								+ (WORDS_PER_ROW * icon_start_row));

	if(_idx == 0x00){			// sandglass 2x2 (blue)
		_icon0[14] = 0x00;
		_icon0[15] = 0x01;
		terawins_set_osd_line_info(_icon0, 0, FG_COLOR12, BG_COLOR0, 0, 0, 0, 0, &terawins_line_info);
		terawins_set_osd_line(&terawins_line_info);	
		_icon0[14] = 0x02;
		_icon0[15] = 0x03;
		terawins_set_osd_line_info(_icon0, 0, FG_COLOR12, BG_COLOR0, 0, 0, 0, 0, &terawins_line_info);
		terawins_set_osd_line(&terawins_line_info);	
		_icon0[14] = ' ';
		_icon0[15] = ' ';
		terawins_set_osd_line_info(_icon0, 0, FG_COLOR12, BG_COLOR0, 0, 0, 0, 0, &terawins_line_info);
		terawins_set_osd_line(&terawins_line_info);	
	}
	else if(_idx == 0x01){		// invalid 3x3 (red)
		_icon0[13] = 0x09;
		_icon0[14] = 0x0a;
		_icon0[15] = 0x0b;
		terawins_set_osd_line_info(_icon0, 0, FG_COLOR10, BG_COLOR0, 0, 0, 0, 0, &terawins_line_info);		
		terawins_set_osd_line(&terawins_line_info);	
		_icon0[13] = 0x0c;
		_icon0[14] = 0x0d;
		_icon0[15] = 0x0e;
		terawins_set_osd_line_info(_icon0, 0, FG_COLOR10, BG_COLOR0, 0, 0, 0, 0, &terawins_line_info);	
		terawins_set_osd_line(&terawins_line_info);	
		_icon0[13] = 0x0f;
		_icon0[14] = 0x10;
		_icon0[15] = 0x11;
		terawins_set_osd_line_info(_icon0, 0, FG_COLOR10, BG_COLOR0, 0, 0, 0, 0, &terawins_line_info);			
		terawins_set_osd_line(&terawins_line_info);	
	}
	else{
		TW_MSG("\nInput icon num error !\n");
		return;
	}
}


/* --------------------------------------------------------------------- */
void terawins_clear_icon(void)
{
	uint8 i = 0, j = 0;
	
	//--Set start ram address in terawins sram.
	terawins_set_osd_ram_addr(CHAR_START_RAM_ADDR(font_info[font_index].font_nums) 
								+ (WORDS_PER_ROW * icon_start_row));

	terawins_set_osd_line_info(NULL, 0, FG_COLOR0, BG_COLOR0, 0, 0, (FONT_H_PER_ICON_ROW - 1), 0, &terawins_line_info);
	for(j = 0; j < ICON_ROWS; j ++){
		for(i = 0; i < (WORDS_PER_ROW - 1); i++){
			//-- low byte first.(char index.)
			terawins_set_osd_ram_data(0x00);
			//-- high byte.(char attribute.)
			terawins_set_osd_ram_data(0x00);
		}
		terawins_set_osd_ram_data(0x00);
		terawins_set_osd_ram_data(terawins_line_info.row_attr);
	}		
}

/* --------------------------------------------------------------------- */
void terawins_show_subtitle(char _s[][31])
{
	int _sub[30] = {-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
					-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1};;
	uint8 i = 0, j = 0;

	//--Set start ram address in terawins sram.
	terawins_set_osd_ram_addr(CHAR_START_RAM_ADDR(font_info[font_index].font_nums) 
								+ (WORDS_PER_ROW * subtitle_start_row));

	for(i = 0; i < SUBTITLE_ROWS; i++){
		//Reset _str to default value (-1)
		for( ; j > 0; j--){
			_sub[j] = -1;
		}
		j = 0;
		while( (_s[i][j] != '\0') && (j < 30) ){
			_sub[j] = _s[i][j];
			j++;
		}
		// Set line info.
		terawins_set_osd_line_info(_sub, 0, FG_COLOR3, BG_COLOR0, 0, 0, (FONT_H_PER_SUBTITLE_ROW - 1), 0, &terawins_line_info);
		terawins_set_osd_line(&terawins_line_info);	
	}
}

/* --------------------------------------------------------------------- */
void terawins_clear_subtitle(void)
{
	uint8 i = 0, j = 0;

	//--Set start ram address in terawins sram.
	terawins_set_osd_ram_addr(CHAR_START_RAM_ADDR(font_info[font_index].font_nums) 
								+ (WORDS_PER_ROW * subtitle_start_row));

	terawins_set_osd_line_info(NULL, 0, FG_COLOR0, BG_COLOR0, 0, 0, (FONT_H_PER_SUBTITLE_ROW - 1), 0, &terawins_line_info);
	for(j = 0; j < SUBTITLE_ROWS; j ++){
		for(i = 0; i < (WORDS_PER_ROW - 1); i++){
			//-- low byte first.(char index.)
			terawins_set_osd_ram_data(0x00);
			//-- high byte.(char attribute.)
			terawins_set_osd_ram_data(0x00);
		}
		terawins_set_osd_ram_data(0x00);
		terawins_set_osd_ram_data(terawins_line_info.row_attr);
	}
}

/* --------------------------------------------------------------------- */
#define FONT_VOL_BAR				0x12
#define FONT_VOL_SPACE_BAR			0x13
#define TOTAL_VOL_BAR				10
#define VOL_START					12
void terawins_show_mute_vol(uint32 _idx)
{
	uint8 i = 0;
	int _mute[30] = {-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 'M', 'U', 'T', 'E', 
					-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1};
	int _vol[30] = {-1, -1, -1, -1, -1, -1, -1, -1, 'V', 'O', 'L', ' ', -1, -1, -1, 
					-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1};
	
	terawins_set_osd_ram_addr(CHAR_START_RAM_ADDR(font_info[font_index].font_nums)
								+ (WORDS_PER_ROW * mute_vol_start_row));

	if(_idx == OSD_MUTE){
	//--Mute
		//TW_MSG("\nMute !!");
		terawins_set_osd_line_info(_mute, 0, FG_COLOR3, BG_COLOR7, 0, 0, (FONT_H_PER_MUTE_VOL_ROW - 1), 0, &terawins_line_info);
	}
	else{
	//--Vol
		//TW_MSG("\nVol !!");		
		for(i = 0; i < _idx; i ++){
			_vol[VOL_START + i] = FONT_VOL_BAR;
		}
		for( ; i < TOTAL_VOL_BAR; i ++){
			_vol[VOL_START + i] = FONT_VOL_SPACE_BAR;
		}
		terawins_set_osd_line_info(_vol, 0, FG_COLOR3, BG_COLOR7, 0, 0, (FONT_H_PER_MUTE_VOL_ROW - 1), 0, &terawins_line_info);
	}
	// Set osd line info.
	terawins_set_osd_line(&terawins_line_info);
}

/* --------------------------------------------------------------------- */
void terawins_clear_mute_vol(void)
{
	uint8 i = 0;
	
	//--Set start ram address in terawins sram.
	terawins_set_osd_ram_addr(CHAR_START_RAM_ADDR(font_info[font_index].font_nums) 
								+ (WORDS_PER_ROW * mute_vol_start_row));

	terawins_set_osd_line_info(NULL, 0, FG_COLOR0, BG_COLOR0, 0, 0, (FONT_H_PER_MUTE_VOL_ROW - 1), 0, &terawins_line_info);
	for(i = 0; i < (WORDS_PER_ROW - 1); i++){
		//-- low byte first.(char index.)
		terawins_set_osd_ram_data(0x00);
		//-- high byte.(char attribute.)
		terawins_set_osd_ram_data(0x00);
	}
	terawins_set_osd_ram_data(0x00);
	terawins_set_osd_ram_data(terawins_line_info.row_attr);
}

/* --------------------------------------------------------------------- */
void terawins_show_time(char *_t)
{
	int _time[30] = {-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
					-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1};;
	uint8 i = 0;
	int j = 0;

	//--Set start ram address in terawins sram.
	j = terawins_set_osd_ram_addr(CHAR_START_RAM_ADDR(font_info[font_index].font_nums) 
									+ (WORDS_PER_ROW * time_start_row));
	if(j != 0){
		return;
	}
	while( (_t[i] != '\0') && ((i + TIME_START_COL) < 30) ){
		_time[i + TIME_START_COL] = _t[i];
		i++;
	}
	// Set line info.
	terawins_set_osd_line_info(_time, 0, FG_COLOR3, BG_COLOR0, 0, 0, (FONT_H_PER_TIME_ROW - 1), 0, &terawins_line_info);
	terawins_set_osd_line(&terawins_line_info);	
}

/* --------------------------------------------------------------------- */
void terawins_clear_time(void)
{
	uint8 i = 0;
	
	//--Set start ram address in terawins sram.
	terawins_set_osd_ram_addr(CHAR_START_RAM_ADDR(font_info[font_index].font_nums) 
								+ (WORDS_PER_ROW * time_start_row));

	terawins_set_osd_line_info(NULL, 0, FG_COLOR0, BG_COLOR0, 0, 0, (FONT_H_PER_TIME_ROW - 1), 0, &terawins_line_info);
	for(i = 0; i < (WORDS_PER_ROW - 1); i++){
		//-- low byte first.(char index.)
		terawins_set_osd_ram_data(0x00);
		//-- high byte.(char attribute.)
		terawins_set_osd_ram_data(0x00);
	}
	terawins_set_osd_ram_data(0x00);
	terawins_set_osd_ram_data(terawins_line_info.row_attr);
}

/* --------------------------------------------------------------------- */
static void terawins_clear_gap(void)
{
	uint8 i = 0, j = 0;
	
	//--Set start ram address in terawins sram.
	terawins_set_osd_ram_addr(CHAR_START_RAM_ADDR(font_info[font_index].font_nums) 
								+ (WORDS_PER_ROW * gap_start_row));

	terawins_set_osd_line_info(NULL, 0, FG_COLOR0, BG_COLOR0, 0, 0, (FONT_H_PER_GAP_ROW - 1), 0, &terawins_line_info);
	for(j = 0; j < gap_rows; j ++){
		for(i = 0; i < (WORDS_PER_ROW - 1); i++){
			//-- low byte first.(char index.)
			terawins_set_osd_ram_data(0x00);
			//-- high byte.(char attribute.)
			terawins_set_osd_ram_data(0x00);
		}
		terawins_set_osd_ram_data(0x00);
		terawins_set_osd_ram_data(terawins_line_info.row_attr);
	}
}


/* --------------------------------------------------------------------- */
void terawins_clear_all_osd_item(void)
{
#if 1
	//-- Clear icon
	terawins_clear_icon();
	//-- Clear gap
	terawins_clear_gap();
	//-- Clear subtitle
	terawins_clear_subtitle();
	//-- Clear mute/vol
	terawins_clear_mute_vol();
	//-- Clear time
	terawins_clear_time();
#else
	uint8 i = 0, j = 0;
	//-- Clear ICON
	terawins_clear_icon();

	terawins_set_osd_line_info(NULL, 0, FG_COLOR0, BG_COLOR0, 0, 0, 1, 0, &terawins_line_info);
	//TW_MSG("\n---- Clear all osd item ----\n");
	for(j = 0; j < (total_rows - ICON_ROWS); j ++){
		for(i = 0; i < (WORDS_PER_ROW - 1); i++){
			//-- low byte first.(char index.)
			terawins_set_osd_ram_data(0x00);
			//-- high byte.(char attribute.)
			terawins_set_osd_ram_data(0x00);
		}
		terawins_set_osd_ram_data(0x00);
		terawins_set_osd_ram_data(terawins_line_info.row_attr);	
	}
#endif	
}


/* --------------------------------------------------------------------- */
void init_terawins_osd(void)
{
	/* Init osd. */ 
	terawins_set_osd_cmd(OSD_CTRL, 				0x00);	// turn off osd, set to char mode.
	terawins_set_osd_cmd(CHAR_DLY1, 			0x00);
	terawins_set_osd_cmd(CHAR_DLY2, 			0x00);	
	terawins_set_osd_cmd(CHAR_DLY3, 			0x00);
	terawins_set_osd_cmd(CHAR_FONT_SIZE, 		0x09);
	terawins_set_osd_cmd(CHAR_RAM_START_ADDR,	0x00);	
	terawins_set_osd_cmd(CHAR_BS_CTRL, 			0x00);
	terawins_set_osd_cmd(CHAR_BS_COLOR, 		0x00);
	terawins_set_osd_cmd(CHAR_HEIGHT_SCALING, 	0x20);	
	terawins_set_osd_cmd(BLINKING_CTRL, 		0x05);
	terawins_set_osd_cmd(BMAP_WIN_SIZE_UPPER, 	0x00);
	terawins_set_osd_cmd(BMAP_WIN_SIZE_WIDTH, 	0x80);	
	terawins_set_osd_cmd(BMAP_WIN_SIZE_HEIGHT, 	0x60);
	terawins_set_osd_cmd(BMAP_DOT_ENLARGE, 		0x11);
//	terawins_set_osd_cmd(OSD_COLOR_LUT_PORT, 	0x84);	
	terawins_set_osd_cmd(CHAR_RAM_ACCESS_CTRL, 	0x00);
	terawins_set_osd_cmd(W1_START_ROW_NUM, 		0x00);//00
	terawins_set_osd_cmd(W1_END_ROW_NUM, 		0x00);	
	terawins_set_osd_cmd(W1_START_COL_NUM, 		0x00);
	terawins_set_osd_cmd(W1_END_COL_NUM, 		0x00);
	terawins_set_osd_cmd(W1_SHADOW_SIZE, 		0x00);	
	terawins_set_osd_cmd(CHAR2BP_IDX_BASE, 		0xf0);
	terawins_set_osd_cmd(CHAR2BP_BASEADDR_LO, 	0x00);
	terawins_set_osd_cmd(CHAR2BP_BASEADDR_HI, 	0x08);	
	terawins_set_osd_cmd(ALPHA_BLENDING_CTRL, 	0x42);
	terawins_set_osd_cmd(REVISION_ID, 			0x12);
	terawins_set_osd_cmd(CHAR_RAM_STOP_ADDR, 	0x60);	

	//-- Clear osd ram.
	terawins_clear_osd_ram();

	//-- Load BMP LUT.
	terawins_load_osd_color(terawins_osd_color_table);

	//-- Load fonts.
	font_info[font_index].font_nums = terawins_load_osd_fonts(AllFonts);

	//-- Set line info start address and font size.
	terawins_set_osd_cmd(CHAR_FONT_SIZE, font_info[font_index].font_size);
	terawins_set_osd_cmd(CHAR_RAM_START_ADDR, CHAR_START_ADDR(font_info[font_index].font_nums));

	//-- Setup menu info.
	terawins_set_display_range();
	terawins_clear_all_osd_item();	

	//-- Enable char mode OSD function.
	terawins_enable_osd();
}

/* --------------------------------------------------------------------- */
void test_terawins(unsigned int _val)
{
	unsigned int i;
	if(_val == 0){
		TW_MSG("\npwm 0ff\n");
		terawins_pwm_on_off(0);
	}
	else if(_val == 1){
		TW_MSG("\npwm 0n\n");
		terawins_pwm_on_off(1);
	}
	else if(_val == 2){
		TW_MSG("\ntera 0ff\n");
		terawins_on_off(0);
	}
	else if(_val == 3){
		TW_MSG("\ntera 0n\n");
		terawins_on_off(1);
	}
	else if(_val == 4){
		TW_MSG("\ntest 0ff\n");
		terawins_on_off(0);
		terawins_pwm_on_off(0);		
	}
	else if(_val == 5){
		TW_MSG("\ntest 0n\n");
		terawins_on_off(1);
		terawins_pwm_on_off(1);		
	}
	else{
		TW_MSG("Retry !!");
	}


#if 0	//--Test the ICON
	if(_val == 0x00){
		terawins_show_icon(0);
	}
	else if(_val == 0x01){		// show icon
		terawins_show_icon(1);
	}
	else{		// clear icon
		terawins_clear_icon();
	}
#endif

#if 0	//-- Show terawins osd item.
	int i = 0;
	char *s1 = "This is a test function.";
	char *s2 = "*************************";
	char *s3 = "This is a test function.";	
	char *s4 = "23:23:23";	
	char _s[3][31];
	
	if(_val == 0x00){			// disable all
		terawins_clear_all_osd_item();
	}
	else if(_val == 0x01){		// show sandglass
		terawins_show_icon(0);
	}
	else if(_val == 0x02){		// clear icon
		terawins_clear_icon();
	}
	else if(_val == 0x03){		// show stop
		terawins_show_icon(1);
	}
	else if(_val == 0x04){		// clear icon
		terawins_clear_icon();
	}
	else if(_val == 0x05){		// show stop
		terawins_show_icon(0);
	}
	else if(_val == 0x06){		// clear icon
		terawins_clear_icon();
	}
	else if(_val == 0x07){		// show subtitle
		for(i = 0; i < 31; i++){
			_s[0][i] = s1[i];
		}
		for(i = 0; i < 31; i++){
			_s[1][i] = s2[i];
		}
		for(i = 0; i < 31; i++){
			_s[2][i] = s3[i];
		}
		terawins_show_subtitle(_s);
	}
	else if(_val == 0x08){		// clear subtitle
		terawins_clear_subtitle();
	}
	else if(_val == 0x09){		// show vol
		terawins_show_mute_vol(OSD_VOL3);
	}
	else if(_val == 0x0a){		// clear vol
		terawins_clear_mute_vol();
	}
	else if(_val == 0x0b){		// show mute
		terawins_show_mute_vol(OSD_MUTE);
	}
	else if(_val == 0x0c){		// clear mute
		terawins_clear_mute_vol();
	}
	else if(_val == 0x0d){		// show time
		terawins_show_time(s4);
	}
	else if(_val == 0x0e){		// clear time
		terawins_clear_time();
	}
	else if(_val == 0x0f){		// show all
		TW_MSG("\nicon_start_row = %d", icon_start_row);
		TW_MSG("\ngap_start_row = %d", gap_start_row);
		TW_MSG("\nsubtitle_start_row = %d", subtitle_start_row);
		TW_MSG("\nmute_vol_start_row = %d", mute_vol_start_row);
		TW_MSG("\ntime_start_row = %d", time_start_row);
		TW_MSG("\ntotal_rows = %d\n", total_rows);
		TW_MSG("\ngap_lines = %d", gap_lines);
		TW_MSG("\ngap_rows = %d", gap_rows);
		TW_MSG("\nLCD_START_Y = %d\n", LCD_START_Y);
		TW_MSG("\nterawins_start_y = %d\n", terawins_start_y);
	}
	else{						// disable all
		terawins_clear_all_osd_item();
	}
#endif


#if 0	//-- Test pwm on/off
	if(_val == 0){
		TW_MSG("0ff");
		terawins_pwm_on_off(_val);
	}
	else if(_val == 1){
		TW_MSG("0n");
		terawins_pwm_on_off(_val);
	}
	else{
		TW_MSG("Retry !!");
	}
#endif


#if 0	//-- Switching tables.
	if(_val == 0){
		terawins_set_resolution(TERAWINS_640x480);
	}
	else if(_val == 1){
		terawins_set_resolution(TERAWINS_720x480);
	}
	else if(_val == 2){
		terawins_set_resolution(TERAWINS_720x576);
	}
	else if(_val == 3){
		terawins_set_resolution(TERAWINS_800x600);
	}
	else{
		TW_MSG("Not Support !");
	}
#endif

#if 0	//-- Test terawins command
	unsigned int i;
	//-- Set cmd.
	i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, (uint8)((_val&0xff00)>>8), (uint8)(_val&0x00ff));	
#endif

#if 0	//-- Test the TeraWins osd function
	struct _terawins_menu_info sm_info;
	int s0[30] = {'S', 'E', 'T', 'U', 'P', -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
					-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1};
	int s1[30] = {'V', 'I', 'D', 'E', 'O', -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
					-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1};
	int s2[30] = {'A', 'U', 'D', 'I', 'O', -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
					-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1};
	int s3[30] = {'I', 'M', 'A', 'G', 'E', -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
					-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1};
	int s4[30] = {'S', 'Y', 'S', 'T', 'E', 'M', -1, -1, -1, -1, -1, -1, -1, -1, -1,
					-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1};	
	struct _terawins_line_info sl_info;	

	struct _terawins_window_info win_info;		
	
	terawins_enable_osd();
	
	if(_val == 0){
	//-- Mute
		terawins_show_mute();
	}
	else if(_val == 1){
	//-- VOL
		terawins_show_volume(8);	
	}
	else if(_val == 2){
	//-- Try the setup menuconfig.
		TW_MSG("\nSetup !!");	
		// Set osd menu info.
		terawins_set_osd_menu_info(0x50, 0x40, 5, &sm_info);
		terawins_set_osd_menu(&sm_info);
		// Set osd ram address.
		terawins_set_osd_ram_addr(CHAR_START_RAM_ADDR(font_info[font_index].font_nums));	
		// Set osd line info.
		terawins_set_osd_line_info(s0, 1, FG_COLOR2, BG_COLOR0, 1, 1, 1, 0, &sl_info);		
		terawins_set_osd_line(&sl_info);	
		terawins_set_osd_line_info(s1, 0, FG_COLOR3, BG_COLOR1, 1, 2, 1, 0, &sl_info);		
		terawins_set_osd_line(&sl_info);	
		terawins_set_osd_line_info(s2, 1, FG_COLOR4, BG_COLOR2, 1, 3, 1, 0, &sl_info);		
		terawins_set_osd_line(&sl_info);	
		terawins_set_osd_line_info(s3, 0, FG_COLOR5, BG_COLOR3, 1, 4, 1, 0, &sl_info);		
		terawins_set_osd_line(&sl_info);	
		terawins_set_osd_line_info(s4, 1, FG_COLOR6, BG_COLOR4, 1, 5, 1, 0, &sl_info);		
		terawins_set_osd_line(&sl_info);			
	}
	else if(_val == 3){
	//-- Test the osd window function.
		terawins_set_osd_window_info(1, 1, 0, 8, 1, 0, BG_COLOR5, 1, 0, BG_COLOR2, 3, 5, &win_info);
		terawins_set_osd_window(&win_info);
	}
	else if(_val == 4){
	//-- Test the osd window function.
		terawins_set_osd_window_info(2, 2, 0, 8, 1, 1, BG_COLOR5, 1, 1, BG_COLOR2, 3, 5, &win_info);
		terawins_set_osd_window(&win_info);
	}
	else if(_val == 5){
	//-- Enable osd window function.	
		terawins_enable_osd_win(&win_info);
	}
	else if(_val == 6){
	//-- Disable osd window function.
		terawins_disable_osd_win(&win_info);
	}
	else if(_val == 7){
	//-- osd blinking
		terawins_set_osd_blinking(1, 2);
	}
	else if(_val == 8){
	//-- osd alpha blending.
		terawins_set_osd_alpha_blending(0, 4);
	}

#endif

}

#endif	//-- #if CONFIG_TERAWINS_TCON

