/* 
 * Skymedi version information
 */

#include <linux/config.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/types.h>
#include <linux/proc_fs.h>
#include "skymedi.h"

static unsigned int skymedi_versions_end = 0;
static char skymedi_versions[VERSIONS_SIZE];

static ssize_t proc_versions_read(struct file *, char *, size_t count, loff_t *);
static ssize_t proc_versions_write(struct file *, const char *, size_t, loff_t *);
static long long proc_versions_lseek(struct file *, loff_t, int);

unsigned char skymedi_version[] = COMPANY_NAME " Kernel Build at " __TIME__ " " __DATE__;

struct file_operations proc_versions_operations = {
  llseek: proc_versions_lseek,
  read:   proc_versions_read,
  write:  proc_versions_write,
};

#if 1 //wschen 2007-10-17
static int proc_nls_read(char *page, char **start, off_t off, int count, int *eof, void *data);

static int proc_nls_read(char *page, char **start, off_t off, int count, int *eof, void *data)
{
  int len = 0;

  if (count <= 0) {
    return -EINVAL;
  }

  len += sprintf(page, "%s\n", CONFIG_NLS_DEFAULT);
  *eof = 1;

  return len;
}


#endif

static ssize_t proc_versions_read(struct file *file, char *buf,
                                size_t count, loff_t *ppos)
{
  if ((file == NULL) || (buf == NULL) || (ppos == NULL)) {
    return -EINVAL;
  }
  if (count == 0) {
    return 0;
  }
  if (skymedi_versions_end < (*ppos+count)) {
    count = skymedi_versions_end-*ppos;
  }

  memcpy(buf, skymedi_versions+(*ppos), count);
  file->f_pos += count;

  return count;
}


size_t sky_versions_write(const char *buf, size_t count, int cr)
{
  unsigned int newend;
  unsigned long flags;

  if (count == 0) {
    return 0;
  }

  local_irq_save(flags);

  if (skymedi_versions_end == VERSIONS_SIZE) {
    count = 0;
    goto sky_versions_write_out;
  }

  newend = skymedi_versions_end+count;
  if (cr) {
    ++newend;
  }
  if (VERSIONS_SIZE < newend) {
    count = VERSIONS_SIZE-skymedi_versions_end;
    if (cr) {
      --count;
    }
    if (count == 0) {
      goto sky_versions_write_out;
    }
  }

  memcpy(skymedi_versions+skymedi_versions_end, buf, count);
  skymedi_versions_end += count;
  if (cr) {
    skymedi_versions[skymedi_versions_end++] = '\n';
  }

sky_versions_write_out:
  local_irq_restore(flags);
  return count;
}

static ssize_t proc_versions_write(struct file * file, const char * buf,
                                 size_t count, loff_t *ppos)
{
  size_t tcount;

  if ((file == NULL) || (buf == NULL) || (ppos == NULL)) {
    return -EINVAL;
  }

  tcount = sky_versions_write(buf, count, 0);

  return tcount;
}


static long long proc_versions_lseek(struct file * file, loff_t offset, int orig)
{
  switch (orig) {
  case 0:
    file->f_pos = offset;
    return (file->f_pos);
  case 1:
    file->f_pos += offset;
    return (file->f_pos);
  case 2:
    file->f_pos = skymedi_versions_end;
    return (file->f_pos);
  default:
    return (-EINVAL);
  }
}


int __init skymedi_init(void)
{
  struct proc_dir_entry *entry;

  entry = create_proc_entry("versions", S_IRUGO|S_IWUGO, NULL);
  if (entry == NULL) {
    return 0;
  }
  entry->proc_fops = &proc_versions_operations;

  sky_versions_write(skymedi_version, strlen(skymedi_version), 1);

#if 1 //wschen 2007-10-17
  create_proc_read_entry("nls_default", 0, 0, proc_nls_read, NULL);
#endif

  return 0;

}

static void __exit skymedi_exit(void)
{
  return;
}

EXPORT_SYMBOL(sky_versions_write);

module_init(skymedi_init);
module_exit(skymedi_exit);

