
/*
 *  linux/include/asm-arc/arc-ac97.h
 *
 *  ARC AC97 Interface
 *
 *  ARC International, 2006.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#ifndef __ASMARC_AC97_H
#define __ASMARC_AC97_H

#ifdef __KERNEL__

/* This is what our controller looks like: */

typedef struct {
  volatile unsigned long ID;

  volatile unsigned long padding1[3];

  volatile unsigned long play_data1;
  volatile unsigned long play_data2;
  volatile unsigned long play_data3;

  volatile unsigned long padding2[5];

  volatile unsigned long play_trig1;
  volatile unsigned long play_trig2;
  volatile unsigned long play_trig3;

  volatile unsigned long padding3[5];

  volatile unsigned long cap_data1;
  volatile unsigned long cap_data2;
  volatile unsigned long cap_data3;

  volatile unsigned long padding4[5];

  volatile unsigned long cap_trig1;
  volatile unsigned long cap_trig2;
  volatile unsigned long cap_trig3;

  volatile unsigned long padding5[5];

  volatile unsigned long cmd;
  volatile unsigned long status;
  volatile unsigned long intr_id;
  volatile unsigned long intr_mask;
  volatile unsigned long intr_ctrl;
  volatile unsigned long point1;
  volatile unsigned long point2;
  volatile unsigned long point3;
} ARC_AC97_if;

/* Aurora-defined base address */
#define AC97_CONTROLLER_BASE  0xc0fcb000
#define AC97_INTERRUPT 12

#define MAX_FRAG 128
#define FRAGSIZE 2048
#define FIFO_LEN 4096
#define TRANSFER_SIZE 1024
#define TRIGGER_LEVEL  3072
#define TMPBUF_SIZE (FRAGSIZE+10)
#define DEFAULT_LATENCY 1
#define START_THRESHOLD 0.5

/* Some register decoding */
#define ID_CONFIG(regval)   ((regval & 0xff000000) > 24)
#define ID_REVISION(regval) ((regval & 0xff0000) > 16)
#define ID_MID(regval)      ((regval & 0x3f00) > 8)
#define ID_ID(regval)       (regval & 0x3f)

/* Command types */
#define CMD_READ 0x800000
#define CMD_WRITE 0x0

/* Interrupt IDs */
#define CAP1_INTR   0x04
#define CAP2_INTR   0x02
#define CAP3_INTR   0x01
#define PLAY1_INTR  0x20
#define PLAY2_INTR  0x10
#define PLAY3_INTR  0x08
#define STATUS_INTR 0x40

#define CAPTURE_INTRS 0x07
#define PLAY_INTRS    0x38

#define INTR_ENABLE 1
#define INTR_CLEAR  2
#define INTR_STATUS 3


/* Status bits */
#define AC97C_READY 0x8000


/* Working around some of the platform-specific stupidity in soundcore.c ... */

#ifdef valid_dma
#undef valid_dma
#endif
#define valid_dma(n) 1

#define MAX_DMA_CHANNELS 1

#endif
#endif
