/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) Oct 01, 2004
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 *  linux/include/asm-arc/cache.h
 *
 *  Copyright (C) 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Authours : Rahul Trivedi
 */

#ifndef __ARC_ASM_CACHE_H
#define __ARC_ASM_CACHE_H

/* cache register masks,shifts */
#define CACHE_CONFIG_MASK 0x00000f00
#define CACHE_CONFIG_SHIFT 8
#define CACHE_SIZE_MASK 0x0000f000
#define CACHE_SIZE_SHIFT 12
#define CACHE_LINE_LENGTH_MASK 0x000f0000
#define CACHE_LINE_LENGTH_SHIFT 16

/* used to give SHMLBA a value to avoid Cache Aliasing */
extern unsigned int ARC_shmlba ;		
extern void a7_cache_init(void);

#if CONFIG_ARC700_CACHE 	
struct mm_struct;
struct vm_area_struct;
struct page;

extern void flush_cache_all(void);
extern void __flush_cache_all(void);
extern void flush_cache_mm(struct mm_struct *mm);
extern void flush_cache_range(struct mm_struct *mm, unsigned long start,unsigned long end);
extern void flush_cache_page(struct vm_area_struct *vma,unsigned long page);

#else
#define flush_cache_all()                       do { } while (0)
#define flush_cache_mm(mm)                      do { } while (0)
#define flush_cache_range(mm,start,end)         do { } while (0)
#define flush_cache_page(vma,vmaddr)            do { } while (0)


#endif /* CONFIG_ARC_CACHE */

#ifdef CONFIG_ARC700_USE_ICACHE
extern void flush_icache_all(void);
extern void flush_icache_range(unsigned long start, unsigned long end);
extern void flush_icache_page(struct vm_area_struct *vma,struct page *page);

#define flush_icache_user_range(vma, page, addr, len) \
                                        flush_icache_page((vma), (page))

#else
#define flush_icache_all()		        do { } while (0)
#define flush_icache_range(start,end)           do { } while (0)
#define flush_icache_page(vma,page)             do { } while (0)
#define flush_icache_user_range(vma,pg,adr,len)         do { } while (0)

#endif /*CONFIG_ARC700_USE_ICACHE*/

#ifdef CONFIG_ARC700_USE_DCACHE
extern void flush_page_to_ram(struct page *page);
extern void flush_dcache_page(struct page *page);
extern void flush_dcache_page_virt(unsigned long *page);
extern void flush_dcache_range(unsigned long start,unsigned long end);

extern void dma_cache_wback_inv(unsigned long start, unsigned long size);
extern void dma_cache_wback(unsigned long start, unsigned long size);
extern void dma_cache_inv(unsigned long start, unsigned long size);
extern void flush_dcache_all(void);
extern void inv_dcache_all(void);
extern void inv_dcache_range(unsigned long start, unsigned long end);
void flush_and_inv_dcache_all(void);
#else

#define flush_page_to_ram(page)                 do { } while (0)
#define flush_dcache_range(start,end)           do { } while (0)
#define flush_dcache_page(page)                 do { } while (0)
#define flush_dcache_page_virt(page)            do { } while (0)
#define dma_cache_wback_inv(start,size)         do { } while (0)
#define dma_cache_wback(start,size)             do { } while (0)
#define dma_cache_inv(start,size)               do { } while (0)
#define flush_dcache_all(start,size)            do { } while (0)
#define inv_dcache_all()			do { } while (0)
#define inv_dcache_range(start,size)		do { } while (0)
#define flush_and_inv_dcache_all()		do { } while (0)
#endif /*CONFIG_ARC700_USE_DCACHE*/


#define L1_CACHE_BYTES 		32
/* Uncached access macros */
#define arc_read_uncached_32(ptr)					\
({ 									\
 	unsigned int __ret;						\
 	__asm__ __volatile__ ("ld.di %0, [%1]":"=r"(__ret):"r"(ptr));	\
	__ret;								\
 })
 	
#define arc_write_uncached_32(ptr, data)				\
({ 									\
 	__asm__ __volatile__ ("st.di %0, [%1]"::"r"(data), "r"(ptr));	\
 })

#endif /* _ASM_CACHE_H */
