/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) Oct 01, 2004
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 *  linux/include/asm-arc/serial.h
 *
 *  Copyright (C) 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * Authors : Amit Bhor
 */

#ifndef	_ASM_ARC_SERIAL_H
#define	_ASM_ARC_SERIAL_H

#include <linux/config.h>

#ifdef __KERNEL__

/*
 * This is our internal structure for each serial port's state.
 * 
 * Many fields are paralleled by the structure used by the serial_struct
 * structure.
 *
 * For definitions of the flags field, see tty.h
 */

struct aa3_serial {
	char is_cons;       /* Is this our console. */

	/* We need to know the current clock divisor
	 * to read the bps rate the chip has currently
	 * loaded.
	 */
	int baud;
	int			magic;
	int			baud_base;
	int			port;
	int			irq;
	int			flags; 		/* defined in tty.h */
	int			type; 		/* UART type */
	struct tty_struct 	*tty;
	int			read_status_mask;
	int			ignore_status_mask;
	int			timeout;
	int			xmit_fifo_size;
	int			custom_divisor;
	int			x_char;	/* xon/xoff character */
	int			close_delay;
	unsigned short		closing_wait;
	unsigned short		closing_wait2;
	unsigned long		event;
	unsigned long		last_active;
	int			line;
	int			count;	    /* # of fd on device */
	int			blocked_open; /* # of blocked opens */
	long			session; /* Session of opening process */
	long			pgrp; /* pgrp of opening process */
	unsigned char 		*xmit_buf;
	int			xmit_head;
	int			xmit_tail;
	int			xmit_cnt;
	struct tq_struct	tqueue;
	struct tq_struct	tqueue_hangup;
	struct termios		normal_termios;
	struct termios		callout_termios;
	wait_queue_head_t	open_wait;
	wait_queue_head_t	close_wait;
};

typedef volatile struct {
  volatile int id0;
  volatile int id1;
  volatile int id2;
  volatile int id3;
  volatile int data;
  volatile int status;
  volatile int baudl;
  volatile int baudh;
} aa3_uart __attribute__((packed));

#define SERIAL_MAGIC 0x5301

/*
 * Events are used to schedule things to happen at timer-interrupt
 * time, instead of at rs interrupt time.
 */
#define ARCSERIAL_EVENT_WRITE_WAKEUP	0

/* number of characters left in xmit buffer before we ask for more */
#define WAKEUP_CHARS 256

/* 
 * Define the number of ports supported and their irqs.
 */
#define NR_PORTS CONFIG_ARC700_NUM_SERIAL_PORTS
#define UART_IRQ_DEFNS {VUART_INT,VUART1_INT}

#define	CONSOLE_BAUD_RATE CONFIG_ARC700_SERIAL_BAUD
#define BASE_BAUD CONSOLE_BAUD_RATE

#define UART_BASEADDR { 0xC0FC1000, 0xC0FC1100, }

#define UART_RXENB	0x04
#define UART_RXEMPTY	0x20
#define UART_FRAME_ERR	0x01
#define UART_OVERFLOW_ERR  0x02

#define UART_TXENB      0x40
#define UART_TXEMPTY   0x80

/* UART control registers */
#define ARC_UART_CREG_DATATX	UART_BASEADDR + 0x4
#define ARC_UART_CREG_DATARX	UART_BASEADDR + 0x4
#define ARC_UART_CREG_STATUS	UART_BASEADDR + 0x5
#endif /* __KERNEL__ */

/* ioctls */
#define SET_BAUD 1
#define GET_BAUD 2
//-- Flowbee 2007-12-11
#define UART_RX_ON_OFF	3


#endif	/* _ASM_ARC_SERIAL_H */
