#ifndef SKY_I2C_H_
#define SKY_I2C_H_

#include <asm/sky_gpio.h>

#define SKY_I2C_MAJOR		252
/* for ioctl used */
#define I2C_DELAY1_R			0x0800
#define I2C_DELAY1_W			0x0801
#define I2C_DELAY2_R			0x0810
#define I2C_DELAY2_W			0x0811

#define I2C_SCL_MASK			(0x00000001<<CONFIG_SKY_I2C_SCL)
#define I2C_SDA_MASK			(0x00000001<<CONFIG_SKY_I2C_SDA)

#define SCL_INPUT				GPIO_W32(GPIO_DIR, GPIO_R32(GPIO_DIR) & ~I2C_SCL_MASK)
#define SCL_OUTPUT			GPIO_W32(GPIO_DIR, GPIO_R32(GPIO_DIR) | I2C_SCL_MASK)
#define SCL_HI				GPIO_W32(GPIO_DATA, GPIO_R32(GPIO_DATA) | I2C_SCL_MASK)
#define SCL_LO				GPIO_W32(GPIO_DATA, GPIO_R32(GPIO_DATA) & ~I2C_SCL_MASK)

#define SCL					((GPIO_R32(GPIO_DATA) & I2C_SCL_MASK)>>CONFIG_SKY_I2C_SCL)
#define SDA_INPUT			GPIO_W32(GPIO_DIR, GPIO_R32(GPIO_DIR) & ~I2C_SDA_MASK)
#define SDA_OUTPUT			GPIO_W32(GPIO_DIR, GPIO_R32(GPIO_DIR) | I2C_SDA_MASK)
#define SDA_HI				GPIO_W32(GPIO_DATA, GPIO_R32(GPIO_DATA) | I2C_SDA_MASK)
#define SDA_LO				GPIO_W32(GPIO_DATA, GPIO_R32(GPIO_DATA) & ~I2C_SDA_MASK)
#define SDA					((GPIO_R32(GPIO_DATA) & I2C_SDA_MASK)>>CONFIG_SKY_I2C_SDA)

#define DELAY_T1 				200
#define DELAY_T2 				150
#define DELAY(cnt)	\
({	\
	do { int _x; for(_x=0;_x< cnt;_x++){__builtin_arc_nop();};} while(0);	\
})

#define SKY_I2C_BUF_SIZE		1024
typedef struct SKY_I2C_ {	
	u8 addr;
	u32 data_len;	
	u8 *data;
	u32 delay_t1;
	u32 delay_t2;
	u8 busy;
	spinlock_t lock;
}SKY_I2C_DEV, *SKY_I2C_DEV_PTR;

extern SKY_I2C_DEV sky_i2c_dev;

extern void i2c_set_delay(u32 delay1, u32 delay2);
extern int i2c_read(int address, unsigned char* buf, int count, int msb);
extern int i2c_write(int address, unsigned char* buf, int count);
extern void i2c_reset(void);

#endif
