/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) Oct 01, 2004
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 *  linux/include/asm-arc/system.h
 *
 *  Copyright (C) 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Authors : Amit Bhor
 */

#ifndef __ASM_ARC_SYSTEM_H
#define __ASM_ARC_SYSTEM_H
#include <linux/config.h>
#include <asm/arcregs.h>

#ifdef __ASSEMBLY__
#ifdef __KERNEL__
/*
 * Enable level 1 irqs, the parameter reg is the temp register to use
 */

.macro	sti_level_1	reg
	lr	\reg, [status32]
	or	\reg, \reg, STATUS_E1_MASK
	flag 	\reg
.endm	

/*
 * Disable level 1 irqs, parameter reg is the temp register to use
 */

.macro	cli_level_1	reg
	lr	\reg, [status32]
	and	\reg,\reg, ~(STATUS_E1_MASK)
	flag	\reg
.endm	

/*
 * Disable level 1 and 2 irqs, parameter reg is the temp register to use
 */
.macro	cli_level_12	reg
	lr	\reg, [status32]
	and	\reg,\reg, ~(STATUS_E1_MASK | STATUS_E2_MASK)
	flag	\reg
.endm	

/*
 * Enable level 2 irqs, the parameter reg is the temp register to use
 */

.macro	sti_level_2	reg
	lr	\reg, [status32]
	or	\reg, \reg, STATUS_E2_MASK
	flag 	\reg
.endm	

/*
 * Disable level 2 irqs, parameter reg is the temp register to use
 */

.macro	cli_level_2	reg
	lr	\reg, [status32]
	and	\reg,\reg, ~(STATUS_E2_MASK)
	flag	\reg
.endm	

#endif /*__KERNEL__*/

#else /* __ASSEMBLY__ */

#ifdef __KERNEL__
#define prepare_to_switch()     do { } while(0)

/* switch_to macro based on the ARM implementaion */
extern struct task_struct *__switch_to(struct task_struct *prev, struct task_struct *next);
/* mb() to tell GCC not to keep value of "current" in register */

/* FIXME :: Check the EX intruction */
extern inline unsigned long __xchg (unsigned long with,
                                    __volatile__ void *ptr, int size)
{
        
	__asm__ __volatile__ (" ex	%0, [%1]"
			      : "=r" (with),"=r"(ptr)
			      : "0" (with), "1" (ptr)	
			      : "memory" );

        return (with);
}

#define xchg(ptr, with) \
  ((__typeof__ (*(ptr)))__xchg ((unsigned long)(with), (ptr), sizeof (*(ptr))))

#endif /*__KERNEL__*/

#define switch_to(prev, next, last)	{		\
	do {						\
		last = __switch_to( prev, next);	\
		mb();					\
							\
	}while (0);					\
}	
	
/*
 * Save the current interrupt enable state & disable IRQs
 * See status register.
 */

#define __save_flags_cli(x)			\
	__asm__ __volatile__ (			\
	"lr	r20, [status32] \n\t"		\
	"mov	%0, r20 \n\t"			\
	"and	r20, r20, %1 \n\t"		\
	"flag 	r20	\n\t"			\
	:"=r" (x)				\
	:"n" (~(STATUS_E1_MASK | STATUS_E2_MASK))		\
	:"r20", "memory");	

	
/*
 * Enable IRQs
 */
#define __sti()					\
	__asm__ __volatile__ (			\
	"lr	r20, [status32] \n\t"		\
	"or	r20, r20, %0 \n\t"		\
	"flag 	r20	\n\t"			\
	:					\
	:"n" (STATUS_E1_MASK | STATUS_E2_MASK)		\
	:"r20");	
	
/*
 * Disable IRQs
 */
#define __cli()					\
	__asm__ __volatile__ (			\
	"lr	r20, [status32] \n\t"		\
	"and	r20, r20, %0 \n\t"		\
	"flag r20	\n\t"			\
	:					\
	:"n" (~(STATUS_E1_MASK | STATUS_E2_MASK))		\
	:"r20");	

/*
 * restore saved IRQ & FIQ state
 */
#define __restore_flags(x)			\
	__asm__ __volatile__ (			\
	"flag	%0"				\
	:					\
	:"r" (x)				\
	);

/*
 * save current IRQ & FIQ state
 */
#define __save_flags(x)				\
	__asm__ __volatile__ (			\
	"lr	r20, [status32] \n\t"		\
	"mov	%0, r20 \n\t"			\
	:"=r" (x)				\
	:					\
	:"r20", "memory");	

/* 
 * mask/unmask an interrupt
 */

#define __mask_intr(x)                         \
        __asm__ __volatile__ (                 \
        "lr     r19, [status32] \n\t"          \
	"lr	r20, [status32] \n\t"	       \
	"and	r20, r20, %1 \n\t"	       \
	"flag   r20	\n\t"		       \
	"lr	r20, [auxienable] \n\t"        \
        "and    r20, r20, %0 \n\t"             \
        "sr     r20,[auxienable] \n\t"         \
	"flag   r19	\n\t"		       \
        :                                      \
        :"r" (x), "n" (~(STATUS_E1_MASK | STATUS_E2_MASK))   \
        :"r19", "r20", "memory");

#define __unmask_intr(x)                       \
        __asm__ __volatile__ (                 \
        "lr     r19, [status32] \n\t"          \
	"lr	r20, [status32] \n\t"	       \
	"and	r20, r20, %1 \n\t"	       \
	"flag   r20	\n\t"		       \
	"lr	r20, [auxienable] \n\t"        \
        "or     r20, r20, %0 \n\t"             \
        "sr     r20, [auxienable] \n\t"        \
	"flag   r19	\n\t"		       \
        :                                      \
        :"r" (x),  "n" (STATUS_E1_MASK | STATUS_E2_MASK) \
        :"r19", "r20", "memory");

#define mb() __asm__ __volatile__ ("" : : : "memory");
#define rmb() mb()
#define wmb() mb()
#define set_mb(var, value)  do { var = value; mb(); } while (0)
#define set_wmb(var, value) do { var = value; wmb(); } while (0)

/* For spinlocks etc */
#define local_irq_save(x)	__save_flags_cli(x)
#define local_irq_restore(x)	__restore_flags(x)
#define local_irq_set(x)	__save_flags(x); sti()
#define local_irq_disable()	__cli()
#define local_irq_enable()	__sti()

#ifdef CONFIG_SMP
#error SMP not supported

#define smp_mb()		mb()
#define smp_rmb()		rmb()
#define smp_wmb()		wmb()


#else

#define smp_mb()		barrier()
#define smp_rmb()		barrier()
#define smp_wmb()		barrier()

#define cli()			__cli()
#define sti()			__sti()
#define save_flags(x)		__save_flags(x)
#define restore_flags(x)	__restore_flags(x)
#define save_flags_cli(x)	__save_flags_cli(x)
#define save_and_cli(x)		__save_flags_cli(x)

#define mask_interrupt(intbit)   __mask_intr(intbit)
#define unmask_interrupt(intbit) __unmask_intr(intbit)

#endif /* CONFIG_SMP */





#endif /* __ASSEMBLY__ */

#endif /* ASM_ARC_SYSTEM_H */
