
/*
 * board initialization should put one of these into dev->platform_data
 * and place the ohs900 onto platform_bus named "ohs900-hcd".
 */

#define BUS_ID_SIZE	20

struct device {
//	struct klist		klist_children;
//	struct klist_node	knode_parent;		/* node in sibling list */
//	struct klist_node	knode_driver;
//	struct klist_node	knode_bus;
	struct device 	* parent;

//	struct kobject kobj;
	char	bus_id[BUS_ID_SIZE];	/* position on parent bus */
//	struct device_attribute uevent_attr;

	struct semaphore	sem;	/* semaphore to synchronize calls to its driver.	 */

//	struct bus_type	* bus;		/* type of bus device is on */
	struct device_driver *driver;	/* which driver has allocated this device */
	void		*driver_data;	/* data private to the driver */
	void		*platform_data;	/* Platform specific data, device  core doesn't touch it */
	void		*firmware_data; /* Firmware specific data (e.g. ACPI, BIOS data),reserved for device core*/
//	struct dev_pm_info	power;

	u64		*dma_mask;	/* dma mask (if dma'able device) */
	u64		coherent_dma_mask;/* Like dma_mask, but for
					     alloc_coherent mappings as
					     not all hardware supports
					     64 bit addresses for consistent
					     allocations such descriptors. */

//	struct list_head	dma_pools;	/* dma pools (if dma'ble) */

//	struct dma_coherent_mem	*dma_mem; /* internal for coherent mem    override */

	void	(*release)(struct device * dev);
};

struct device_driver {
	const char		* name;
	struct bus_type		* bus;

//	struct completion	unloaded;
//	struct kobject		kobj;
//	struct klist		klist_devices;
//	struct klist_node	knode_bus;

//	struct module		* owner;

	int	(*probe)	(struct device * dev);
	int	(*remove)	(struct device * dev);
	void	(*shutdown)	(struct device * dev);
	int	(*suspend)	(struct device * dev, u32 state, u32 phase);
	int	(*resume)	(struct device * dev);
};

struct ohs900_platform_data {
	unsigned	can_wakeup:1;

	/* given port_power, msec/2 after power on till power good */
	u8		potpg;

	/* mA/2 power supplied on this port (max = default = 250) */
	u8		power;

	/* ohs900 relies on an external source of VBUS current */
	void 		(*port_power)(struct device *dev, int is_on);

	/* pulse ohs900 nRST (probably with a GPIO) */
	void 		(*reset)(struct device *dev);

	// some boards need something like these:
 	// int 		(*check_overcurrent)(struct device *dev);
 	// void 	(*clock_enable)(struct device *dev, int is_on);
};

struct platform_device {
	const char	* name;
	u32		id;
	struct device	dev;
	u32		num_resources;
	struct resource	* resource;
};

#define kcalloc(n, size, flags)	kmalloc(n*size, flags)

static inline void *
dev_get_drvdata (struct device *dev)
{
	return dev->driver_data;
}

static inline void
dev_set_drvdata (struct device *dev, void *data)
{
	dev->driver_data = data;
}
